// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include<iostream>
#include "param.h"
#include<vector>
#include "qdir.h"
#include "qstring.h"
#include "qfileinfo.h"
#include "Dal.h"
#include"errstr.h"
#include <cmath>
#include <cstring>
#include <vector>
#include "loadsrclistfiles.h"

/***************************************************************
*
*
*
* 
****************************************************************/
void loadsrclistfiles( vector<string> &srcList )
{

    vector<string> directoryList = stringListParameter("directorylist");
   
    if(directoryList.size() == 0)
    {
      directoryList.push_back("./");

    }

 

    for(unsigned i = 0; i < directoryList.size(); i++)
    {

        string directory = directoryList[i];
      
    
 
        QDir thisDir( directory.c_str() );
        if ( !thisDir.isReadable() ) 
        {
            errstr << "Could not open directory " << directory << Sas::warning("BadDirectory");
            continue;

        }
        errstr << "Searching for OM SWS files in directory " << directory  << Sas::message(Msg::App, Msg::Verbose);
       
      
        int k = directory.length() - 1;
        if ( !( (directory.at(k) == '/') || (directory.at(k) == '\\')) )
        {
            if ( directory.find('/') !=string::npos)
                directory.append("/");
            else
                directory.append("\\");
        }
     
  


        const QFileInfoList * files = thisDir.entryInfoList();
        if ( files ) 
        {
            QFileInfoListIterator it( *files );
            QFileInfo * fi;
            while( (fi = it.current()) != 0 ) 
            {
                ++it;
                QString file = directory.c_str() + fi->fileName();
               
	     
                /************************************************************
	        *
                *
                *
                *
                ************************************************************/
                if ( file.contains("P0")      && 
                     file.contains("SWS")    &&
                     ( file.contains("FIT") || file.contains("FTZ")) )
             
                {
                
	                
	            if( QFile::exists(file) )   
		    {
		        srcList.push_back( (const char*)file );
                        errstr << "adding file " << file << " to the list to be processed" << Sas::message(Msg::App, Msg::Verbose);
 

                    } 

              

                }
            }
        }
       
    }


    if(srcList.size() > 0)
    {

        errstr << "Final number of files to be processed=" << srcList.size() << Sas::message(Msg::App, Msg::Verbose);
 

    }
    else
    {
        errstr << "No product OM source-list files have been found" << Sas::fatal("NoFilesToProcess");

    }
  

}


/***************************************************************
*
* If only processing one observation and no observation file given
* on the command line, then this routine will attempt tp find it
*
* 
****************************************************************/
void loadObservationSrcListFile( )
{

    string obsFile = stringParameter("obsset");
    if( obsFile.length() > 2)
    {
      return;

    }

    vector<string> directoryList = stringListParameter("directorylist");
   
    if(directoryList.size() > 1)
    {
        return;
    }
    if( directoryList.size() == 0)
    {
        directoryList.push_back("./");

    }




    string directory = directoryList[0];
      
   

 
    int k = directory.length() - 1;
    if ( !( (directory.at(k) == '/') || (directory.at(k) == '\\')) )
    {
        if ( directory.find('/') !=string::npos)
            directory.append("/");
        else
            directory.append("\\");
    }
     
  


     
    
    QDir thisDir( directory.c_str() );
    if ( !thisDir.isReadable() ) 
    {
        errstr << "Could not open directory " << directory << Sas::warning("BadDirectory");
        return;

    }

    errstr << "Searching for observation source-list file in directory " << directory << Sas::message(Msg::App, Msg::Verbose);
     
    const QFileInfoList * files = thisDir.entryInfoList();
    if ( files ) 
    {
        QFileInfoListIterator it( *files );
        QFileInfo * fi;
        while( (fi = it.current()) != 0 ) 
        {
            ++it;
            QString file = directory.c_str() + fi->fileName();
          
	     
            /************************************************************
	    *
            *
            *
            *
            ************************************************************/
            if ( file.contains("P0")      && 
                 ( file.contains("OBS")  || file.contains("COMB") )  &&
                     ( file.contains("FIT") || file.contains("FTZ")) )
             
            {
                
	                
	        if( QFile::exists(file) )   
		{

		    obsFile = (const char *)file; 		        
                    if ( isObservationSrcListFile( (const char *)file ) )
		    {
                        setParameter("obsset", obsFile);
                        errstr << "Found observation source-list file " << file << Sas::message(Msg::App, Msg::Verbose);
                        return; 
                    }
                } 

              
            }
        }
       
    }

    errstr << "Unable to find observation source-list file" << Sas::message(Msg::App, Msg::Verbose);
   

}
/**************************************************************
*
*
*
***************************************************************/
bool isObservationSrcListFile( string file )
{

    DataSet * set = dataSetServer->open( file, DataSet::Read);
    bool okay = false;
    
    if( set )
    {

    
        if ( set->hasAttribute( "FILTER") && set->hasBlock("SRCLIST") )
        {

            string filter = set->attribute( "FILTER")->asString();
                 

            if( filter == "n/a" || filter == "N/A" )
            {
	        okay = true;    

	    } 
     
        }
        dataSetServer->close( set); // Finished with the file
       

    }
    return okay;
} 
