// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*****************************************************************************
*
* omvariabiliy
* Written by Christopher Brindle, MSSL, November 2006 
*
*
******************************************************************************/
#include <qstring.h>
#include <qdir.h>
#include <qdatetime.h>
#include <cstring>
#include <vector>
#include <map>
#include "omvariability.h"
#include "OmSourceParameters.h"
#include "astrometrydefs.h"

//gcc4.8.2 portability changes
#include <unistd.h>

#include "sourcelist.h"
#include "utility.h"
#include "statistics.h"
//#include "omfilter.h"
#include"Dal.h"

#include<cstdio>
#include<fstream>

#include "f90cxx.h"
#include "graph.h"
#include "AngleFormatter.h"

#include <cstdlib>
#include <ctime>

#include "loadsrclistfiles.h"

#include"omvariability_defs.h"


void stetsonVariabilityData( map<int,  map<string, VariabilityData, OmFilter> > &stetsonDataMap);

bool computeStetsonVariabilityData( map<string, VariabilityData, OmFilter>  &theData);

void checkInputFiles(vector<string> &fileList, string &fileType, int &nobs);

bool loadObservationSourceListFileData( string file,   vector < OmSourceParameter > &srcList);


void averageObservationMagnitudes( map<string, map<string, vector<SWSData>, OmFilter > > & obsMapFilterMapData,
                                   map<string, SWSData, OmFilter > &averagedFilterDataMap);


void averageData( vector<SWSData> &data,  SWSData &averagedData);

void executeOmsrclistcomb( vector<string> &fileList );


void averageMagnitudes(vector<float> &mag, vector<float> &magErr, 
		       float &meanMag, float &meanMagErr,
                       vector<float> &rate, vector<float> &RateErr, 
		       float &meanRate, float &meanRateErr);

void checkObsFile( string obsId, vector<string> &fileList );

void openPgplot();


void averageTimes( vector<SWSData> &data,  SWSData &averagedData);





void getNiceMagRange(const float median, const float yMin1, const float yMax1, float & yMin2, float &yMax2 );


/**********************************************************
*
*
*
***********************************************************/
void drawTheGraph( string obsId, int srcId, int nGraphs, 
                               vector< vector<float> > &xData,
                               vector< vector<float> > &yData, 
                               vector< vector<float> > &yDataErr, 
                               vector<double> &raList, vector<double> &decList,                
                               vector<string> &comment, 
		               map<string, VariabilityData, OmFilter>  &varData,
		               map<string, VariabilityData, OmFilter>::iterator &iter1,  
		               float minTime, float maxTime);



void  checkData( map<string, map<string, vector<SWSData>, OmFilter > > dataMap);

void checkSource( int number, vector<double> &ra, vector<double> &dec );

void checkSource1( int number, vector<double> &ra, vector<double> &dec );

void checkSource1( int number, vector<float> &ra, vector<float> &dec );


/**********************************************************************
*
*
* 
*
*
*
*
**********************************************************************/
float minStd1 = 2.0;
int maxNumGraphs = 1;




static int numBugs = 0;
static double maxDistance = 0.0; 
bool pgplotInitialised = false;



/**********************************************************************
*
* Entry point to the main program
*
***********************************************************************/
void omvariability()
{

    
    /***************************************************************************
    *
    * Do some intitialization
    *
    ***************************************************************************/
    cout<<"Started"<<endl;

  
  
    /*******************************************************************************
    *
    *  Get the list of SWS files
    *
    *******************************************************************************/
    vector<string> swsFileList = stringListParameter( "srclistsets" );




    if( swsFileList.size() == 0 )
    {
        loadsrclistfiles( swsFileList );
    
    }


    cout<<"Loaded lists"<<endl;




    unsigned minNumCounterParts = intParameter("minnumcounterparts");
    if( swsFileList.size() < minNumCounterParts )
    {
        errstr << "Too few input source-list files for the minimum number of counterparts" 
	       << Sas::warning("cantcontinue");
               
	     
        exit(0);

    }

  



    /************************************************
    * Try to load the observation source-list file
    *
    ***********************************************/
    loadObservationSrcListFile();

    cout<<"Loaded the source file list"<<endl;
    /*********************************************************
    * FOR TESTING PURPOSES
    *
    **********************************************************/
    //   swsFileList.resize(50);
    //  for(unsigned i = 1; i < swsFileList.size() ; i++)
    // {
    //    swsFileList[i] = swsFileList[0];
    // }

    /****************************************************************************
    *
    * Check to make sure that the files are either all SWS source-list files
    * or all observation source-list files
    *
    *****************************************************************************/
    string obsId = "";
    int numObservations = 0;

    checkInputFiles( swsFileList, obsId, numObservations );

    if(numObservations == 0)
    {

        Sas::fatal("toofewfiles");
     

    }

    cout<<"Blah blah"<<endl;
    //   numObservations = 2;
    //  cout << "Line 213 numobservations=" << numObservations << endl;   
    if(numObservations > 1)
    {

    cout<<"Executig"<<endl;
        /**********************************************
	*
        *
        *
        *
        ***********************************************/
        executeOmsrclistcomb( swsFileList );

    }
    else
    {
    cout<<"CheckObs"<<endl;
      checkObsFile( obsId, swsFileList );


    }
   
    cout<<"GOt to this pnt"<<endl;

    /*****************************************************************************
    *
    * Need to separate the lists by observation and filter
    *
    *****************************************************************************/
    map<string, map<string, vector<SWSData>, OmFilter > > obsMapFilterMapData;


 
    for(unsigned i = 0; i < swsFileList.size(); i++)
    {     
        SWSData data;
	        cout << i << ") Loading data from " << swsFileList[i] << endl;
        
        if( loadSWSData( swsFileList[i], data) )
	{            
   	    if (i > 0 && data._obsId != obsId)
	    {

	        errstr << swsFileList[i - 1] << " and " << swsFileList[i] 
                       <<  " come from different observations"
		       << Sas::warning("differentobsids");
               
      
            }
            obsId = data._obsId;
	  
	    /***********************************************************
	    *
	    * Load the data from each SWS file
            *
	    ***********************************************************/
	    obsMapFilterMapData[data._obsId][data._filter].push_back( data  );
                       
        }
        else
	{
	    errstr << "Could not load data from " << swsFileList[i] << " -ignoring file"
		   << Sas::warning("unabletoloaddata");
         
        }


    } // End of looping through the input list of source-list files


    cout<<"and GOt to this pnt"<<endl;


    /*******************************************************
    *
    * FOR TESTING
    *
    ********************************************************/
    //   checkData(obsMapFilterMapData);



    //    cout << "NUMBER OF POSSIBLE BUGS=" << numBugs << " MAX SEPARATION BETWEEN 2 SOURCES=" << maxDistance << endl;



    if (obsMapFilterMapData.size() < 1)
    {
        errstr << "Too few source-list files (" 
               << obsMapFilterMapData.size() << ") to continue\n"
               << "Please check that the input files are either all product SWS source-list or all product observation source-list files"
	       << Sas::fatal("toofewfiles");
         
    }
    

        cout << "Number of observations=" << obsMapFilterMapData.size() << endl;

    map<string, map<string, vector<SWSData>, OmFilter > >::iterator iter = obsMapFilterMapData.begin();
    for( iter = obsMapFilterMapData.begin(); iter != obsMapFilterMapData.end(); ++iter)
    {
      //        cout << "Processing data for observation id " << iter->first << endl;  

	process( iter->first, iter->second, true, "" );
    } 
    
    cout<<"and process"<<endl;
    /**********************************************************************
    *
    * If source-list files come from more than one observation then
    * need to process 
    *
    *
    **********************************************************************/
 
 
    if(  obsMapFilterMapData.size() > 1) // Only process if more than one observation IDs
    {
   
        /****************************************************************
        *
        * Loop through each observation and
        *     Loop through each filter and
        *         Loop through each sourceID and
        *            average the magnitudes from each exposure for the sourceID
        *   
        *
        *****************************************************************/
        map<string, SWSData, OmFilter> averagedFilterDataMap;
  
    

        averageObservationMagnitudes( obsMapFilterMapData, averagedFilterDataMap );

        map<string, vector<SWSData>, OmFilter> averagedFilterDataMap1;

        map<string, SWSData, OmFilter>::iterator iter;



        for(iter = averagedFilterDataMap.begin(); iter != averagedFilterDataMap.end(); ++iter)
	{
	    string filter = iter->first;

	  	   
            averagedFilterDataMap1[filter].push_back( averagedFilterDataMap[filter] );


        }
  

        /**************************************************************************
	*
        *
        *
        *
        **************************************************************************/
        string obsId = "test";
      
        process( obsId, averagedFilterDataMap1, true, "_OBS");
 
      


    }
    if (pgplotInitialised)
        cbpgclos(); 

    cout<<"and finished process"<<endl;
    
} 

/***********************************************************************
*
* Processes the 
*
*
*
***********************************************************************/
void process( string obsId, map<string, vector<SWSData>, OmFilter >  &filterMapData, 
     bool addVariabilityColumns, string subHeading )
{



   
    errstr << "Processing source-lists for observation " << obsId
           << Sas::message(Msg::App, Msg::Sparse);


    map<string, vector<SWSData>, OmFilter >::iterator iter;

    /*************************************************************************
    *
    * Loop through the SWS data for each filter
    *
    * For each source we want a list of the mags, etc, for each filter
    *
    **************************************************************************/
    map<int, map<string, SWSData, OmFilter> >  sourceMapList; 


    /*************************************************************************
    *
    * Get the user value for the minimun number of detections of the same
    * source to compute variability statistics for that source
    *
    ************************************************************************/
    int minNumCounterParts = intParameter("minnumcounterparts");  



    /**********************************************************************
    *
    * Loop through all the filters
    *
    ***********************************************************************/
    for(iter = filterMapData.begin(); iter != filterMapData.end(); ++iter)
    {
        string filter = iter->first; // The actual filter

  
                      
        errstr << "Processing source-lists for filter " << filter
               << Sas::message(Msg::App, Msg::Sparse);

        vector<SWSData> data = iter->second; // The list of SWS data for the filter
      

    
   

        map<int, int > sourceSizeMap;
 
        /***************************************************************
	*
	* Loop throug all the source-lists
        * and count the number of counterparts for each source ID
        *
        ****************************************************************/ 
      
        for(unsigned i = 0; i < data.size(); i++)
	{
	
	 

	  
	    // Loop through the sources for source-list i 
	    for(unsigned j = 0; j < data[i]._ra.size(); j++) 
	    {
	  
	        // Increment the number of independent detections of this source
	        ++sourceSizeMap[data[i]._srcId[j] ];  // The number of counterparts for this source ID
                  
            }
        }
        
    
     
        vector<SWSData> filterDataList;
        map<int, int>::iterator iter2;

      

        /***********************************************************************
	*
        * Loop through each source in the sourceSizeMap and check how many
        * counterparts it has- if greater than minNumCounterParts then
        * compute variability statistics
	*
        ************************************************************************/   
        for(iter2 = sourceSizeMap.begin(); iter2 != sourceSizeMap.end(); ++iter2)
	{
	
	    int number = iter2->first;   // The number of the unique source

	   
 

	    //  if(number != 63) continue;
            int size = iter2->second;    // The number of counterparts for this unique source

	   

	  
            if(size >= minNumCounterParts) // At least
	    {
	      
                /********************************************************************
		*
                * Loop through all the source-lists again
                *
                ********************************************************************/           

	        
                for(unsigned i = 0; i < data.size(); i++)
	        {

		    
		    /*******************************************************	  
		    *
		    * Loop through the sources for this file
                    *
                    ********************************************************/
                    for(unsigned j = 0, k = 0; j < data[i]._ra.size(); j++)
	            {
		      
                          
		        if ( data[i]._srcId[j] == number && !isNull( data[i]._mag[j]) )
			{

			    /**********************************************************
			    *
			    *
                            *
                            *
                            ***********************************************************/
                            sourceMapList[number][filter]._srcId.push_back( data[i]._srcId[j] );                         
	        
			    sourceMapList[number][filter]._ra.push_back( data[i]._ra[j] );                         
	        	    sourceMapList[number][filter]._dec.push_back( data[i]._dec[j] );
	                    sourceMapList[number][filter]._mag.push_back( data[i]._mag[j] );
                            sourceMapList[number][filter]._magErr.push_back( data[i]._magErr[j] );
                            sourceMapList[number][filter]._significance.push_back( data[i]._significance[j] );
                            sourceMapList[number][filter]._corrCountRate.push_back( data[i]._corrCountRate[j] );
                            sourceMapList[number][filter]._corrCountRateErr.push_back( data[i]._corrCountRateErr[j] );
                            sourceMapList[number][filter]._eFlag.push_back( data[i]._eFlag[j] );
                            sourceMapList[number][filter]._cFlag.push_back( data[i]._cFlag[j] );
                            sourceMapList[number][filter]._qFlag.push_back( data[i]._qFlag[j] );


			    /****************************************************************************
			    *
                            *
                            *
                            *
                            ****************************************************************************/ 
		


			    sourceMapList[number][filter]._mjdStartList.push_back( data[i]._mjdStartList[j]);
			    sourceMapList[number][filter]._mjdEndList.push_back( data[i]._mjdEndList[j] );
			    sourceMapList[number][filter]._mjdMidList.push_back( data[i]._mjdMidList[j] );


      
                           sourceMapList[number][filter]._obsId = data[i]._obsId;

                           sourceMapList[number][filter]._object = data[i]._object;


		    
			   sourceMapList[number][filter]._obsIdList[number].push_back( data[i]._obsIdList[number][k++]);

			 
                      
                            /*****************************************************************************
			    *
                            * Jump out of the loop because we have found the source in the SWS file
                            *
                            ******************************************************************************/
			    //			    break;
			}   

                    }        
  
               }
 
            }

        } // End of looping through all the sources
	
    }     // End of looping through all the filters 

  

    /************************************************************************
    *
    * Check that there are some sources to plot
    *  
    *************************************************************************/               
    if( sourceMapList.size() == 0)
    {

  
	errstr <<  "No sources to plot"
	       << Sas::warning("novariablesources");


        return;
    }



   

    //    cbpgpanl(1, 1);

    errstr << "Plotting the variable sources"
          << Sas::message(Msg::App, Msg::Sparse);


    /******************************************************
    *
    *
    *
    * NB SWSData stores a vector of _mag, _magErr, _corrCountRate, _corrCountRateErr
    *
    *
    *
    *
    *
    *
    ******************************************************/

    map<string, map<int, VariabilityData>, OmFilter> variabilityData;   

    map<int, map<string, SWSData, OmFilter> >::iterator iter1; 

    
    /*********************************************************************
    *
    *
    * Loop through all the unique sources that have a minimum of 
    * minnumcounterparts counterparts
    *
    *
    * Wish to compute variability data following 
    * Welch, D.L., and Stetson, P.B., Astron J., 1993, 105, p.1813 
    **********************************************************************/

    //    map<int,  map<string, VariabilityData, OmFilter> > stetsonDataMap;

  
  
    for(iter1 =  sourceMapList.begin(); iter1 != sourceMapList.end(); ++iter1)
    {
        
        
        int number = iter1->first;      // The unique source id
	// if(number != 63) continue; 
        map<string, SWSData, OmFilter> swsData = iter1->second;   
      

        map<string, SWSData, OmFilter>::iterator iter2;

        map<string, VariabilityData, OmFilter> varDataMap;
        for(iter2 = swsData.begin(); iter2 != swsData.end(); ++iter2)
	{    
         
	    string filter = iter2->first;
            SWSData data = iter2->second;

	    //    cout << "Line 705- filter=" << filter << " number=" << number << endl; 
	    // cout << "LINE 700 calling checksource1\n";

	    //checkSource1( number, data._ra, data._dec );




	    /***************************************************************
	    *
            * Compute the variability statistics for source number number
            * and filter filter
            *
            ****************************************************************/
            varDataMap[filter]._filter = filter; 
            varDataMap[filter]._number = number; 
            varDataMap[filter]._object = data._object; 
            varDataMap[filter]._obsId = data._obsId; 
            varDataMap[filter]._swsData = data; 
            varDataMap[filter]._obsIdList = data._obsIdList[number] ; 

        
	    varDataMap[filter]._obsIdList.push_back( data._obsId ); 

	
            /***************************************************************
	    *
            * Compute the variability parameters
            *
            ****************************************************************/
                                             
	    computeVariabilityParameters(data, varDataMap[filter]);
            variabilityData[filter][number] = varDataMap[filter];

            /************************************************************
	    *
            *
            *
            **************************************************************/
	    //	    stetsonDataMap[number][filter] = varDataMap[filter];


 

          
        }   // End of looping through the filters for that source
       

    

        /***************************************************************
        *
        *
        *
        ****************************************************************/
        plotSourceVariability( obsId, number, varDataMap );

      

    }   // End of looping through the sources

 
    /************************************************************************
    *
    * Compute Welch & Stetson variability paramters
    *
    *************************************************************************/
    //    stetsonVariabilityData(  stetsonDataMap );



    if( !addVariabilityColumns) {


        return;
     }
   


    /************************************************************************
    *
    * Add variability columns to the source-list file
    *
    ************************************************************************/

    errstr << "Adding the variability columns to file " << stringParameter("obsset")
           << Sas::message(Msg::App, Msg::Sparse);


    map<string, map<int, VariabilityData>, OmFilter>::iterator iter2;  
  
    for(iter2 = variabilityData.begin(); iter2 != variabilityData.end(); ++iter2)
    {
        string filter = iter2->first;
        map<int, VariabilityData> varData = iter2->second;
        addVariabilityColumn( filter, varData, subHeading);

     
    }
  
}
/*****************************************************************************
* 
* Loads data from the exposure SWS file
*
*
******************************************************************************/
bool loadSWSData( string file, SWSData &data)
{

    /***************************************************
    *
    * Try to Open the file
    *
    ***************************************************/

    DataSet *dataSet = dataSetServer->open(file, Dal::Read);
    if (!dataSet)
    {
      errstr << "Unable to read file " << file
	     << Sas::warning("fileproblem");
           
     
        return false; 

    }   

    const Table *table = dataSet->table("SRCLIST");
   
    if (table->hasColumn("SRC_ID")) // Need this to match sources 
    { 

        const Table *table = dataSet->table("SRCLIST");
	//cout << "L793 nrows=" << table->rows() << endl;
        if( table->rows()== 0)        
	{

            dataSetServer->close(dataSet); // Finished with the file


            return false;


        }
        else
	{
            LoadColumnData( table,  "SRC_ID",  data._srcId);
        } 
 
    }
    else
    {
      //cout << "L812 no SRC_ID column is found " << endl;
        dataSetServer->close(dataSet); // Finished with the file

        return false;


    }
   

    if ( !dataSet->hasAttribute("OBS_ID"))
    {
        dataSetServer->close(dataSet); // Finished with the file

        return false;

    }



    data._fileName = file;


    /************************************************************
    *
    * Get the start and end times
    *
    *************************************************************/
    string startObs = dataSet->attribute( "DATE-OBS")->asString();
    string endObs = dataSet->attribute( "DATE-END")->asString();

    data._filter = dataSet->attribute("FILTER")->asString();

    data._obsId = dataSet->attribute("OBS_ID")->asString();  


    data._object = dataSet->attribute("OBJECT")->asString();  
      
    data._mjdStart  = getTime( startObs );
  
    data._mjdEnd = getTime( endObs );


    data._mjdMid =  ( data._mjdEnd + data._mjdStart ) / 2.0;


    data._mjdStartList.resize( table->rows() );
    data._mjdEndList.resize( table->rows() );
    data._mjdMidList.resize( table->rows() );


       for( unsigned i = 0; i < table->rows(); i++)
       {     
           data._mjdStartList[i] = data._mjdStart;
           data._mjdEndList[i] = data._mjdEnd;
           data._mjdMidList[i] = data._mjdMid;
           int srcId = data._srcId[i];


           data._obsIdList[srcId].push_back( data._obsId );
        }
         
 

 
    /*********************************************************
    *
    * FOR TESTING
    *********************************************************/






       

               static int count = -1;
        static int count1 = -1;

	/*  COMMENTED OUT 

        char text[100];
        ++count;

        sprintf(text, "OBSID%05d", count / 16 + 1);
        data._obsId = text;
  

 
        cout << "Line 910- obsid=" << data._obsId << endl;
        data._mjdStart = float(count / 4) * 100.0;
        data._mjdEnd = float(count / 4 + 1) * 100.0;

        data._mjdMid =  ( data._mjdEnd + data._mjdStart ) / 2.0;

  
        if(count%4 == 0)
       {      
        ++count1;
        data._filter = "V";
        } 
        else if(count%4 == 1)
        {
        data._filter = "B";
        } 
        else if(count%4 == 2)
        {
        data._filter = "U";
        }
        else
        {
          data._filter = "UVW2"; 
        }


	*/     // END OF COMMENTED OUT 
   
    data._object = dataSet->attribute("OBJECT")->asString();

    
 
    /********************************************************
    *
    *  Load the RAs and Decs
    ********************************************************/
    if (table->hasColumn("RA_CORR"))
    {
        LoadColumnData( table,  "RA_CORR",  data._ra);
        LoadColumnData( table,  "DEC_CORR", data._dec);
    }
    else if (table->hasColumn("RA"))
    {              
        LoadColumnData( table,  "RA", data._ra);
        LoadColumnData( table,  "DEC", data._dec);
 
    }
    else
    {

        dataSetServer->close(dataSet); // Finished with the file
        return false;

    }

    LoadColumnData( table,  "SIGNIFICANCE",  data._significance);


    /*****************************************************
    * Load the instrumental magnitudes
    ******************************************************/

  
    LoadColumnData( table,  "MAG",  data._mag);
    LoadColumnData( table,  "MAGERR", data._magErr);


    /*****************************************************
    * Load the corrected count-rates
    ******************************************************/

  
    LoadColumnData( table,  "CORR_RATE",  data._corrCountRate);
    LoadColumnData( table,  "CORR_RATE_ERR", data._corrCountRateErr);


    /*****************************************************
    * Load the flags
    ******************************************************/
    LoadColumnData( table,  "EFLAG",  data._eFlag);
    LoadColumnData( table,  "CFLAG",  data._cFlag);
    LoadColumnData( table,  "QFLAG",  data._qFlag);



    dataSetServer->close(dataSet); // Finished with the file



    //  for(unsigned i = 0; i < data._mag.size(); i++)
    // {

    // if ( data._srcId[i] == 69 || data._srcId[i] == 63)
    // {
    //	cout << "RA AND DEC FOR SRD " << data._srcId[i] << endl;

    //	cout << i << ") ra= " <<  setiosflags(ios::fixed) << setprecision(5) <<  data._ra[i] << " dec=" << data._dec[i]  
    //	     << " extensionflag=" << (int)data._eFlag[i] << endl;

    //   break;

    // }


    // }


        return true;
   
        int size = 10;
     data._srcId.resize(size);
     data._mjdStartList.resize( size );
      data._mjdEndList.resize( size );
     data._mjdMidList.resize( size );

     data._eFlag.resize(size);
     data._cFlag.resize(size);
     data._qFlag.resize(size);

     data._mag.resize(size);
     data._magErr.resize(size);

     data._corrCountRate.resize(size);
     data._corrCountRateErr.resize(size);

     data._significance.resize(size);


     data._ra.resize(size);
     data._dec.resize(size);


     cout << "Load swsdata line 1019- file=" << file << endl;
     for(unsigned i = 0; i < data._mag.size(); i++)
      {
         int number = rand()%10001;
        data._mag[i] += -0.5 + (double)number / 10000.0;
         data._corrCountRate[i] = 10000000.0 * pow( 10.0, -0.4 * data._mag[i] );
       

    	int k = (count / 16 + 1) * 100;   
      data._mjdStart = float(10000 + k) + float(i + count1) / 24.0;
     data._mjdEnd = float(10000+k) + float(i + 1 + count1) / 24.0;

          data._mjdMid =  ( data._mjdEnd + data._mjdStart ) / 2.0;

            data._mjdStartList[i] = data._mjdStart;
       data._mjdEndList[i] = data._mjdEnd;
        data._mjdMidList[i] = data._mjdMid;
  
       cout << i << " ) mjdstart=" << 	setiosflags(ios::fixed) << setprecision(4)
             << data._mjdStart
    	     << " mjdMid=" <<   data._mjdMid
    	     << " mjdEnd=" <<   data._mjdEnd << endl;

        }
    return true;


}
/*****************************************************************************
*
*
******************************************************************************/
void loadSrcListData(string file, vector < OmSourceParameter > &parameters, string &filter)
{
    int size = 0;
    string expId;
    if ( loadSourceParameters( file, "SRCLIST", size, parameters, filter, expId) )
    {



    }

}


/*******************************************************************************
*
*
* Compute the chi-squared
*
*
*********************************************************************************/
float computeChiSquared(vector<float> &x, vector<float> &dx, float &theMean, float &prob)
{

    /**********************************************************
    * Compute the weighted mean
    ***********************************************************/

    double mean = (double)0.0;
    vector<double> weight(x.size() );
    double sumWeight = (double)0.0;

    for(unsigned i = 0; i < x.size(); i++)
    {
        double w = (double)1.0 / ( double(dx[i]) * double(dx[i]));
        weight[i] = w;
        mean += double(x[i]) * w;     
        sumWeight += w;
      
    }

    mean /= sumWeight;
    
    double chiSquared = (double)0.0;

    for(unsigned i = 0; i < x.size(); i++)
    {
        double diff = x[i] - mean;
        chiSquared += diff * diff * weight[i];

    }
    theMean = (float)mean;

    //    cout << "COMPUTECHISQUARED mean=" << mean << " chisquared=" << chiSquared << endl;



    int ndf = x.size() - 1;
    prob = F90MODPROC( coords, chisqprobability)( &chiSquared, &ndf);


         
    return float( chiSquared / double(x.size() - 1) );
 

}



/*******************************************************************
*
*
* Gives various statistics about variability
*
*
*
*
*
*******************************************************************/
void variabilityStatistics( int maxSize)
{

    ofstream os("variabilitystats.dat");

    cout << "OM VARIABILITY STATISTICS\n"; 

    os << "OM VARIABILITY STATISTICS\n"; 


    QDateTime time = QDateTime::currentDateTime();;

    cout << "Computed on " << time.toString ( "ddd MMMM d yy hh:mm:ss.z" )  
         << " Number of OM observation source-list files checked=" 
         <<  noObservationFilesChecked << endl;


    os << "Computed on " << time.toString ( "ddd MMMM d yy hh:mm:ss.z" )  
         << " Number of OM observation source-list files checked=" 
         <<  noObservationFilesChecked << endl;


    cout << setw(10) << "FILTER" 
       << setw(15) << "N SOURCES"
       << setw(20) << "N > 2 COUNTER PARTS"
       << setw(20) << "N VARIABLE"
       << setw(10) << "% VARIABLE"
       << endl; 



    os << setw(10) << "FILTER" 
       << setw(15) << "N SOURCES"
       << setw(20) << "N > 2 COUNTER PARTS"
       << setw(20) << "N VARIABLE"
       << setw(10) << "% VARIABLE"
       << endl; 


    for(unsigned i = 0; i < omFilter.size(); i++)
    {
        string filter = omFilter[i];
        
        cout << setw(10) << filter 

             << setw(15) << noSources[filter]      // The total number of sources in the combined source-lists with non-null significance for the filter.

             << setw(20) << noMultiSources[filter] // The total number of sources in the combined source-lists with 2 or more counterparts for the filter.
             << setw(20) << noVarSources[filter];  // The total number of sources in the combined source-lists which may be variable (nsd>=minnstd1)



         os  << setw(10) << filter 

             << setw(15) << noSources[filter]

             << setw(20) << noMultiSources[filter]
             << setw(20) << noVarSources[filter];


       
 
        if(noVarSources[filter] > 0)
	{

	  cout << setiosflags(ios::fixed) << setprecision(2)
	       << setw(10) << 100.0 * double(noVarSources[filter]) / double(noMultiSources[filter]) << endl;
	
          cout.unsetf(ios::showpos);
	  os << setiosflags(ios::fixed) << setprecision(2)
	       << setw(10) << 100.0 * double(noVarSources[filter]) / double(noMultiSources[filter]) << endl;
	
          os.unsetf(ios::showpos);



        }
        else
	{
          cout << setw(10) << "0\n";
          os << setw(10) << "0\n";
        }

    }


    /*********************************************************************
    *
    *
    **********************************************************************/
    cout << "\nThe following give the number of sources with variability parameters computed from \"N Computed\" sources\n";

    cout << setw(10) << "N Computed";



    os << "\nThe following give the number of sources with variability parameters computed from \"N Computed\" sources\n";

    os << setw(10) << "N Computed";



    for(unsigned i = 0; i < omFilter.size(); i++)
    {
        cout << setw(10) << omFilter[i];
        os << setw(10) << omFilter[i];
      
    }
    cout << endl;
    os << endl;

    for(int j = 1; j <= maxSize; j++)
    {

      

        int count = 0;
        for(unsigned i = 0; i < omFilter.size(); i++)
        {

            count += filterCount[omFilter[i]][j];
        }
        if(count == 0) continue;

        cout << setw(10) << j;
        os << setw(10) << j;

        for(unsigned i = 0; i < omFilter.size(); i++)
        {

 
	  cout << setw(10) << filterCount[omFilter[i]][j];
	  os << setw(10) << filterCount[omFilter[i]][j];


        }
        cout << endl;
        os << endl;
    }

    
    /******************************************************************************
    *
    *
    *
    *
    *******************************************************************************/
    cout << "\n\nFlag statistics\n";
    cout <<"\"Sample size\" is the total number of sources with 2 or more counterparts, for a given filter\n";
    cout <<"\"Num Variable\" is the total number of variability parameters that have been computed for a given filter (nstd>=" << minStd1 << ")\n";
    cout <<"\"% Variable\" is the percentage of sources with more than 2 counterparts that satisfy the variability criteria, for a given filter\n";

    cout <<"\"% e flag=1\" is the percentage of \"Num Variable\" in which one or more of the merged sources has an extension flag of 1\n";  
    cout <<"\"% q flag > 0\" is the percentage of \"Num Variable\" in which one or more of the merged sources has a quality flag > 0\n";
    cout <<"\"% c flag > 0\" is the percentage of \"Num Variable\" in which one or more of the merged sources has a confusion flag > 0\n";
    cout <<"\"% flags eq 0\" is the percentage of \"Num Variable\" in which all the merged sources have all flags = 0\n\n";



    cout << setw(10) << "Filter"
         << setw(15) << "Sample size"  
     
         << setw(15) << "Num Variable"  
         << setw(15) << "% Variable"  
         << setw(15) << "% e flag=1"
         << setw(15) << "% q flag > 0"
         << setw(15) << "% c flag > 0"
         << setw(15) << "% flags eq 0"
   
         << endl;



    os << "\n\nFlag statistics\n";
   
    os <<"\"Sample size\" is the total number of sources with 2 or more counterparts, for a given filter\n";
    os <<"\"Num Variable\" is the total number of variability parameters that have been computed for a given filter\n";
    os <<"\"% Variable\" is the percentage of sources with 2 or more counterparts that satisfy the variability criteria, for a given filter\n";


    os <<"\"% e flag=1\" is the percentage of \"Num Variable\" in which one or more of the merged sources has an extension flag of 1\n";  
    os <<"\"% q flag > 0\" is the percentage of \"Num Variable\" in which one or more of the merged sources has a quality flag > 0\n";
    os <<"\"% c flag > 0\" is the percentage of \"Num Variable\" in which one or more of the merged sources has a confusion flag > 0\n";
    os <<"\"% flags eq 0\" is the percentage of \"Num Variable\" in which all the merged sources have all flags = 0\n\n";

  
    os << setw(10) << "Filter"
         << setw(15) << "Sample size"  
         << setw(15) << "Num Variable"  
         << setw(15) << "% Variable"   
         << setw(15) << "% extension flag=1"
         << setw(15) << "% q flag > 0"
         << setw(15) << "% c flag > 0"
         << setw(15) << "% flags eq 0"
         << endl;


    for(unsigned i = 0; i < omFilter.size(); i++)
    {

      string filter = omFilter[i];
      float ePerc = 0.0, qPerc = 0.0, cPerc = 0.0, gPerc = 0.0;
     
      if( noMultiSources[filter] > 0)
      {      
          percVariable[filter] = 100.0 * (double)noVarSources[filter] / (double)noMultiSources[filter];
      }   




      if(countSources[filter] > 0)
      { 
 
          ePerc  = 100.0 * (double)countExtended[filter] / (double)noVarSources[filter]; 
          qPerc  = 100.0 * (double)countQuality[filter] /  (double)noVarSources[filter]; 
          cPerc  = 100.0 * (double)countConfusion[filter] / (double)noVarSources[filter]; 
          gPerc  = 100.0 * (double)countFlagsZero[filter] / (double)noVarSources[filter]; 

      } 


      cout << setw(10) << omFilter[i] 
         << setw(15) << noMultiSources[filter] 
         << setw(15) << noVarSources[filter] 
	 << setiosflags(ios::fixed)
	 << setprecision(2)
         << setw(10) << percVariable[filter]       
         << setw(15) << ePerc   
         << setw(15) << qPerc  
         << setw(15) << cPerc  
         << setw(15) << gPerc << endl;   
        cout.unsetf(ios::showpos);


     os << setw(10) << omFilter[i] 
        << setw(15) << noMultiSources[filter] 
        << setw(15) << noVarSources[filter] 
	<< setiosflags(ios::fixed)
	   << setprecision(2)
           << setw(10) << percVariable[filter]
           << setw(15) << ePerc   
           << setw(15) << qPerc  
           << setw(15) << cPerc  
           << setw(15) << gPerc << endl;   
     os.unsetf(ios::showpos);

    }
  

}






/*************************************************************************
*
* Given a list of sources, stored in a map, with the filter as the key. 
* plots the magnitudes as a function of time
*
*
*************************************************************************/
void plotSourceVariability( string obsId, int srcId, map<string, VariabilityData, OmFilter>  &varData)
{

  
    int minNumSources = intParameter("minnumcounterparts");

    int eFlag = intParameter("eflag");

    int cFlag = intParameter("cflag");

    int qFlag = intParameter("qflag");


    float minStd = realParameter("minstd"); // The minimum nSigma for inclusion in the variability catalogue


    map<string, VariabilityData, OmFilter>  ::iterator iter;   

    /**************************************************************************
    *
    * Loop through each merged source - each one will have at least 2 counterparts
    * and data for one or more filters
    *
    **************************************************************************/
    int nGraphs = (int)varData.size();  // No of filters for this source

 

    vector<vector<float> > xData(nGraphs), yData(nGraphs), yDataErr(nGraphs);
    vector< vector<string> > symbolText(nGraphs);
    vector<string> comment;


    vector<int> colourIndex;         // Colour for each filter



    vector<double> raList, decList;

  
   
    /*****************************************************************
    *
    * Loop through each filter for this source
    *
    ******************************************************************/
    nGraphs = 0;
    float minTime = 1e9, maxTime = 0.0;





    /****************************************************************
    *
    * Each data point may come from a different observation
    * Want to identify which observation each point comes from
    *
    *****************************************************************/
   

    map<string, VariabilityData, OmFilter>  ::iterator iter1  = varData.begin(), iter2 = varData.begin();   



    //    map<string, vector<vectorint> pointColour; // A different colour for each observation


    /***************************************************************
    *
    *
    *
    ***************************************************************/

    for(iter = varData.begin(); iter != varData.end(); ++iter, ++iter2)  // Looping through the filters for this source
    {
    
    
        string filter = iter->first; // The filter
 
   
	VariabilityData data = iter->second;

        
        if(data._nstd < minStd) continue;

        if( (int)data._swsData._ra.size() < minNumSources) continue;  

	
	

        if( eFlag != -1)
	{
	    if (data._eFlagMax != eFlag) continue;

        }
        if( cFlag != -1)
	{
	    if (data._cFlagMax != cFlag) continue;

        }

        if( qFlag != -1)
	{
	    if (data._qFlagMax != qFlag) continue;

        }

     
        raList.push_back( data._meanRa);
        decList.push_back( data._meanDec);
    
	
        /****************************************************************
	*
        * Loop through the sources for the filter
        *
        *****************************************************************/
        float minMag = 100.0;
        float maxMag = -minMag;

        for(unsigned i = 0; i < data._swsData._ra.size(); i++)
        {
     

	  //	    dataObsMap[nGraphs][i] = data._swsData._obsIdList[i];

	    /**********************************************
	    *
            * Need the time
            *
            ***********************************************/	               
	
            
            float time = data._swsData._mjdMidList[i];
           


          
	    xData[nGraphs].push_back( time );

 

            maxTime = max(maxTime, time );

            minTime = min(minTime, time );
   

	    yData[nGraphs].push_back( data._swsData._mag[i] );
            minMag = min(minMag, data._swsData._mag[i] );
            maxMag = max(maxMag, data._swsData._mag[i] );


  	    yDataErr[nGraphs].push_back( data._swsData._magErr[i] );




	    //  string exposureString =  ToString( source[i].exposure ) + " " + source[i].expId;
            string exposureString = "";    
	    symbolText[nGraphs].push_back( exposureString );
  
  

         }

        /***********************************************************
	*
        *
        *
        ************************************************************/

	//	checkSource1( srcId, raList, decList );



     

        float magRange = maxMag - minMag;    
        nGraphs++;
        float median = Median( data._swsData._mag);      
        char text[100];  
	sprintf(text, "Filter %s, max n\\gs=%.1f median mag=%.2f mag range=%.2f", 
               (const char *)filter.c_str(), data._nstd, median, magRange);  
           


	//	sprintf(text, "Filter %s, median mag=%.2f mag range=%.2f", 
        //       (const char *)filter.c_str(), median, magRange);  
           






	comment.push_back( text );
	colourIndex.push_back( nGraphs );


        if( nGraphs == maxNumGraphs)
	{
            
	    drawTheGraph( obsId, srcId, nGraphs, xData, yData, yDataErr, 
                          raList, decList, comment, varData, iter1,  
                          minTime, maxTime);
	 
            int numGraphs = (int)varData.size() - nGraphs;

            xData.clear();
	 
            yData.clear();
            yDataErr.clear();
            raList.clear();
            decList.clear();
            comment.clear();
            colourIndex.clear(); 
            iter1 = iter;
	    ++iter1;
            nGraphs = 0;
            xData.resize(numGraphs);
            yData.resize(numGraphs);
            yDataErr.resize(numGraphs);
            symbolText.resize(numGraphs);
          	

        }
     
  
    } // End of looping through the filters for the source


    if(nGraphs > 0) 
    { 	  
       

   
        drawTheGraph(obsId, srcId, nGraphs, xData, yData, yDataErr, raList, decList, 
                     comment, varData, iter1, minTime, maxTime);
   
        
    
   
    }
}
/**********************************************************
*
*
*
*
*
*
***********************************************************/
void drawTheGraph( string obsId, int srcId, int nGraphs, 
                               vector< vector<float> > &xData,
                               vector< vector<float> > &yData, 
                               vector< vector<float> > &yDataErr, 
		               vector<double> &raList, vector<double> &decList,
                               vector<string> &comment, 
		               map<string, VariabilityData, OmFilter>  &varData,
		               map<string, VariabilityData, OmFilter>::iterator &iter1,   
                               float minTime, float maxTime)

{

  //  if(srcId != 224) return;
    if(obsId == "") {};


    /****************************************************************
    *
    *
    *
    *****************************************************************/
    openPgplot();




    int minNumSources = intParameter("minnumcounterparts");

    int eFlag = intParameter("eflag");

    int cFlag = intParameter("cflag");

    int qFlag = intParameter("qflag");


    float minStd = realParameter("minstd"); // The minimum nSigma for inclusion in the variability catalogue



    /*************************************************
    *
    *
    * Normalize the times
    *
    *
    *************************************************/
    char xLabel[100];


    vector<float> yMin(nGraphs), yMax(nGraphs);    
      
    if( (maxTime - minTime) < 10.0) // ie less than one day (From a single observation)
    {
   
        float minTime1 = floor( minTime ); // The lowest integer number
   
        minTime -= minTime1;
        maxTime -= minTime1;
      
        sprintf(xLabel, "Modified Julian Day %.4f + (hours)", minTime1 );
   
        for(int i = 0; i < nGraphs; i++)
	{       
            for(unsigned j = 0; j < xData[i].size(); j++)
            {
	      xData[i][j] = (xData[i][j] - minTime1) * 24.0;
               
            }
        }
        minTime -= 0.01;
        maxTime += 0.01;


        minTime = floor( minTime * 24.0 );
        maxTime = ceil( maxTime  * 24.0);
       

    } 
    else  // Greater than one day (From more than one observations)
    {

        minTime = floor( minTime );
        maxTime -= minTime;    

        sprintf(xLabel, "Modified Julian Day %.4f + (days)", minTime);
	
        for(int i = 0; i < nGraphs; i++)
	{       
            for(unsigned j = 0; j < xData[i].size(); j++)
            {
	      xData[i][j] -= minTime;
               
            }
        }


	maxTime += 50.0;
	minTime = -50.0;  

    }

    /*****************************************************************
    *
    * Get ymin and ymax values
    *
    ******************************************************************/
 
    for(int i = 0; i < nGraphs; i++)       
    {

        float yMin1 = 100.0, yMax1 = -100.0;

        for(unsigned j = 0; j < xData[i].size(); j++)
        {
	  
            yMin1 = min(yMin1, yData[i][j] - yDataErr[i][j]);
            yMax1 = max(yMax1, yData[i][j] + yDataErr[i][j]);

        }
       
        float median = Median( yData[i] ); 
 
        getNiceMagRange(median, yMin1, yMax1, yMin[i], yMax[i] );

    };

  


    /****************************************************************
    *
    *
    *****************************************************************/
    cbpgsci(1);               

    
    /*******************************************************************
    *
    * Compute mean RA and Dec
    *
    *******************************************************************/
    //   checkSource1( srcId, raList, decList );




    double meanRa = 0, meanDec = 0;
    averageCelestialCoordinates( raList, decList, meanRa, meanDec);

 
    AngleFormatter af;
    af.setMode(AngleFormatter::HMS);

    string ra = af.asString( meanRa);

       
    af.setMode(AngleFormatter::DMS);
    string dec = af.asString( meanDec );
   
      	 
 

    char text[100];
    sprintf(text, "Source ID %*d, RA=%s, Dec=%s", int( log10( (float)srcId + 1.1)), srcId, ra.c_str(), dec.c_str()); 


    cbpgsch(1.0);
 
    /****************************************************************************
    *
    * Now draw the graph
    *
    ******************************************************************************/

 
    /********************************************
    *
    * Plot the magnitudes for each filter on the same graph
    * Add a constant to each mag to make the graph look prettier
    *
    *
    ********************************************/


    /****************************************************************
    * Each data point may have come from a different observation
    * Each point will be given a different colour to identify which
    * observation it came from
    *****************************************************************/
    map<int, map<int, string> > dataObsMap;

 
    map<string, int> numObs;

    /*******************************************************************
    *
    *
    *
    ********************************************************************/
    int k = 0;   
    for(iter1 = varData.begin() ; iter1 != varData.end() ; iter1++) 	    
    {

       
        k++;
        VariabilityData data = iter1->second;


        if(data._nstd < minStd)
	{ 
            continue;
        }



	if( data._swsData._obsIdList[srcId].size() == 0)
	{             
            numObs[obsId] = 1;
         
        }
        else
	{
	  

            for (unsigned i = 0; i < data._swsData._obsIdList[srcId].size(); i++)
            {
	    
                /***********************************************************
	        *
                *
                *
                ************************************************************/
                string obsId = data._swsData._obsIdList[srcId][i];
     

	        dataObsMap[k][i] = obsId;
             
                numObs[obsId] = 1;
         

            }
        }


    }





    //   cout << "\n\n**************************************************************\n";
    //    cout << "DRAWTHEGRAPH Line 3102- The list of all the obs for this unique source number\n";


    map<string, int>::iterator iter8;
    map<string, int> colourIndexMap;
    int num = 1;

    for(iter8 = numObs.begin(); iter8 != numObs.end(); ++iter8)
    {
      //   cout << ") " << iter8->first << endl;    
        colourIndexMap[iter8->first] = num++;

    }


 
    /*******************************************************************
    *
    *
    *
    ********************************************************************/

    k = 0;



    vector<int> colourIndex(nGraphs, 2);    
    /****************************************************************
    *
    *
    * Several graphs on top of one another
    *
    *
    *****************************************************************/

    char msgy[] = "Vega Magnitude";
    char msgTitle[] = "";
    drawGraph( nGraphs, xData, yData, yDataErr, colourIndex,
               comment, xLabel,
               msgy,
               msgTitle,
               0.1,
               0.9,
               minTime,
               maxTime,
               //yMin,
               //yMax,
               yMax,
               yMin,
               false,
               0.25);
 
    cbpgsch(0.9);
    cbpgslw(3);
  
    cbpgmtxt("T", 1.5, 0.0, 0.0, text);


    /***********************************************************
    *
    * 
    *
    ************************************************************/
    int i = 1;
    int k1 = numObs.size();
    string text10;


 
    if(k1 == 1)
        text10 = "OBSERVATION ID ";
    else
        text10 = "OBSERVATION IDS- ";

    for(iter8 = numObs.begin(); iter8 != numObs.end(); ++iter8)
    {
      
 
        string obsId = iter8->first;
        if( i < k1)
	{
            text10 += obsId + string(",  ");
        }
        else 
	{
            text10 += obsId;
        }
        i++; 


    }
    if( k1 == 1)
    {
        iter1 = varData.begin();
        VariabilityData data = iter1->second;
	text10 += " TARGET " +  data._object;

    } 
  

    cbpgslw(3);
    cbpgsch(0.8); 
    cbpgsci(1);
    cbpgmtxt("T", 0.5, 0.0, 0.0, text10.c_str());
  

    cbpgsci(1);

     
    /**************************************************************
    *
    * Draw the line where the median is
    *
    ***************************************************************/   
    k = 0;

    /*******************************************************************
    *
    *
    *
    ********************************************************************/
    for(iter1 = varData.begin() ; iter1 != varData.end() ; iter1++) 	    
    {

        
        if(k == nGraphs)
        {
	    break;
	   

        }

     
          
        VariabilityData data = iter1->second;

        if(data._nstd < minStd)
	{ 
            continue;
        }


        /*******************************************************************
	*
        *
        *
        *******************************************************************/   
        cbpgsvp( Graph::x1List[k], Graph::x2List[k], Graph::y1List[k], Graph::y2List[k]);
 
        
        cbpgswin( Graph::xminList[k], Graph::xmaxList[k], Graph::yminList[k], Graph::ymaxList[k]);

        float median = Median( yData[k] ); 
       

	cbpgsci(3);   
        cbpgsls( 2 );
        cbpgslw( 3 );
        cbpgmove( -1000000000.0, median);
        cbpgdraw( 1000000000.0, median);
                
        cbpgslw(1);
        cbpgsls(1);
        cbpgsci(1);		     
              	
        if( (int)data._swsData._ra.size() < minNumSources) continue;  

        if( eFlag != -1)
	{
	    if (data._eFlagMax != eFlag) continue;

        }
        if( cFlag != -1)
	{
	    if (data._cFlagMax != cFlag) continue;

        }

        if( qFlag != -1)
	{
	    if (data._qFlagMax >= qFlag) continue;

        }


        bool eSet = false;
        bool qSet = false;
        bool cSet = false;
        cbpgsfs(2);

     
        for(unsigned i = 0; i < data._swsData._ra.size(); i++)
        {     
            cbpgsch(0.6);
                  
	    if( data._swsData._eFlag[i] != 0)
	    {                    
                cbpgsci( 8 );          
		cbpgpt1( xData[k][i], yData[k][i], 24);              
                eSet = true;                   
	    }
             
	    if( data._swsData._qFlag[i] != 0)
	    {
                cbpgsci( 9 );     
		cbpgpt1( xData[k][i], yData[k][i], 25);    
                qSet = true;
                 
	    }


            if( data._swsData._cFlag[i] != 0)
	    {
                cbpgsci( 10 ); 
	        cbpgpt1( xData[k][i], yData[k][i], 26);
                cSet = true;
                       
	    }
             

	    cbpgsci(1);
           
        }

        int nSet = 1;  
        if( eSet )
        {
            cbpgsci( 8 );             
	    cbpgsch(0.8);
	    cbpgmtxt("T", -nSet, 0.8, 0.0, "Extension flag=1");
            cbpgsch(1.0);
            nSet++;
                
        }

        if( qSet )
        {
            cbpgsch(0.8);
            cbpgsci( 9 ); 
	    cbpgmtxt("T", -nSet, 0.8, 0.0, "Quality flag>0");
            cbpgsch(1.0);
            nSet++; 

        }
                        
        if( cSet )
	{
            cbpgsci( 10 ); 
            cbpgsch(0.8);
	    cbpgmtxt("T", -nSet, 0.8, 0.0, "Confusion flag>0");
                        
            cbpgsch(1.0);    
            nSet++;
    

        }                                  
	k++;      
            
    }

    /********************************************
    *
    * Finished the plot
    *
    ********************************************/
        
    cbpgpage();  


}



/**********************************************************************
*
*
*
*
*
***********************************************************************/
void updateStatistics(   map<int, map<string, VariabilityData, OmFilter> > &data, int &maxSize) 
{


    if(data.size() == 0) return;

    //   cout << "update statistics\n";
    /**********************************************************************
    *
    * Loop through all the sources for this observation id
    *
    ***********************************************************************/

    map<int, map<string, VariabilityData, OmFilter> > ::iterator iter1;   

    for(iter1 = data.begin(); iter1 != data.end(); iter1++)  // Looping through the sources
    {
	                 
        map<string, VariabilityData, OmFilter> varData = iter1->second;
            
     
        /*********************************************************
	*
	* Loop through each filter for this source
        *
        **********************************************************/

        map<string, VariabilityData, OmFilter>::iterator iter2;

	for (iter2 = varData.begin(); iter2 != varData.end(); iter2++) 
	{
           
 

	    string filter = iter2->first;  // The filter
	    //         cout << "nstd=" << varData[filter]._nstd << endl;
            if(varData[filter]._nstd < minStd1) continue; // Only count if nstd > minstd1 (2.0)
 
                 
	    vector<OmSourceParameter> source = iter2->second.source; // The source parameters


	    ++filterCount[filter][ source.size() ];

            maxSize = max(maxSize, (int)iter2->second.size ); 
    

            countSources[filter] += source.size();
            /*******************************************************************
	    * 
            * Find percentage of sources with an extension flag of 1
            * Quality flag > 0 and confusion flag > 0
            *
            ********************************************************************/
            int n1 = 0, n2 = 0, n3 = 0, n4 = 0; 
            for(unsigned i = 0; i < source.size(); i++)
            {
                if(source[i].extensionflag > 0 && n1 == 0)
                {
                    ++countExtended[filter];
                    ++n1;
                }

                if(source[i].qualityflag > 0 && n2 == 0)
                {
                    ++countQuality[filter];
                    ++n2;;
                }

                if(source[i].confusionflag > 0 && n3 == 0)
                {
                    ++countConfusion[filter];
                    ++n3;
                }

                /***************************************************************************************
		* Count how many sources have all flags equal to zero
                *
                ***************************************************************************************/

                if(source[i].extensionflag == 0 && source[i].qualityflag == 0 && source[i].confusionflag == 0)
		{
		  ++n4;

                }          

            }
            if(n4 > 0) ++countFlagsZero[filter];            

	    //          cout << "Filter " << filter << endl;
	    //       cout << "countextened=" << countExtended[filter] << " quality=" << countQuality[filter] << " confusion=" << countConfusion[filter] << endl;

        }

    }

}


/******************************************************************************
*
*
* Loop through each source in the combined source-list and check for
* variable ones 
*
*
*******************************************************************************/
void computeVariabilityParameters(SWSData &swsData, VariabilityData &varData)
{

 
    /********************************************************************
    *
    * Sort the parameters into increasing time order
    *
    ********************************************************************/
    sort( swsData._mjdStartList.begin(), swsData._mjdStartList.end() );
    sort( swsData._mjdEndList.begin(),   swsData._mjdEndList.end() );

    int size = (int)swsData._mjdStartList.size();
    varData._mjdMid =   (swsData._mjdEndList[size - 1] + swsData._mjdStartList[0] ) / 2.0; 
     


   

  
    /********************************************************
    *
    * Compute some statistics
    *
    *********************************************************/


      
    varData.min = minVal( swsData._mag );
    varData.max = maxVal( swsData._mag );
                             

  
    varData._eFlagMax = maxVal( swsData._eFlag );
    varData._cFlagMax = maxVal( swsData._cFlag );
    varData._qFlagMax = maxVal( swsData._qFlag );

    //    varData._obsIdList = swsData._obsIdList[srcId];


    //    cout << "compute variability parameters lin 4735- srcid=" << srcId << endl;
    //    for(unsigned i = 0; i < swsData._obsIdList[srcId].size(); i++)
    //  {
    //  cout << i << ") obsid=" << swsData._obsIdList[srcId][i] << endl;
    // }

 

    /**********************************************************************
    *
    * Compute statistics using corrected count rates
    *
    **********************************************************************/ 

    float median = Median( swsData._corrCountRate );
 
    varData.median = median;


   
    /***************************************************************
    * Compute mean ra and dec
    *
    ***************************************************************/         
    //    cout << "COMPUTEVARIABILITYPARAMETERS LINE 4745\n";
    //    checkSource1( -1, swsData._ra, swsData._dec ) ;


    averageCelestialCoordinates( swsData._ra, swsData._dec, varData._meanRa, varData._meanDec);


    //    cout << "Line 4657 averaged variability data ra=" << varData._meanRa << " dec=" << varData._meanDec << endl;
 

     
    /****************************************************************************
    *
    * Find out the maximum number of standard deviations away from the median
    *
    * corrected count-rate
    ****************************************************************************/            
    float maxNstd = 0.0;
 
    
    float minMag = 10000.0;
    float maxMag = -minMag;

    float minRate = 10000.0;
    float maxRate = -minRate;

    for(unsigned i = 0; i < swsData._mag.size(); i++)
    {
        minMag = min(minMag, swsData._mag[i]);
        maxMag = max(maxMag, swsData._mag[i]);
        minRate = min(minRate, swsData._corrCountRate[i]);
        maxRate = max(maxRate, swsData._corrCountRate[i]);

        maxNstd = max( maxNstd, abs( median - swsData._corrCountRate[i] ) / max( (float)0.00000000001, swsData._corrCountRateErr[i]) ); 
         
    }

    /********************************************************************
    *
    * Compute the reduced chi-squared and probability
    *
    ********************************************************************/
    varData.chiSquared = computeChiSquared(swsData._corrCountRate, 
                                           swsData._corrCountRateErr, 
                                           varData.mean, varData.prob);
 



    /*********************************************************************
    *
    * Get the time duration
    *
    **********************************************************************/
	//    varData.timeLength = swsData._[source1.size() - 1].stime - source1[0].stime;                  
        
          
    varData._nstd = maxNstd;
   

} 
/******************************************************************
*
*
* Adds the variability columns for a given filter
* size is the number of rows in the mastercatalogue
*
*
*******************************************************************/
void addVariabilityColumn( string filter,                         // The filter
                           map<int, VariabilityData> &varData, string subHeading)    // The list of variability parameters for each SRCID  
                           
{

 
  

    /**************************************************************
    *
    * Add the columns to the output catalogue
    *
    **************************************************************/

    string outputFile = stringParameter( "obsset" );


    if ( outputFile.length() < 3)
    {
      return; 

    }




    if ( !QFile::exists(outputFile.c_str()))
    {
      return; 

    }



        cout << "Adding variability columns for filter " << filter << " to " << outputFile << endl;



    /******************************************************************
    *
    * 
    *
    *******************************************************************/
    DataSet * set = dataSetServer->open( outputFile, DataSet::Modify);
    Table *table = set->table("SRCLIST");

    int size = table->rows();



    /********************************************************************
    *
    * Need to load the SRCID column from the OBS file
    *
    ********************************************************************/
    vector<int> srcId;
    LoadColumnData( table,  "SRCID", srcId);
    //    dataSetServer->close( set); // Finished with the file



    
    map<int, int> srcNo;
          
    for(int i = 0; i < size; i++)
    {
        srcNo[ srcId[i] ] = i; 
	//	cout << i << ") srcid=" << srcId[i] << endl;
    }




    vector<float> chi(size, -9999.9);     // For chi-squared in output catalogue file
    vector<float> maxDev(size, -9999.9);  // for MAXDEV




    map<int, VariabilityData>::iterator iter;
    for(iter = varData.begin(); iter != varData.end(); ++iter)
    {
        int number = iter->first; // NB 0-size - 1
        int i = srcNo[number];
        VariabilityData data = iter->second;
        chi[i] = data.chiSquared;
        maxDev[i] = data._nstd;       

	//  cout << "Line 4649 srcid=" << i << " chi=" << chi[i] << " maxdev=" << maxDev[i] << endl;

    }


    string columnName = filter + "_CORR_RATE_ERR"; 
  
    int position = table->columnNumber( columnName ) + 1; 
 

    columnName = filter + "_CHI2" + subHeading; 


    string units = "";
    string comment = "Variability chi**2 statistic (" + filter + " filter)";


    /********************************************************
    *
    *
    *
    *
    ******************************************************/


    if (!addColumn( table,   
                    (char *)columnName.c_str(), 
                    (char *)units.c_str(),
                    (char *)comment.c_str(),   
	            size,
	            chi,
                    position) )
    {

    }
 

    position = table->columnNumber( columnName ) + 1; 
 
    columnName = filter + "_MAXDEV" + subHeading; 
    cout << "Adding column " << columnName << endl;


    comment = "Max. dev. from median count-rate (" + filter + " filter)";

    units = "sigma";


    /********************************************************
    *
    *
    *
    *
    ******************************************************/
    if (!addColumn( table, 
                    (char *)columnName.c_str(), 
                    (char *)units.c_str(),
                    (char *)comment.c_str(),   
	            size,
	            maxDev,
                    position))
    {

    }


    cout << "Added column " << columnName << endl;    


    dataSetServer->close( set); // Finished with the file


}
/*******************************************************************
*
*
*
*
********************************************************************/
void stetsonVariabilityData( map<int,  map<string, VariabilityData, OmFilter> > &stetsonDataMap)
{

    cout << "Computing Stetson variability data- size(stetsonDataMap=" << stetsonDataMap.size() << endl;
    map<int,  map<string, VariabilityData, OmFilter> >::iterator iter;

    for( iter = stetsonDataMap.begin(); iter != stetsonDataMap.end(); ++iter)
    {
        int srcId = iter->first;
        cout << "Source ID " << srcId << endl;


        map<string, VariabilityData, OmFilter>  theData = iter->second;
        cout << "Number of filters for this source=" << theData.size() << endl;

        if (theData .size() > 1)
	{
	    if ( computeStetsonVariabilityData( theData ) )
	    {

            }

        }
    }
}

/*****************************************************************************
*
*
*
******************************************************************************/
bool computeStetsonVariabilityData( map<string, VariabilityData, OmFilter>  &stetsonDataMap)
{

    
    cout << "Line 3924 Computing stetson variability data\n";
    map<string, VariabilityData, OmFilter>::iterator iter = stetsonDataMap.begin();

  

    VariabilityData varData = iter->second;




    string filter = iter->first;
    double mjd1 = varData._mjdMid ;


    int k = 1;
  
    cout << k << ") Filter " << filter << " mjd=" << mjd1 << endl;
  

    ++iter;
    map<string, VariabilityData, OmFilter> theData;
    theData[filter] = varData;


    for( ; iter != stetsonDataMap.end(); ++iter, ++k)
    {

        VariabilityData varData = iter->second;
        string filter = iter->first;
        double mjd2 = varData._mjdMid;
        double timeDiff =  mjd1 - mjd2;
        if(abs( timeDiff ) < 1000000.0)
	{
	    cout << k << ") Filter " << filter << " mjd=" << mjd2 << " time diff=" << timeDiff << endl;
            theData[filter] = varData;

        }


    }

    if( theData.size() < 2)
    {
        return false;

    }


    //    double magSum = 0.0;
    //    double sumWeight = 0.0;
    for(iter = theData.begin(); iter != theData.end(); ++iter)
    {

        VariabilityData data = iter->second;
        SWSData data1 = data._swsData;
	vector<float> mag = data1._mag;
	vector<int> srcId = data1._srcId;
        vector<double> mjd = data1._mjdMidList;
        cout << "The mags-\n";
        for(unsigned i = 0; i < mag.size(); i++)
	{
	  cout << i << ") srcId=" << srcId[i] << " mag=" << mag[i] << "mjd="  
               << setiosflags(ios::fixed) << setprecision(5)
	       << mjd[i] << endl;
        }        

    }

    return true;

}
/****************************************************************************
*
* Check to make sure that the files are either all SWS source-list files
* or all observation source-list files
*
*****************************************************************************/
void checkInputFiles(vector<string> &fileList, string &obsId, int &numObservations)
{

    vector<string> newFileList;
    map<string, int> obsIdMap;


    numObservations = 0;
    // Check the first file
    int k = -1;
    DataSet * set = 0;

    //   cout << "checkinputfiles line 2863- size=" << fileList.size() << endl;   
    for(unsigned i = 0; i < fileList.size(); i++)
    { 
        set = dataSetServer->open(fileList[i], DataSet::Read);
   
        if ( set->hasAttribute( "OBS_ID") )
        {

            const Table *table = set->table("SRCLIST");
  
            if( table->rows() >  0)        
	    {

	        if( table->hasColumn( "RA" ) )        
		{
	            newFileList.push_back( fileList[i] );
	            k = i;
	            break;
                }
            }
        }
        dataSetServer->close(set);
       
 
    }


    //  cout << "checkinputfiles line 2890- size=" << fileList.size() << endl;   
  

    if( k == -1)
    {
      numObservations = 0;
      return;

    }   


    obsIdMap[ set->attribute( "OBS_ID")->asString() ] = 1;


    obsId =  set->attribute( "OBS_ID")->asString(); 


    dataSetServer->close(set);

  
 


    for(unsigned i = k + 1; i < fileList.size(); i++)
    {

        DataSet * set = dataSetServer->open(fileList[i], DataSet::Read);

        if ( set->hasAttribute( "OBS_ID") )
        {

            const Table *table = set->table("SRCLIST");
  
            if( table->rows() >  0)        
	    {

	        if( table->hasColumn( "RA" ) )        
		{
	 
	            newFileList.push_back( fileList[i] );

                    string filter = set->attribute( "FILTER")->asString();
                    obsIdMap[ set->attribute( "OBS_ID")->asString() ]++;

                    if( filter == "n/a" || filter == "N/A" )
	            {
                        errstr << fileList[k] << " and " << fileList[i] 
                               <<  " are different kinds of source-list files"
		               << Sas::fatal("differentkindsofsourcelistfiles");
              
		    }
                }
            }

        }    
        dataSetServer->close(set);
        
    }

    numObservations = obsIdMap.size();

    fileList = newFileList;

    //  cout << "checkinputfiles line 2953- size=" << fileList.size() << endl;   
  


}

  
/**********************************************************************
*
* Loads data from an OM observation source-list file
*
*
*
***********************************************************************/
bool loadObservationSourceListFileData( string file,   vector < OmSourceParameter > &srcList)
{

    DataSet *dataSet = dataSetServer->open( file, Dal::Read);

    if (!dataSet) // Couldn't open the file
    {
        return false;

    }    
 
    const Table *table = dataSet->table( "SRCLIST", Dal::Read);
    if (!table) // Couldn't read the table
    {
        dataSetServer->close(dataSet); // Finished with the file
        return false;

    }


    /**********************************************************************
    *
    * Load the RAs and Decs
    *
    **********************************************************************/
    vector<double> data;
    int size = 0;

    if(table->hasColumn("RA_CORR"))
    {
        if ( !LoadVectorDoubleColumnData( table, "RA_CORR", size, data))
        {

            dataSetServer->close(dataSet); // Finished with the file

            Sas::fatal("Error loading column from table-", "RA_CORR");

        }


        srcList.resize(size);
        for(int i = 0; i < size; i++)
            srcList[i].rightascension = data[i];


        data.clear();
        if ( !LoadVectorDoubleColumnData( table, "DEC_CORR", size, data))
        {
            dataSetServer->close(dataSet); // Finished with the file
            Sas::fatal("Error loading column from table-", "DEC_CORR");


        }
        for(int i = 0; i < size; i++)
            srcList[i].declination = data[i];
    }
    else
    {

    
        if ( !LoadVectorDoubleColumnData( table, "RA", size, data))
        {

            dataSetServer->close(dataSet); // Finished with the file
            Sas::fatal("Error loading column from table-", "RA");


        }


        srcList.resize(size);
        for(int i = 0; i < size; i++)
            srcList[i].rightascension = data[i];

        data.clear();
        if ( !LoadVectorDoubleColumnData( table, "DEC", size, data))
        {

            Sas::fatal("Error loading column from table-", "DEC");


        }
        for(int i = 0; i < size; i++)
            srcList[i].declination = data[i];
   
    }


    /*******************************************************************
    *
    * Need to get some significances
    *
    ********************************************************************/
    map<string, vector<float> > significanceMap;


    for(int i = 0; i < 7; i++)
    {
        string filter = OmFilter::filter[i];
        vector<float> significance(srcList.size(), -1.0);
 

        string name = OmFilter::filter[i] + "_SIGNIF";
        if(table->hasColumn(name) )
	{
	  //    cout << "Loading significances from column " << name << endl;

	    //   vector<float> data, data1, data2;
            vector<float> data;

            int size = 0;
            if ( !LoadVectorFloatColumnData( table, name, size, data))
            {
            }
            significanceMap[filter] = significance;


            for(int i = 0; i < size; i++)
	    {
	        if (!isNull( data[i] ) )
                    significanceMap[filter][i] = data[i] ;
            }
   
                     
           
        }
        name = OmFilter::filter[i] + string("_SIGNIFICANCE");
        if(table->hasColumn(name) )
	{
	  //  cout << "Loading significances from column " << name << endl;

	    vector<float> data;
            int size = 0;
            if ( !LoadVectorFloatColumnData( table, name, size, data))
            {


            }
            significanceMap[filter] = significance;
      
            for(int i = 0; i < size; i++)
	    {     
                if (!isNull( data[i] ) )
                    significanceMap[filter][i] = data[i];
       
            } 
     
        }
    }



    dataSetServer->close(dataSet); // Finished with the file
 
    map<string, vector<float> >::iterator iter;


    vector<float> significance(srcList.size(), -1.0);
 
    for(unsigned i = 0; i < srcList.size(); i++)
      srcList[i].significance = -1.0;
    

    for(iter =  significanceMap.begin(); iter != significanceMap.end(); ++iter)
    {
        vector<float> data = iter->second;
        for(unsigned i = 0; i < data.size(); i++)
	    srcList[i].significance = max(   srcList[i].significance, data[i] );
          

    }

    return true; 


}
 
/*****************************************************************************
*
*  
* 
*
*
*
*
*
******************************************************************************/
void averageObservationMagnitudes( map<string, map<string, vector<SWSData>, 
                                   OmFilter > > & obsMapFilterMapData,
                                   map<string, SWSData, OmFilter > &averagedFilterDataMap)

{
  
 
    cout << "averageObservationMagnitudes line 5429\n";


    map<string, map<string, vector<SWSData>, OmFilter > >::iterator iter;


    /******************************************************************
    *
    * Loop through the observations
    *
    *******************************************************************/
 
    for(iter =  obsMapFilterMapData.begin(); iter != obsMapFilterMapData.end(); ++iter)
    {
        string obsId = iter->first; // The observation ID
      
        
        map<string, vector<SWSData>, OmFilter >  filterData = iter->second;
        map<string, vector<SWSData>, OmFilter >::iterator iter1;
   


        /****************************************************************
	*
        * Loop through the filters for this observation
        *
        *****************************************************************/


        for(iter1 = filterData.begin(); iter1 != filterData.end(); ++iter1)
        {
	    string filter = iter1->first;

           

            /**********************************************************
	    * iter1->second is the list of SWS data for this filter
            *
            * Need to average the magnitudes for all the sources with the
            * same ID
            ***********************************************************/
            averageData( iter1->second, averagedFilterDataMap[filter] );            


         
        }

    }


}

/*****************************************************************************
*
*  This routine is only used when there are more than one observations
*  to be processed. In this case all the magnitudes for a particular
*  unique source and filter need to be averaged so that variability
*  parameters can be computed.
*
*  SWSData is a vector (one element for each source-list file)
*  storing parameters for each one in vector form
*
*
*
******************************************************************************/
void averageData( vector<SWSData> &data, SWSData &averagedData)
{
  

    map<int, SWSData > dataToAverage;

    map<int, vector<double> > startTime;
    map<int, vector<double> > endTime;
    map<int, vector<double> > midTime;

    /********************************************************
    *
    * Loop through all the source-list files for the filter
    *
    *********************************************************/

    for(unsigned i = 0; i < data.size(); i++)
    {
           
        /********************************************************
	* Loop through all the sources in the file
        *
        *********************************************************/
        for(unsigned j = 0; j < data[i]._srcId.size(); j++)
	{ 
	    int k = data[i]._srcId[j]; // The unique source id
	    //    if(k != 63) continue;

          
	    //    //	    cout << j << ") averagedata line 5505, srcid 21- ra= " <<  setiosflags(ios::fixed) << setprecision(5) <<  data[i]._ra[j] << " dec=" << data[i]._dec[j]  << endl;


            /*************************************************
	    *
            * Need a list of parameters for each of the sources
            * from the files with the same ID
            * Store as a map
            *************************************************/

            dataToAverage[k]._ra.push_back( data[i]._ra[j] );
            dataToAverage[k]._dec.push_back( data[i]._dec[j] );
            dataToAverage[k]._mag.push_back( data[i]._mag[j] );
            dataToAverage[k]._magErr.push_back( data[i]._magErr[j] );
            dataToAverage[k]._corrCountRate.push_back( data[i]._corrCountRate[j] );
            dataToAverage[k]._corrCountRateErr.push_back( data[i]._corrCountRateErr[j] );
            dataToAverage[k]._significance.push_back( data[i]._significance[j] );
            dataToAverage[k]._eFlag.push_back( data[i]._eFlag[j] );
            dataToAverage[k]._qFlag.push_back( data[i]._qFlag[j] );
            dataToAverage[k]._cFlag.push_back( data[i]._cFlag[j] );
            midTime[k].push_back( data[i]._mjdMid );
            startTime[k].push_back( data[i]._mjdStart );
            endTime[k].push_back( data[i]._mjdEnd );

            /**************************************************************
	    *
            * Need to record the observation ID
            *
            **************************************************************/
	    averagedData._obsIdList[k].push_back( data[i]._obsId );


     
        }

    } // End of looping through the data parameters for each file

 

    //    averageTimes(data, averagedData);

    //  averagedData._mjdStart = data[0]._mjdStart;
    //  averagedData._mjdEnd   = data[0]._mjdEnd ;
    // averagedData._mjdMid   = data[0]._mjdMid ;



    /*****************************************************************
    *
    * Loop through the map of source IDs and do the averaging, etc
    *
    ******************************************************************/
      SWSData data1 = data[0];


    map<int, SWSData >::iterator iter;

    for(iter = dataToAverage.begin(); iter != dataToAverage.end(); ++iter)
    {
        int k = iter->first; // The unique-source ID
	//	if(k != 63) continue;  
           

	//    cout << "Line 5637 sourceiD=" << k << endl;
    
        float meanMag = 0.0;
        float meanMagErr = 0.0;

        float meanRate = 0.0;
        float meanRateErr = 0.0;

        /*********************************************************************
	*
        *
        * Now average the magnitudes and corrected count-rates
        *
        *
        **********************************************************************/
        averageMagnitudes(dataToAverage[k]._mag, 
                          dataToAverage[k]._magErr, 
                          meanMag, meanMagErr, 
                          dataToAverage[k]._corrCountRate, 
                          dataToAverage[k]._corrCountRateErr, 
                          meanRate, meanRateErr);

    
        averagedData._srcId.push_back( k );
                         

        double raMean = 0.0, decMean = 0.0;
        averageCelestialCoordinates( dataToAverage[k]._ra, 
                                     dataToAverage[k]._dec, 
                                     raMean, decMean);


   
	//        checkSource1( k, dataToAverage[k]._ra, dataToAverage[k]._dec );

        /*******************************************************
	*
        * Store the mean, etc
        *
        ********************************************************/


	averagedData._ra.push_back( raMean );                         
	averagedData._dec.push_back( decMean );

	averagedData._mag.push_back( meanMag );
        averagedData._magErr.push_back( meanMagErr );


       
         averagedData._corrCountRate.push_back( meanMag );
         averagedData._corrCountRateErr.push_back( meanMagErr );
       

         /******************************************************************
	 *
         *  NB NEED TO SORT FLAGS OUT
         *
         ******************************************************************/

	 averagedData._eFlag.push_back( dataToAverage[k]._eFlag[0] );
	 averagedData._cFlag.push_back( dataToAverage[k]._cFlag[0] );
	 averagedData._qFlag.push_back( dataToAverage[k]._qFlag[0] );

	 averagedData._significance.push_back( dataToAverage[k]._significance[0] );



         /************************************************************************
	 *
         * Compute the mean mid-time of the source exposure
         *
         ************************************************************************/

         double meanStartTime = Mean( startTime[k], false );

	 averagedData._mjdStartList.push_back( meanStartTime );

         double meanEndTime = Mean( endTime[k], false );

	 averagedData._mjdEndList.push_back( meanEndTime );



         double meanMidTime = Mean( midTime[k], false );

	 averagedData._mjdMidList.push_back( meanMidTime );

	 //     cout << "line 5611 meanmidtime=" << meanMidTime << endl;



	 //	 averagedData._mjdStartList.push_back( data[0]._mjdStart );
	 // averagedData._mjdEndList.push_back( data[0]._mjdEnd );
	 // averagedData._mjdMidList.push_back( data[0]._mjdMid );
    


                     
                            
    } 

    averagedData._obsId = data[0]._obsId ;
    
  

}
/*********************************************************************
*
*
*
* Executes omsrclistcomb
*
*
*
********************************************************************/
void executeOmsrclistcomb( vector<string> &fileList )
{

     
    string command = string("omsrclistcomb sourcelistsets='") + fileList[0];

    for(unsigned i = 1; i < fileList.size(); i++)
    { 
        command += string(" ");
        command += fileList[i];
    }

    string fileName = stringParameter("obsset");

    bool delete1 = false;
    cout << "L3299 fileName=" << fileName << endl;

    if(fileName.length() > 2)
    {
        if( QFile::exists( fileName.c_str() ) )
        {
            errstr <<  fileName << " exists- will be overwritten- sleeping for 5 seconds" 
	           << Sas::warning("fileExists");
            	sleep(5);
        }
      
    }
    else
    {
        delete1 = true; // Don't want to keep the variability data

        // Get a unique file name
        char name [L_tmpnam];
   
        tmpnam ( name );
        fileName = name;


    }

    command += string("' outset=") + string(fileName);
  


    int k = system(command.c_str() );

    if(delete1)
    {
        remove( fileName.c_str() );
    }
    if( k != 0)
    {

        errstr << "Failure executing omsrclistcomb\n"
               << command
	       << Sas::fatal("executionfailure");
 
    }

    
}
/*********************************************************************
*
*
* Averages magnitudes
*
***********************************************************************/
void averageMagnitudes(vector<float> &magnitude, vector<float> &magErr, 
		       float &meanMag, float &meanMagErr,
                       vector<float> &rate, vector<float> &rateErr, 
		       float &meanRate, float &meanRateErr)


{
   
  

  
    if ( magnitude.size() == 1) 
    {
        meanMag = magnitude[0];
        meanMagErr = magErr[0];

        meanRate = rate[0];
        meanRateErr = rateErr[0];

        return;
    }



    double sum1 = 0.0, sum2 = 0.0;
    double f2 = 0.0;
    double mag2 = 0.0; 
   
   
    for(unsigned i = 0; i < magnitude.size(); i++)
    {

        
        if (magnitude[i] >= 0.0 && magnitude[i] < 100.0 && !isNull( magnitude[i] ) ) 
	{
          
	  //	    double flux = pow(10.0, (-0.4 * magnitude[i]));                 
	  //   double fluxError = flux * max(0.01f, magErr[i]);

	    double flux = (double)rate[i];                 
            double fluxError = (double)rateErr[i];

            f2 = flux;
            mag2 = (double)magnitude[i];




            double weight = (double)1.0 / ( fluxError * fluxError);
    
            sum1 += weight;
            sum2 += flux * weight;
   
        }
					      }
     
    if(sum1 > 0.0) 
    {
        meanRate = sum2 / sum1; // The mean

        /***********************************************************
	* The uncertainty in the weighted mean
        ************************************************************/
    
        meanRateErr = sqrt( 1.0 / sum1); 
    
        meanMagErr = max(0.01, 1.085735 * meanRateErr / meanRate);
 
  
        meanMag = -2.5 * log10(meanRate / f2) + mag2;;


    }
    else
    {
        meanMag = magnitude[0];
        meanMagErr = magErr[0];
    }

    static int count = 0;
    if(count < 10)
    {
      count++;
      cout << "\n******averagemagnitudes\n";
      for(unsigned i = 0; i < magnitude.size(); i++)
      {
	cout << i << ") mag=" << magnitude[i] << "+/-" << magErr[i] << endl;
	cout << i << ") rate=" << rate[i] << "+/-" << rateErr[i] << endl;



      }
      cout << "meanmag=" << meanMag << "+/-" << meanMagErr << endl;
      cout << "meanrate=" << meanRate << "+/-" << meanRateErr << endl;


    }


}
/*********************************************************************
*
*
*
*********************************************************************/
void averageTimes( vector<SWSData> &data,  SWSData &averagedData)
{
    double sum1 = 0, sum2 = 0;
    

    cout << "Average times line 6133\n";
    for(unsigned i = 0; i < data.size(); i++)
    {
        sum1 +=  data[i]._mjdStart;
        sum2 +=  data[i]._mjdEnd;
      

        cout << i << ") start= " << data[i]._mjdStart <<
                     " end "  << data[i]._mjdEnd << 
	             " mid "  << data[i]._mjdMid << endl; 



    }
    double size = (double)data.size();
    averagedData._mjdStart = sum1 / size;
    averagedData._mjdEnd   = sum2 / size;
    averagedData._mjdMid   =  (averagedData._mjdStart + averagedData._mjdEnd) / 2.0;

    cout << "Averaged start=" <<  averagedData._mjdStart << endl;
    cout << "Averaged end=" <<  averagedData._mjdEnd << endl;

  cout << "Averaged mid=" <<  averagedData._mjdMid << endl;

}
/*****************************************************************
*
* Want the median mag value to be centred at the middle of the y-axis
*
******************************************************************/
void getNiceMagRange(const float median, const float yMin1, const float yMax1, float & yMin2, float &yMax2 )
{

  // cout << "getnicerange Line 5648 ymin1=" << yMin1 << " ymax1=" << yMax1 << endl;
  

    if( (yMax1 - yMin1) < 2.0)
    {

        float dy1 = yMax1 - median;
        float dy2 = median - yMin1;
        float dy = max(0.1f, max(dy1, dy2));
	//      cout << "Line 5655- median=" << median << " dy1=" << dy1 << " dy2=" << dy2 << endl;
        yMin2 = median - dy;
        yMax2 = median + dy;
	//      cout << "Line 5656 ymin2=" << yMin2 << " ymax2=" << yMax2 << endl;
        yMin2 = floor(yMin2 * 10.0) / 10.0;
        yMax2 = ceil(yMax2 * 10.0) / 10.0;


	//      cout << "Line 5662 ymin2=" << yMin2 << " ymax2=" << yMax2 << endl;
  



    }
    else
    {
        yMin2 = floor(yMin1 * 5.0) / 5.0;
        yMax2 = ceil(yMax1 * 5.0) / 5.0;
    }



    //    cout << "line 5673 ymin1=" << yMin1 << " ymax1=" << yMax1 << endl;
    // cout << "ymin2=" << yMin2 << " ymax2=" << yMax2 << endl;





}

/*******************************************************
*
* FOR TESTING
*
********************************************************/
void  checkData( map<string, map<string, vector<SWSData>, OmFilter > > dataMap)
{  
    map<string, map<string, vector<SWSData>, OmFilter > >::iterator iter;


    cout << "CHECKDATA- checking all the counterpart sources\n";
    for(iter = dataMap.begin(); iter != dataMap.end(); ++iter)
    {

        string obsId = iter->first;
        cout << "Checking data for OBSID " << obsId << endl;

        map<string, vector<SWSData>, OmFilter>  data = iter->second;

        map<string, vector<SWSData>, OmFilter > ::iterator iter1;


        map<int, vector<int> > srcIdMap;
        map<int, vector<double> > raMap;
        map<int, vector<double> > decMap;


        for(iter1 = data.begin(); iter1 != data.end(); ++iter1)
        {


            string filter = iter1->first;
            cout << "Checking data for filter " << filter << endl;

            vector<SWSData> swsData = iter1->second;
            for(unsigned i = 0; i < swsData.size(); i++)
	    {
                for(unsigned j = 0; j < swsData[i]._srcId.size(); j++)
		{
	            int number = swsData[i]._srcId[j];

                    srcIdMap[number].push_back( number );                         
	            raMap[number].push_back( swsData[i]._ra[j] );                         
	            decMap[number].push_back( swsData[i]._dec[j] );                         
	        


                }

            }


        }
        map<int, vector<int> >::iterator iter3;
        for(iter3 = srcIdMap.begin(); iter3 != srcIdMap.end(); ++iter3)
	{
	  if( iter3->second.size() > 1)
	  {
            int number = iter3->first;  
            vector<double> raList = raMap[number];
            vector<double> decList = decMap[number];

	    checkSource1( number, raList, decList);


          }

        }
        


 

    }
}
/****************************************************************
*
* FOR TESTING PURPOSES
*
****************************************************************/
void checkSource( int number, vector<double> &ra, vector<double> &dec )
{

  if(number != 21) return;


    double ra0 = ra[0] * degreesToRadians;
    double dec0 = dec[0] * degreesToRadians;

   
    for(unsigned i = 1; i < ra.size(); i++)
    {

      double d = distance( ra0, dec0, ra[i] * degreesToRadians, dec[i] * degreesToRadians);
      if(d > 2.0)
      {
	  cout << "CHECKING RAS AND DECS FOR SRD-ID " << number << endl;


	cout << "POSSIBLE BUG- DISTANCE=" << d << " arcsecs\n";

        ++numBugs;
        maxDistance = max(maxDistance, d);
        for(unsigned j = 0; j < ra.size(); j++)
	{
      
            
	  cout << j << ") ra= " <<  setiosflags(ios::fixed) << setprecision(5) <<  ra[i] << " dec=" << dec[i]  << endl;

   

        }

        break;

      }

    }

}
/****************************************************************
*
*
****************************************************************/
void checkSource1( int number, vector<double> &ra, vector<double> &dec )
{

  if(number != 21) return;

    double ra0 = ra[0] * degreesToRadians;
    double dec0 = dec[0] * degreesToRadians;

    cout << "\n\n\nCHECKSOURCE1 - SRC-ID " << number << " RAS AND DECS\n";





    for(unsigned j = 0; j < ra.size(); j++)
    {
      
         cout << j << ") ra= " <<  setiosflags(ios::fixed) << setprecision(5) <<  ra[j] << " dec=" << dec[j]  << endl;

   

   }

  


    double maxDistance = 0.0;
    for(unsigned i = 1; i < ra.size(); i++)
    {

      double d = distance( ra0, dec0, ra[i] * degreesToRadians, dec[i] * degreesToRadians);
    
      maxDistance = max(maxDistance, d);
           

    }
    cout << "Maximum separation=" << maxDistance << " arcsecs\n";

}

/********************************************************************
*
*
*
********************************************************************/
void openPgplot()
{

    if ( pgplotInitialised) {
        return;
    }

    string plotFile = stringParameter( "plotfile" ) + "/CPS";

    int k = cbpgopen( (char *)plotFile.c_str());
    if(k) {};
  
    SetUpDefaultColourIndices();

    int nxSub = intParameter("nxsub");
    int nySub = intParameter("nysub");

    if(nxSub == 1) 
    {
        maxNumGraphs = nySub;
    }
    else
    {
        maxNumGraphs = nySub;
        cbpgsubp(nxSub, 1);

    }

    pgplotInitialised = true;
  
}
/****************************************************************
*
*
****************************************************************/
void checkSource1( int number, vector<float> &ra, vector<float> &dec )
{

  if(number != 21) return;


    double ra0 = ra[0] * degreesToRadians;
    double dec0 = dec[0] * degreesToRadians;

    cout << "\n\n\nCHECKSOURCE1 - SRC-ID " << number << " RAS AND DECS\n";





    for(unsigned j = 0; j < ra.size(); j++)
    {
      
         cout << j << ") ra= " <<  setiosflags(ios::fixed) << setprecision(5) <<  ra[j] << " dec=" << dec[j]  << endl;

   

   }

  


    double maxDistance = 0.0;
    for(unsigned i = 1; i < ra.size(); i++)
    {

      double d = distance( ra0, dec0, (double)ra[i] * degreesToRadians, (double)dec[i] * degreesToRadians);
    
      maxDistance = max(maxDistance, d);
           

    }
    cout << "Maximum separation=" << maxDistance << " arcsecs\n";

}
/******************************************************************************
*
*
*
*****************************************************************************/
void checkObsFile( string obsId, vector<string> &fileList )
{

    cout<<"Checking"<<endl;

    string fileName = stringParameter( "obsset");
    if(fileName.length() < 3) {
      return;
    }

    cout<<"Checking again"<<endl;

    if ( !QFile::exists( fileName.c_str() ))
    {
    cout<<"Soerint it"<<endl;
        executeOmsrclistcomb( fileList );

        if ( !QFile::exists( fileName.c_str() ))
        {
      
            errstr << "The observation file " << fileName << " does not exist"
	           << Sas::fatal("filedoesnotexist");
        }    
  

    }

  



    cout<<"Sortng u7t7et it"<<endl;

    DataSet * set = dataSetServer->open(fileName, DataSet::Modify);
    cout<<"Finished opening it"<<endl;
    string obsId1 = "xxx";
    cout<<"Sorted it"<<endl;
    if( set->hasAttribute( "OBS_ID")) {
        obsId1 =  set->attribute( "OBS_ID")->asString(); 
    }
    else
    {
        errstr << "The observation file has no FITS keyword OBS_ID"
	       << Sas::fatal("cantcontinue");
   
 

    }
    dataSetServer->close( set); // Finished with the

    if( obsId1 != obsId) {
   
      errstr << "The observation file has a different OBS_ID (" << obsId1
             << ") to that of the source-list files (" << obsId << ")"  
	       << Sas::fatal("cantcontinue");
   

    }

    cout<<"End of Checking"<<endl;
}
