// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef OMVARIABILITY_DEFS_H
#define OMVARIABILITY_DEFS_H


/*****************************************************************************
*
*
*
*
******************************************************************************/
#include <qstring.h>
#include <qdir.h>
#include <qdatetime.h>
#include<vector>
#include<map>

#include "omfilter.h"


#include<cstdio>
#include<fstream>

#include "f90cxx.h"




#include <string>

#include <cstdio>
#include <ctime>



/************************************************************
*
* For sorting structure  OmSourceParameter into
* increasing time order
*
************************************************************/
bool compare(const OmSourceParameter &a, const OmSourceParameter & b)
{

    return a.stime < b.stime;

};





//Sas::fatal("Different image types-", "Cannot process images of different types!");




vector<string> tempVector(0);
/*************************************************************
*
*
*
*************************************************************/
class SWSData
{
public:
 
   SWSData() : _meanRa(0), _meanDec(0),
             _mjdStart(0), _mjdEnd(0),
	       _mjdMid(0),
             _obsId(""), _filter(""),
             _object(""), _fileName("")  {}; 

   SWSData(  double meanRa, double meanDec,
	     double mjdStart, double mjdEnd,
	     double mjdMid,
	     string obsId,
             map<int, vector<string> > obsIdList,
	     string filter,
	     string object,
             string fileName, 
	     vector<int> srcId,
	     vector<double> mjdStartList, vector<double> mjdEndList, vector<double> mjdMidList,
	     vector<double> ra, vector<double> dec,
	     vector<float> significance,
	     vector<float> mag, vector<float>magErr,
	     vector<float> corrCountRate, vector<float>corrCountRateErr,
	     vector<int8> eFlag, vector<int8> cFlag,
	     vector<int16> qFlag) : 
        
             _meanRa(meanRa), _meanDec(meanDec),
	     _mjdStart(mjdStart), _mjdEnd(mjdEnd),
	     _mjdMid( mjdMid),
	     _obsId(obsId),
             _obsIdList(obsIdList),
	     _filter(filter),
	     _object(object),
             _fileName(fileName), 
	     _srcId(srcId),
	     _mjdStartList( mjdStartList), _mjdEndList(mjdEndList), _mjdMidList(mjdMidList),
	     _ra(ra), _dec(dec),
	     _significance(significance),
	     _mag(mag), _magErr(magErr),
	     _corrCountRate(corrCountRate), _corrCountRateErr(corrCountRateErr),
	     _eFlag(eFlag), _cFlag(cFlag),
     _qFlag(qFlag) {}; 
        




    SWSData(const SWSData &s);

   ~SWSData() {};
  

/*********************************************************
*
*
*
**********************************************************/
SWSData operator = (const SWSData &s)
{
    _meanRa = s._meanRa;
    _meanDec = s._meanDec;
    _mjdStart = s._mjdStart;
    _mjdEnd = s._mjdEnd;
    _obsId = s._obsId;
    _obsIdList = s._obsIdList;



    _mjdMid = s._mjdMid;

    _filter = s._filter;
    _object = s._object;
    _fileName = s._fileName;


    _srcId = s._srcId;
    _mjdStartList = s._mjdStartList;
    _mjdEndList = s._mjdEndList;
    _mjdMidList = s._mjdMidList;


    _ra = s._ra;
    _dec = s._dec;
    _significance = s._significance;
    _mag = s ._mag;
    _magErr = s._magErr;
    _corrCountRate = s._corrCountRate;  
    _corrCountRateErr = s._corrCountRateErr;
    _eFlag = s._eFlag;
    _cFlag = s._cFlag;
    _qFlag = s._qFlag;

  

     return SWSData(  _meanRa, _meanDec,
	     _mjdStart, _mjdEnd, _mjdMid,
	     _obsId,
	     _obsIdList,
             _filter,
	     _object,
	     _fileName, 
	     _srcId,
	     _mjdStartList, _mjdEndList, _mjdMidList,
	     _ra, _dec,
	     _significance,
	     _mag, _magErr,
	     _corrCountRate, _corrCountRateErr,
	     _eFlag, _cFlag, _qFlag); 
        



};  


    double _meanRa, _meanDec;
    double _mjdStart, _mjdEnd;
    double _mjdMid;
    string _obsId;
    map<int, vector<string> > _obsIdList;
    string _filter;
    string _object;
    string _fileName;
    vector<int> _srcId;
    vector<double> _mjdStartList, _mjdEndList, _mjdMidList;
    vector<double> _ra, _dec;
    vector<float> _significance;
    vector<float> _mag, _magErr;
    vector<float> _corrCountRate, _corrCountRateErr;

    vector<int8> _eFlag, _cFlag;
    vector<int16> _qFlag;

};
/*********************************************************
*
*
*
**********************************************************/
SWSData::SWSData(const SWSData &s)
{
    _meanRa = s._meanRa;
    _meanDec = s._meanDec;
    _mjdStart = s._mjdStart;
    _mjdEnd = s._mjdEnd;
    _obsId = s._obsId;
    _obsIdList = s._obsIdList;

    _mjdMid = s._mjdMid;

    _filter = s._filter;
    _object = s._object;
    _fileName = s._fileName;


    _srcId = s._srcId;
    _mjdStartList = s._mjdStartList;
    _mjdEndList = s._mjdEndList;
    _mjdMidList = s._mjdMidList;

    _ra = s._ra;
    _dec = s._dec;
    _significance = s._significance;
    _mag = s ._mag;
    _magErr = s._magErr;
    _corrCountRate = s._corrCountRate;  
    _corrCountRateErr = s._corrCountRateErr;
    _eFlag = s._eFlag;
    _cFlag = s._cFlag;
    _qFlag = s._qFlag;


};  

/****************************************************
*
*
*
****************************************************/ 
bool compare(const SWSData &a, const SWSData & b)
{

   return a._mjdStart < b._mjdStart;

};

/*****************************************************************
*
*
*
*
*****************************************************************/
class observationData
{
public: 

   string obsId;
   string revolution;
   string seqId;
   string object;
   int no;
   int size;
  
   vector<int> srcId;
   vector<double> ra;
   vector<double> dec;
   vector<double> ra_corr;
   vector<double> dec_corr;

   map<int, int> srcNo;
   map<string, vector<float>, OmFilter > significance;
   observationData() : obsId(""), revolution(""), seqId(""), object(""), no(-9999), size(-9999) {};
   observationData( const observationData & s)
   {
     obsId = s.obsId;
     revolution = s.revolution;
     seqId = s.seqId;
     object = s.object;


     no = s.no;
     size = s.size;
     
     srcId = s.srcId;
     ra = s.ra;
     dec = s.dec;

     ra_corr = s.ra_corr;
     dec_corr = s.dec_corr;

     srcNo = s.srcNo;
   }

};




/*****************************************************************
*
*
*
*
* Stores variability information about a single source in the 
* combined source-list file
*
*
*
*
*****************************************************************/
class VariabilityData
{
public:

  int _number;                 // source number in combined source-list
  int size;               // Number averaged
  float min;              // Minimum value
  float max;              // Maximum value
  float mean;             // Mean
  float median;           // Median
  float _nstd;             // Maximum value of nstd above median    
  int _eFlagMax;
  int _cFlagMax;
  int _qFlagMax;
  float chiSquared;
  float prob;
  double ra, ra_corr;
  double dec, dec_corr;
  double _meanRa, _meanDec; 
  double _mjdMid;
  double timeLength;
  string _obsId;
  vector<string>  _obsIdList;
  string _filter;
  string _object;
  observationData obsData;           // The observations data


  vector<OmSourceParameter> source;  // The averaged sources for this combined source

  SWSData _swsData;


  VariabilityData() : _number(-9999), size(-9999), min(-9999.9), max(-9999.9), mean(-9999.9), 
                      median(-9999.9), _nstd(-9999.9), 
                      _eFlagMax(0), _cFlagMax(0), _qFlagMax(0), 
                      chiSquared(-9999.9), prob(-9999.9),  
                      ra(-9999.9), ra_corr(-9999.9), dec(-9999.9), dec_corr(-9999.9) ,
                      _meanRa(0), _meanDec(0), _mjdMid(0), timeLength(-9999.9),
                      _obsId(""), _obsIdList( tempVector),
		      _filter(""), _object("")
                      
  {
       
  };


  VariabilityData(int number, int size, float min, float max,
                  float mean, float median, float nstd, 
                  int eFlagMax, int cFlagMax, int qFlagMax, 
                  float chiSquared, float prob, 
                  double ra, double raCorr,
                  double dec, double decCorr, 
                  double meanRa, double meanDec,
                  double mjdMid, double timeLength,
                  string obsId, vector<string> obsIdList,
                  string filter, string object, 
                  observationData obsData) : _number(number), size(size), min(min), max(max), mean(mean), 
                      median(median), _nstd(nstd),
                      _eFlagMax( eFlagMax ), _cFlagMax( cFlagMax ), _qFlagMax( qFlagMax ),  
                      chiSquared(chiSquared), prob(prob), 
                      ra(ra), ra_corr(raCorr), dec(dec), dec_corr(decCorr) ,
                      _meanRa(meanRa), _meanDec(meanDec), _mjdMid(mjdMid), timeLength(timeLength),
                       _obsId(obsId), _obsIdList(obsIdList), _filter(filter), 
                      _object(object), obsData(obsData) 
          {



       
  };








    /*********************************************************
    *
    *
    *
    **********************************************************/
    VariabilityData operator = (const VariabilityData &s)
    {

    _number = s._number;
    _filter = s._filter;
    _obsId = s._obsId;

    _obsIdList = s._obsIdList;

    _object = s._object;

    _meanRa = s._meanRa;
    _meanDec = s._meanDec;

    _eFlagMax = s._eFlagMax;
    _cFlagMax = s._cFlagMax;
    _qFlagMax = s._qFlagMax;

    size = s.size;
    min = s.min;
    max = s.max;
    mean = s.mean;
    median = s.median;
    _nstd = s._nstd;
    chiSquared = s.chiSquared;
    prob = s.prob;
    _mjdMid = s._mjdMid;
    timeLength = s.timeLength;
    
    ra = s.ra;
    dec = s.dec;

      
    ra_corr = s.ra_corr;
    dec_corr = s.dec_corr;
    obsData.obsId = s.obsData.obsId;
 
    obsData.revolution = s.obsData.revolution;
    obsData.seqId = s.obsData.seqId;

    
    obsData.no    = s.obsData.no;
    obsData.size  = s.obsData.size; 

   
    obsData.ra  = s.obsData.ra; 
    obsData.dec  = s.obsData.dec; 
    obsData.ra_corr  = s.obsData.ra_corr; 
    obsData.dec_corr  = s.obsData.dec_corr; 
     
 
 
     source = s.source;

    _swsData = s._swsData;

     return   VariabilityData(_number, size, min, max,
                  mean, median, _nstd, 
                  _eFlagMax, _cFlagMax, _qFlagMax, 
                  chiSquared, prob, 
                  ra, ra_corr,
                  dec, dec_corr, 
                  _meanRa, _meanDec,
                  _mjdMid, timeLength,
                  _obsId,_obsIdList,
                  _filter, _object, 
			      obsData);




    };  

  VariabilityData(const VariabilityData &s)
  {
    _number = s._number;
    _filter = s._filter;
    _obsId = s._obsId;

    _obsIdList = s._obsIdList;

    _object = s._object;

    _meanRa = s._meanRa;
    _meanDec = s._meanDec;



    _eFlagMax = s._eFlagMax;
    _cFlagMax = s._cFlagMax;
    _qFlagMax = s._qFlagMax;




    size = s.size;
    min = s.min;
    max = s.max;
    mean = s.mean;
    median = s.median;
    _nstd = s._nstd;
    chiSquared = s.chiSquared;
    prob = s.prob;
    _mjdMid = s._mjdMid;
    timeLength = s.timeLength;
    
    ra = s.ra;
    dec = s.dec;

      
    ra_corr = s.ra_corr;
    dec_corr = s.dec_corr;
    obsData.obsId = s.obsData.obsId;
 
    obsData.revolution = s.obsData.revolution;
    obsData.seqId = s.obsData.seqId;

    
    obsData.no    = s.obsData.no;
    obsData.size  = s.obsData.size; 

   
    obsData.ra  = s.obsData.ra; 
    obsData.dec  = s.obsData.dec; 
    obsData.ra_corr  = s.obsData.ra_corr; 
    obsData.dec_corr  = s.obsData.dec_corr; 
     
 
 
     source = s.source;

    _swsData = s._swsData;
  };  

};






void process(string obsId, map<string, vector<SWSData>, OmFilter  > &filterMapData, 
       bool addVariabilityColumns = false, string subHeading="" );


bool loadSWSData( string file, SWSData &data);
void computeVariabilityParameters(SWSData &swsData, VariabilityData &varData);

void addVariabilityColumn( string filter, map<int, VariabilityData> &varData, string subHeading = "");





bool SWSFileOkay( string file, string &obsId, string &filter, double &mjdStart, double &mjdMid);
void loadSrcListData(string file, vector < OmSourceParameter > &parameters, string &filter);

bool getObsFile( const string, observationData &data); 





void checkForVariability( const string filter, 
                          const vector<SWSData> &data, const int size,
                          map<int, map<string, VariabilityData, OmFilter> > & varData );





void createOutputFile( const int noSources, map<string, map<int, 
                                            map<string, VariabilityData, OmFilter> > > & data);




void  checkForBadExposures( observationData &obsData,  
                            map<string, vector<float> > &exposureMap, 
                            map<string, vector<string> > &exposureIdMap,
                            map<string, int, OmFilter> &obsSizeMap, map<string, int, OmFilter> &noVarSources);



void createOutputFile(string file);



/**************************************************************************
*
*
**************************************************************************/
void upDateTable( string &file, const int noSources, const string obsNo, 
                  map<int, map<string, VariabilityData, OmFilter> > &s);



double getTime(string date);

void plotSourceVariability( string obsId, int srcId, map<string, VariabilityData, OmFilter>  &varData);


/*******************************************************************************
*
*
* Compute the chi-squared
********************************************************************/
float computeChiSquared(vector<float> &x, vector<float> &dx, float &mean, float &prob);



void variabilityStatistics(int);


void updateStatistics(   map<int, map<string, VariabilityData, OmFilter> > &data, int &); 





/*********************************************************************************************************
*
**********************************************************************************************************/
void drawGraphs( float nstd, int &graphNo, string obsId,  map<int, map<string, SWSData, OmFilter> >  &varData);




F90MODPROCPROTO( void, coords, converttogalactic2)(double *, double*, double *, double*);


F90MODPROCPROTO( double, coords, chisqprobability)(double *, int *);



float minMag = 0.0, maxMag = 0.0;

map<string, map<string, int> > obsSizeMap; // Stores the number of sources in each combined source-list file per filter

map<string, int, OmFilter> noSources;
map<string, int, OmFilter> noVarSources;
map<string, int, OmFilter> noMultiSources;



map<string, map<string, int, OmFilter> > noGoodVarSources;  // Number of variable sources (all flags=0), in an observation per filter
map<string, map<string, int, OmFilter> > noBadVarSources;   // Number of variable sources (all flags=0, one or more discrepant exposures), in an observation per filter
map<string, map<string, bool> > goodObservation;    // Records if the observation has one or more variable sources, per filter, and all exposures are the same
map<string, map<string, int> > noVariableSourcesMap; // Records for each observation the number of variable sources per filter 

map<string, float, OmFilter> percVariable; // Percentage of sources with no of counterparts > 2 for a given filter


vector<string> omFilter;



int noObservationFilesChecked = 0;

/***********************************************************************
*
*
************************************************************************/


map<string, QString> obsIdMap;



/******************************************************************************
*
*
******************************************************************************/
map<string, map<int, int>, OmFilter > filterCount;

map<string, int, OmFilter> countSources;     // Counts the total number of sources
map<string, int, OmFilter> countExtended;    // Counts the number of sources with extension flag = 1
map<string, int, OmFilter> countQuality;     // Counts the number of sources with quality flag > 0
map<string, int, OmFilter> countConfusion;   // Counts the number of sources with quality flag > 0
map<string, int, OmFilter> countFlagsZero;   // Counts the number of sources with all flags = 0





/***********************************************************************
*
*
***********************************************************************/

ofstream osbad;
int noBadObs = 0;
int noDirectoriesChecked;
int noVariableDirectories = 0;
int noVariableDirectoriesSameExposures = 0;








template<class T>
std::string ToString(const T& val)
{
    std::stringstream  strm;
    strm << val;
    return strm.str();
}

#endif
