// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

#include "Dal.h"
#include <iostream>
#include <iomanip>
#include "sourcelist.h"
#include <qdir.h>
#include <qstring.h>
#include <typeinfo>
#include <vector>
#include "SasDalUtils.h"
#include "dal_utilities.h"
#include "DataSubspace.h"
//#include <cstdlib>
#include "OmSourceParameters.h"
#include<cmath>


using namespace SasDalUtils;  

/********************************************************
 *
 *
 ******************************************************/
int GetNoRows(const string& fileName, const string& tableName) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


    if ( !dataSet->hasBlock( tableName) ) 
    {
        cout << "Table doesn't exist\n";
        dataSetServer->close(dataSet); // Finished with the file
 
        return 0;

    }

    const Table *table = dataSet->table(tableName);
  
    int size = table->rows();
  
    dataSetServer->close(dataSet); // Finished with the file
 
    return size;
}

/********************************************************
 *
 *
 ******************************************************/
float GetAttribute(const string& fileName, const string& tableName, const string& name) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return 0;    


    if (!dataSet->hasBlock(tableName))    
    {
       cout << "Table doesn't exist\n";
       dataSetServer->close(dataSet); // Finished with the file
 
       return 0;

    }

    const Table *table = dataSet->table(tableName);
  
    float value = -1.0;
    if (table->hasAttribute(name))
        value = table->attribute(name)->asReal();


    dataSetServer->close(dataSet); // Finished with the file
  
    return value;    

}


/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const string & value) 
{
    // Open the file

  //  cout << "addattribute line 87\n";
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if ( !dataSet )
        return;    

    //    cout << "Line 94 adding attribute " << name << " =" << value << endl;

    *dataSet->addAttribute( name, comment ) = value;


    dataSetServer->close(dataSet); // Finished with the file
  
}

/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const bool & value) 
{
    // Open the file
  


    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if ( ! dataSet )
      return;    

  
    *dataSet->addAttribute( name, comment ) = value;




    dataSetServer->close(dataSet); // Finished with the file
  
    

}
/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const float & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    



    *dataSet->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}


/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const double & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    



    *dataSet->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}


/********************************************************
 *
 *
 ******************************************************/
void addAttribute(const string& fileName, const string& name, const string &comment, const int & value) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return;    



    *dataSet->addAttribute( name, comment ) = value;





    dataSetServer->close(dataSet); // Finished with the file
  
    

}








/********************************************************
 *
 *
 ******************************************************/
bool GetBooleanAttribute(const string& fileName, const string& tableName, const string& name) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (dataSet)
      cout << "dataset successfully opened\n";
    else
      return 0;    


   if (!dataSet->hasBlock(tableName)) 
   
   {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return 0;

    }

    const Table *table = dataSet->table(tableName);
  
    bool value = false;
    if (table->hasAttribute(name))
        value = table->attribute(name)->asBool();


    dataSetServer->close(dataSet); // Finished with the file
  
    return value;    

}


/********************************************************
 *
 *
 ******************************************************/
string GetAttribute(const string& fileName, const string& name) 
{
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return string(" ");;    


  
   string value = " ";
   if (dataSet->hasAttribute(name))
     value = (dataSet->attribute(name)->asString()).c_str();



   dataSetServer->close(dataSet); // Finished with the file
  
   return value;    

}
/********************************************************
 *
 *
 ******************************************************/
vector<float> GetAttributes(const string& fileName, const string& tableName, const string name[], int n) 
{

    vector<float> value;
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return value;    


   if (!dataSet->hasBlock(tableName)) 
   {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return value;

    }

   const Table *table = dataSet->table(tableName);
 
   for (int i = 0; i < n; i++)
   { 

       if (table->hasAttribute(name[i]))
       {
	   float f = table->attribute(name[i])->asReal();
      
           value.push_back(f);
       }
       else
           value.push_back(-1.0);
	
   }
   dataSetServer->close(dataSet); // Finished with the file
 
  
   return value;    

}

/********************************************************
*
*
******************************************************/
float *LoadColumnData(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size)
{
     
   
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


    if (!dataSet->hasBlock(tableName)) 
    {
        cout << "Table doesn't exist\n";
        dataSetServer->close(dataSet); // Finished with the file
 
        return 0;

    }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    float *data = 0;

    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        float *xData = columnData->real32Data();
        data = new float[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}
     }
         
    dataSetServer->close(dataSet); // Finished with the file
 
    return data;
};



/********************************************************
*
*
******************************************************/
float *LoadColumnData(const Table *table,  const string& columnName)
{
     
   
  
    int size = table->rows();
   
  
    float *data = 0;

    if (table->hasColumn(columnName))
    { 
      


        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	
     


      
        float *xData = columnData->real32Data();


    



        data = new float[size];


     

	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}

       


     
     }
         
 
   

    return data;

};




/********************************************************
*
*
******************************************************/
bool LoadColumnData(const Table *table,  const string& columnName, vector<int8> & data)
{
     
   
  
    int size = table->rows();
   
  
    data.resize( size );

    if (table->hasColumn(columnName))
    { 
      


        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) return false;
	
     


      
        int8 *xData = columnData->int8Data();


    


     

	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}

       


     
     }
         
 
   

    return true;

};
/********************************************************
*
*
******************************************************/
bool LoadColumnData(const Table *table,  const string& columnName, vector<int16> & data)
{
     
   
  
    int size = table->rows();
   
  
    data.resize( size );

    if (table->hasColumn(columnName))
    { 
      


        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) return false;
	
     


      
        int16 *xData = columnData->int16Data();


    


     

	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}

       


     
     }
         
 
   

    return true;

};




/********************************************************
*
*
******************************************************/
bool LoadColumnData(const Table *table,  const string& columnName, vector<int> & data)
{
     
   
  
    int size = table->rows();
   
  
    data.resize( size );

    if (table->hasColumn(columnName))
    { 
      


        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) return false;
	
     


      
        int *xData = columnData->int32Data();


    


     

	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}

       


     
     }
         
 
   

    return true;

};

  



  
  
/********************************************************
*
*
******************************************************/
bool LoadColumnData(const Table *table,  const string& columnName, vector<float> & data)
{
     
   
  
    int size = table->rows();
   
  
    data.resize( size );

    if (table->hasColumn(columnName))
    { 
      


        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) return false;
	
     


      
        float *xData = columnData->real32Data();


    


     

	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}

       


     
     }
         
 
   

    return true;

};

  
    
/********************************************************
*
*
******************************************************/
bool LoadColumnData(const Table *table,  const string& columnName, vector<double> & data)
{
     
  
    int size = table->rows();
   
  
    data.resize( size );

    if (table->hasColumn(columnName))
    { 
      


        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) return false;
	
     


      
        double *xData = columnData->real64Data();


    


     

	for (int i = 0;  i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}

       


     
     }
         
 
   

    return true;

};

  
  








/********************************************************
*
*
******************************************************/
double *LoadDoubleColumnData(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size)
{
     
   
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


   if (!dataSet->hasBlock(tableName)) 
      {
      cout << "Table doesn't exist\n";
    dataSetServer->close(dataSet); // Finished with the file
 
      return 0;

      }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    double *data = 0;

    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
        data = new double[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}
     }
         
    dataSetServer->close(dataSet); // Finished with the file
 
    return data;
};


/********************************************************
*
*
******************************************************/
double *LoadDoubleColumnData(const Table *table, const string& columnName)
{
     
   
      
    int size = table->rows();
   
  
    double *data = 0;

    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
        data = new double[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}
     }
         
 
    return data;
};







/********************************************************
*
*
******************************************************/


double *LoadColumnData1(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size)
{
     
   
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet) return 0;    


   if (!dataSet->hasBlock(tableName)) 
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return 0;

      }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    double *data = 0;

    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
        data = new double[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
	 
	}
     }
          
    dataSetServer->close(dataSet); // Finished with the file
   
  
    return data;
};

/********************************************************
*
*
******************************************************/

Integer1 *LoadInt1ColumnData(const string& fileName, 
                             const string& tableName, 
                             const string& columnName, 
                             int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);


    if (!dataSet)
   
      {
	cout << "failure to open data set\n";
      return 0;    
      }

         // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return 0;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    Integer1 *data = 0;
    
   
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();


       if (!columnData) exit(0);
	
   
      
       //        int *xData = columnData->int32Data();
       //    Integer1 *xData = columnData->int8Data();
        int8 *xData = columnData->int8Data();

        data = new Integer1[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = (Integer1)xData[i]; 
       
	}
     }
     
       
    dataSetServer->close(dataSet); // Finished with the file
      

  
    return data;
};
/********************************************************
*
*
******************************************************/

Integer1 *LoadInt1ColumnData(const Table *table, const string& columnName)
{
   
    
  
    int size = table->rows();
   
  
    Integer1 *data = 0;
    
   
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();


       if (!columnData) exit(0);
	
   
      
       //        int *xData = columnData->int32Data();
       //    Integer1 *xData = columnData->int8Data();
        int8 *xData = columnData->int8Data();

        data = new Integer1[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = (Integer1)xData[i]; 
       
	}
     }
     
       
 
      

  
    return data;
};

/********************************************************
*
*
******************************************************/

Integer2 *LoadInt2ColumnData(const Table *table, const string& columnName)
{
   
    
  
    int size = table->rows();
   
  
    Integer2 *data = 0;
    
   
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();


       if (!columnData) exit(0);
	
   
      
        int16 *xData = columnData->int16Data();

        data = new Integer2[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = (Integer2)xData[i]; 
       
	}
     }
     
       
 
      

  
    return data;
};

/********************************************************
*
*
******************************************************/

int *LoadIntColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);


    if (!dataSet)
   
      {
	cout << "failure to open data set\n";
      return 0;    
      }

      // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return 0;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    int *data = 0;
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        int *xData = columnData->int32Data();
        data = new int[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
       
	}
     }
     
      
    dataSetServer->close(dataSet); // Finished with the file
 

  
    return data;
};










/********************************************************
*
*
******************************************************/

vector<string> LoadVectorStringColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    vector<string> data(0);
 
    size = 0;
    if (!dataSet)
   
    {
	cout << "failure to open data set\n";
        return data; 
    }

      // Read the table

    if (!dataSet->hasBlock(tableName))     
    {
        cout << "Table doesn't exist\n";
        dataSetServer->close(dataSet); // Finished with the file
      
        return data;

    }


    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
     
        
  

	char * d = columnData -> stringData();
        int width = columnData -> parent() -> elements(); 
        char text[ width + 1]; 
        for(unsigned long i = 0; i < columnData -> count(); i++, d += columnData -> parent() -> elements( ))
	{

          strcpy(text, " ");
	  strncpy(text, d,  width );
          text[width] = '\0'; 
	  //	  cout << i << ") " << text << endl;
	
          data.push_back( string(text) );

      
    	}
        delete columnData;




     }
     
    dataSetServer->close(dataSet); // Finished with the file

     
    return data;
};


/********************************************************
*
*
******************************************************/

vector<int> LoadVectorIntColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    vector<int> data;
 
    size = 0;
    if (!dataSet)
   
    {
	cout << "failure to open data set\n";
        return data; 
    }

      // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return data;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        int *xData = columnData->int32Data();
        


	for (int i=0; i< size; i++) 
        {
	  data.push_back(xData[i]); 
       
	}
     }
     
      
      
  dataSetServer->close(dataSet); // Finished with the file

  
    return data;
};




/********************************************************
*
*
******************************************************/

vector<int8> LoadVectorInt8ColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    vector<int8> data;
 
    size = 0;
    if (!dataSet)
   
    {
	cout << "failure to open data set\n";
        return data; 
    }

      // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return data;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        int8 *xData = columnData->int8Data();
        


	for (int i=0; i< size; i++) 
        {
	  data.push_back(xData[i]); 
       
	}
     }
     
      
      
  dataSetServer->close(dataSet); // Finished with the file

  
    return data;
};




/********************************************************
*
*
******************************************************/

vector<int16> LoadVectorInt16ColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    vector<int16> data;
 
    size = 0;
    if (!dataSet)
   
    {
	cout << "failure to open data set\n";
        return data; 
    }

      // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return data;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        int16 *xData = columnData->int16Data();
        


	for (int i=0; i< size; i++) 
        {
	  data.push_back(xData[i]); 
       
	}
     }
     
      
      
  dataSetServer->close(dataSet); // Finished with the file

  
    return data;
};

/********************************************************
*
*
******************************************************/

vector<int32> LoadVectorInt32ColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    vector<int32> data;
 
    size = 0;
    if (!dataSet)
   
    {
	cout << "failure to open data set\n";
        return data; 
    }

      // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return data;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        int32 *xData = columnData->int32Data();
        


	for (int i=0; i< size; i++) 
        {
	  data.push_back(xData[i]); 
       
	}
     }
     
      
      
    dataSetServer->close(dataSet); // Finished with the file

  
    return data;
};

/********************************************************
*
*
******************************************************/

vector<float> LoadVectorFloatColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    vector<float> data;
 
    size = 0;
    if (!dataSet)
   
    {
	cout << "failure to open data set\n";
        return data; 
    }

      // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return data;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        float *xData = columnData->real32Data();
        


	for (int i=0; i< size; i++) 
        {
	  data.push_back(xData[i]); 
       
	}
     }
     
      
      
    dataSetServer->close(dataSet); // Finished with the file

  
    return data;
};

/********************************************************
*
*
******************************************************/

vector<double> LoadVectorDoubleColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
                       int &size)
{
   

 
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    vector<double> data;
 
    size = 0;
    if (!dataSet)
   
    {
	cout << "failure to open data set\n";
        return data; 
    }

      // Read the table

    if (!dataSet->hasBlock(tableName)) 
    
      {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      
      return data;

    }
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        double *xData = columnData->real64Data();
        


	for (int i=0; i< size; i++) 
        {
	  data.push_back(xData[i]); 
       
	}
     }
     
      
      
    dataSetServer->close(dataSet); // Finished with the file

  
    return data;
};





/********************************************************
*
*
******************************************************/

bool LoadVectorFloatColumnData(const Table *table, 
                       const string& columnName, 
				int &size,
                       vector<float> &data)
{
   



   
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) return false;
	
   
      
        float *xData = columnData->real32Data();
        


	for (int i=0; i< size; i++) 
        {
	  data.push_back(xData[i]); 
       
	}
     }
     
      
      

  
    return true;
};





/********************************************************
*
*
******************************************************/

bool LoadVectorDoubleColumnData(const Table *table, 
                       const string& columnName, 
				int &size,
                       vector<double> &data)
{
   



   
  
    size = table->rows();
   
  
    
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) return false;
	
   
      
        double *xData = columnData->real64Data();
        


	for (int i=0; i< size; i++) 
        {
	  data.push_back(xData[i]); 
       
	}
     }
     
      
      

  
    return true;
};



/********************************************************
*
*
******************************************************/

int *LoadIntColumnData(const Table *table, const string& columnName)
{
   

    int size = table->rows();
   
  
    int *data = 0;
    
   
    if (table->hasColumn(columnName)) 
    { 
        const ColumnData *columnData = table->column(columnName)->data();
	    
 
        if (!columnData) exit(0);
	
   
      
        int *xData = columnData->int32Data();
        data = new int[size];


	for (int i=0; i< size; i++) 
        {
	  data[i] = xData[i]; 
       
	}
     }
     
       
        

  
    return data;
}








/**************************************************
*
*
**************************************************/
vector<QString> loadDirectoryFiles(const string directory, 
         const int type, const string filter, const string fits )
{

  //    cout << "loaddirectoryfiles line 4510 directory=" << directory << endl;  

    QDir d( directory.c_str()  );
    
    d.setFilter( type ); 
    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it( *list );
    QFileInfo *fi;

    vector<QString> fileList;


    //    cout << "Loaddirectoryfiles line 4521\n";


    if (filter == "*")
    { 
        while ((fi = it.current()) != 0)
        {
        
            QString s = fi->fileName().latin1();	
            if(s.length() > 1)
	        fileList.push_back( s );
                     
    
            ++it;
        }
    }
    else 
    { 
        while ((fi = it.current()) != 0)
        {
        
            QString s = fi->fileName().latin1();
	    //	    cout << s << endl; 	
            if(s.length() > 1 && s.contains(filter.c_str()) && s.contains(fits.c_str())  )
	        fileList.push_back( s );
                     
            ++it;
        }
    } 

    return fileList;
}


/**************************************************
*
*
**************************************************/
vector<QString> loadDirectoryFiles(const string directory, 
         const int type)
{

  //    cout << "loaddirectoryfiles line 4510 directory=" << directory << endl;  

    QDir d( directory.c_str()  );
    
    d.setFilter( type ); 
    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it( *list );
    QFileInfo *fi;

    vector<QString> fileList;


    //    cout << "Loaddirectoryfiles line 4521\n";


    
    while ((fi = it.current()) != 0)
    {
        
        QString s = fi->fileName().latin1();	
        if(s.length() > 1)
	fileList.push_back( s );
                     
    
        ++it;
    }
    


    return fileList;
}





/**************************************************
 *
 *
 *************************************************/
vector<QString> loadDirectoryFiles(char *filter, char c)
{
  
    QDir d;
    

    d.setFilter (QDir::Files | QDir::Hidden |
               QDir::NoSymLinks);
    //    d.setSorting(QDir::Size | QDir::Name); // Sort by name
    d.setSorting(QDir::Name); // Sort by name

    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it(*list);
    QFileInfo *fi;

    vector<QString> fileList;
  
    while ((fi = it.current()) != 0)
    {
  


        QString s = fi->fileName().latin1();

        if (s[0] == c && s.contains(filter) && s.contains("FIT") && !s.contains("new") )
	{


  
            string filter = GetAttribute( (const char*)s, "FILTER" );


 
            if (filter != "GRISM1" && filter != "GRISM2" && filter != "BLOCKED") 
	    {


	        fileList.push_back(s);
             
            }
        }
        ++it;
    } 
    return fileList;

}


/**************************************************
 *
 *
 *************************************************/
vector<QString> loadDirectoryFiles(char *filter, char* filter1, char c)
{
  
    QDir d;
    

    d.setFilter (QDir::Files | QDir::Hidden |
               QDir::NoSymLinks);
    //    d.setSorting(QDir::Size | QDir::Name); // Sort by name
    d.setSorting(QDir::Name); // Sort by name

    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it(*list);
    QFileInfo *fi;

    vector<QString> fileList;
  
    while ((fi = it.current()) != 0)
    {
  


        QString s = fi->fileName().latin1();

        if (s[0] == c && ( s.contains(filter) || s.contains(filter1) )  && s.contains("FIT") && !s.contains("new") )
	{


  
            string filter = GetAttribute( (const char*)s, "FILTER" );


 
            if (filter != "GRISM1" && filter != "GRISM2" && filter != "BLOCKED") 
	    {


	        fileList.push_back(s);
           
            }
        }
        ++it;
    } 
    return fileList;

}








/**************************************************
 *
 *
 *************************************************/
vector<QString> loadDirectoryFiles(char *filter)
{
  
    QDir d;
    d.setFilter (QDir::Files );
  
    d.setSorting(QDir::Name); // Sort by name

    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it(*list);
    QFileInfo *fi;

    vector<QString> fileList;
  
    while ((fi = it.current()) != 0)
    {
        QString s = fi->fileName().latin1();
    
        if (s.contains(filter))
	{
	    fileList.push_back(s);
            
        }
        ++it;
    } 
 
    return fileList;

}











/***************************************************************
*
****************************************************************/
bool deleteRowFromTable(const string& fileName,
                        const string& tableName,
                        const int rowNumber)
{


    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);


    if (!dataSet)
    {
	cout << "failure to open data set\n";
        return false;    
    }


    if (!dataSet->hasBlock(tableName)) 
   
    {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
 
      return false;

    }

    Table *table = dataSet->table(tableName);
  
    table->deleteRows(rowNumber, 1);


    dataSetServer->close(dataSet); // Finished with the file
 
    return true;



}



/******************************************************
*
******************************************************/



void setColumnData( const int size, ColumnData *columnData, const int8* data, const int iStart)
{
    int8 * r64 = columnData->int8Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];



}



/******************************************************
*
******************************************************/



void setColumnData( const int size, ColumnData *columnData, const vector<int8> &data, const int iStart )
{
    int8 * r64 = columnData->int8Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];



}


/******************************************************
*
******************************************************/
void setColumnData( const int size, ColumnData *columnData, const int16* data, const int iStart )
{
    int16 * r64 = columnData->int16Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];


}
/******************************************************
*
******************************************************/
void setColumnData( const int size, ColumnData *columnData, const vector<int16> &data, const int iStart)
{
    int16 * r64 = columnData->int16Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];


}
/******************************************************
*
******************************************************/
void setColumnData( const int size, ColumnData *columnData, const int32* data, const int iStart )
{
    int32 * r64 = columnData->int32Data();
    for(int i = 0; i < size; i++)
        r64[iStart + i] = data[i];



}

/******************************************************
*
******************************************************/
void setColumnData( const int size, ColumnData *columnData, const vector<int32> & data, const int iStart )
{
    int32 * r64 = columnData->int32Data();
    for(int i = 0; i < size; i++)
    {
        if( data[i] ==  -9999 )
            columnData -> setToNull( iStart + i );
        else
            r64[iStart + i] = data[i];
    }


}

/******************************************************
*
******************************************************/


void setColumnData( const int size, ColumnData *columnData, const real32* data, const int iStart )
{
  

    real32 * r32 = columnData->real32Data();
    for(int i = 0; i < size; i++)
    {
      if( abs( data[i] + 9999.9) < 1e-5)
          columnData -> setToNull( iStart + i );
      else
          r32[iStart + i] = data[i];

    }

 

}
/******************************************************
*
******************************************************/


void setColumnData( const int size, ColumnData *columnData, const vector<real32>& data, const int iStart )
{

  //  cout << "setcolumndata line 1754\n";
   
    real32 * r32 = columnData->real32Data();
    for(int i = 0; i < size; i++)    
    {
     
      //      cout << i << " ) data=" << data[i] << endl;


        if( abs( data[i] + 9999.9) < 1e-2)
	{
	  //  cout << "Line 1765\n";
            columnData -> setToNull( iStart + i );
   
	    //  cout << "Line 1768\n";
   
        }
        else 

            r32[iStart + i] = data[i];
    }






}

/******************************************************
*
******************************************************/
void setColumnData( const int size, ColumnData *columnData, const real64* data, const int iStart )
{



  

    real64 * r64 = columnData->real64Data();
    for(int i = 0; i < size; i++)
    {
        r64[iStart + i] = data[i];
       

    }
}

/******************************************************
*
******************************************************/
void setColumnData( const int size, ColumnData *columnData, const vector<real64> &data, const int iStart )
{
  

    real64 * r64 = columnData->real64Data();
    for(int i = 0; i < size; i++)
    {
     
        if( abs( data[i] + 9999.9) < 1e-3)
            columnData -> setToNull( iStart + i );
        else 

            r64[iStart + i] = data[i];
    }

}

/******************************************************
*
******************************************************/
//void setColumnData( const int size, ColumnData *columnData, const char* data, const int iStart)
//{

//  if (!columnData) return;
  //    real64 * r64 = columnData->real64Data();
    


//}
/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName,
               const string tableName, 
               const string columnName, 
               const string units,
               const string comment,   
	       int nData,
               vector<real32> &data, 
               int position)
{
     

 


  

    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   

   
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }
    // Get the size of the table
 



 

  


    Table *table = dataSet->table(tableName, Dal::Modify);
    Column *column = 0;
 
   
    if(table->hasColumn( columnName ) )
    {
        column = table->column(columnName);
    }
    else
    {

        
        // Create the column
 



        longVec size(1);
        size[0] = 1;
        column = table->addColumn( columnName, Column::Real32, comment, units, size, position);
    }
 

  



  


    ColumnData *columnData = column->data();

    

  

    setColumnData( nData, columnData, data);




    delete columnData;

    dataSetServer->close(dataSet); // Finished with the file



 


          
    return true;
};








/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName,
               const string tableName, 
               const string columnName, 
               const string units,
               const string comment,   
	       int nData,
               vector<real64> &data, 
               int pos)
{
     


  

    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   



   
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }
    // Get the size of the table
 
  


    Table *table = dataSet->table(tableName, Dal::Modify);
    Column *column = 0;
    int position = pos;
    if(table->hasColumn( columnName ) )
    {
        column = table->column(columnName);
    }
    else
    {

       
        // Create the column
       
         
        longVec size(1);
        size[0] = 1;

        column = table->addColumn( columnName, Column::Real64, comment, units, size, position);
    }
 





    ColumnData *columnData = column->data();

   
  

    setColumnData( nData, columnData, data);




    delete columnData;

    dataSetServer->close(dataSet); // Finished with the file



 



          
    return true;
};




/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName,
               const string tableName, 
               const string columnName, 
               const string units,
               const string comment,   
	       int nData,
               vector<int> &data, 
               int pos)
{
     


  

    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   

   
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }
    // Get the size of the table
 
  


    Table *table = dataSet->table(tableName, Dal::Modify);
    Column *column = 0;
    int position = pos;
    if(table->hasColumn( columnName ) )
    {
        column = table->column(columnName);
    }
    else
    {

       
        // Create the column
        
        
        longVec size(1);
        size[0] = 1;

        column = table->addColumn( columnName, Column::Int32, comment, units, size, position);
    }
 




    column -> nullValue( -9999 );


    ColumnData *columnData = column->data();


  

    setColumnData( nData, columnData, data);




    delete columnData;

    dataSetServer->close(dataSet); // Finished with the file



 



          
    return true;
};







/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real32> &data, 
                     int pos)
{
     

  
    // Get the size of the table

    int position = pos;  
   
 
 
   
    // Create the column
    
    longVec size(1);
    size[0] = 1;



    Column *column = 0;

    if(table->hasColumn( columnName ) )
    {
        column = table->column(columnName);
    }
    else
    {

        column = table->addColumn( columnName, Dal::Real32, comment, units, size, position);
    } 


   
    //   column -> nullValue( -9999 );




    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data);


    delete columnData;


          
    return true;
};





/******************************************************
*
******************************************************/
bool appendToColumn( Table *table, 
                     const string columnName, 
                     const int iStart,   
                     const vector<string> &data)
{
     

  
   
    // Create the column
    Column *column = table->column( columnName );
 
    ColumnData *columnData = column->data();

 
    for (unsigned i = 0; i < data.size(); i++) 
    {
      //        cout << i << ") " << data[i] << endl;
        if(data[i].length() > 0) {  
	    
	    setStringColumnRow( columnData, iStart + i, data[i] );
	   
 
	} 
    }
     
  
    

    delete columnData;

    return true;
};



/******************************************************
*
*
*
*
******************************************************/
bool appendToColumn( Table *tab, 
                     const string columnName, 
                     const int iStart,   
                     const vector< vector<float> > &data)
{
     


 
    Column *col = tab->column( columnName );


    ColumnData *d = col->data();

 
    real32* r32 = d->real32Data();


    for (unsigned long row = iStart, k = 0; row < iStart + data.size(); ++row, ++k) 
    {
     
       
        for (unsigned long element = 0; element < data[k].size(); ++element) 
        {


     
            unsigned long pos = d->position(row, element);
          

            if( abs( data[k][element] + 9999.9) < 1e-3)
	    {
                d -> setToNull( pos );
            }
            else
	    {  
               r32[d->position(row, element)] = data[k][element];
            }
        }
     
        for (unsigned long element = data[k].size(); element < d->elements(); ++element) 
        {
            unsigned long pos = d->position(row, element);
          
            d -> setToNull( pos );
 

        }


    }
    delete d;

  

    return true;
};








/******************************************************
*
******************************************************/
bool appendToColumn( Table *tab, 
                     const string columnName, 
                     const int iStart,   
                     const vector< vector<double> > &data)
{
     


 
    Column *col = tab->column( columnName );


    ColumnData *d = col->data();

 

    real64* r32 = d->real64Data();






    for (unsigned long row = iStart, k = 0; row < iStart + data.size(); ++row, ++k) 
    {
    
        for (unsigned long element = 0; element < data[k].size(); ++element) 
        {
	
            unsigned long pos = d->position(row, element);
            

            if( abs( data[k][element] + 9999.9) < 1e-3)
                d -> setToNull( pos );
 
            else

                r32[d->position(row, element)] = data[k][element];
        }

        for (unsigned long element = data[k].size(); element < d->elements(); ++element) 
        {
            unsigned long pos = d->position(row, element);
          
            d -> setToNull( pos );
 

        }
     

    }
    delete d;

  

    return true;
};




    







/******************************************************
*
******************************************************/
bool appendToColumn( Table *tab, 
                     const string columnName, 
                     const int iStart,   
                     const vector< vector<int8> > &data)
{
     


 
    Column *col = tab->column( columnName );


    ColumnData *d = col->data();

 




    col -> nullValue( 127 );



    int8* r32 = d->int8Data();


    for (unsigned long row = iStart, k = 0; row < iStart + data.size(); ++row, ++k) 
    {
        for (unsigned long element = 0; element < data[k].size(); ++element) 
        {
            unsigned long pos = d->position(row, element);
          

            if( data[k][element] == (int8)127)
                d -> setToNull( pos );
 
            else

                r32[d->position(row, element)] = data[k][element];
        }

        for (unsigned long element = data[k].size(); element < d->elements(); ++element) 
        {
            unsigned long pos = d->position(row,element);
          
            d -> setToNull( pos );
 

        }


    }
    delete d;


    return true;
};





/******************************************************
*
******************************************************/
bool appendToColumn( Table *tab, 
                     const string columnName, 
                     const int iStart,   
                     const vector< vector<int16> > &data)
{
     


 
    Column *col = tab->column( columnName );


    ColumnData *d = col->data();

 




    col -> nullValue( -9999 );



    int16* r32 = d->int16Data();


    for (unsigned long row = iStart, k = 0; row < iStart + data.size(); ++row, ++k) 
    {
        for (unsigned long element = 0; element < data[k].size(); ++element) 
        {
            unsigned long pos = d->position(row, element);
          

            if( data[k][element] == -9999)
                d -> setToNull( pos );
 
            else

                r32[d->position(row, element)] = data[k][element];
        }

        for (unsigned long element = data[k].size(); element < d->elements(); ++element) 
        {
            unsigned long pos = d->position(row, element);
          
            d -> setToNull( pos );
 

        }


    }
    delete d;


    return true;
};







/******************************************************
*
******************************************************/
bool appendToColumn( Table *tab, 
                     const string columnName, 
                     const int iStart,   
                     const vector< vector<int32> > &data)
{
     


 
    Column *col = tab->column( columnName );


    ColumnData *d = col->data();

 

 


    col -> nullValue( -9999 );



    int32* r32 = d->int32Data();


    for (unsigned long row = iStart, k = 0; row < iStart + data.size(); ++row, ++k) 
    {
        for (unsigned long element = 0; element < data[k].size(); ++element) 
        {
            unsigned long pos = d->position(row, element);
          

            if( data[k][element] == -9999)
                d -> setToNull( pos );
 
            else

                r32[d->position(row, element)] = data[k][element];
        }

        for (unsigned long element = data[k].size(); element < d->elements(); ++element) 
        {
            unsigned long pos = d->position(row,element);
          
            d -> setToNull( pos );
 

        }


    }
    delete d;


    return true;
};

















/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real64> &data, 
                     int position)
{
     

  
  
  
   
    // Create the column


   
  


    longVec size(1);
    size[0] = 1;

    
    
    Column *column = 0;

    if(table->hasColumn( columnName ) )
    {
        column = table->column(columnName);
    }
    else
    {

        column = table->addColumn( columnName, Dal::Real64, comment, units, size, position);
    } 

 


    //    column -> nullValue( -9999);



    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data);


    delete columnData;


          
    return true;
};






/****************************************************************
 *
 ****************************************************************/
bool appendToColumn(Table *table, 
                     const string columnName, 
		     const int iStart,    	      
                     const vector<float> &data)
{
     

 
  //   cout << "appendtocolumn line 2638\n";
  //   cout << "istart=" << iStart << " column name=" << columnName << endl;
  //   cout << "No of rows in tbale=" << table->rows() << endl;
    // Create the column
    Column *column = table->column( columnName );
 
    ColumnData *columnData = column->data();

    //    cout << "appendtocolumn line 2646\n";


    setColumnData( (int)data.size(), columnData, data, iStart);


    //   cout << "appendtocolumn line 2652\n";


    delete columnData;


          
    return true;
};






/****************************************************************
 *
 ****************************************************************/
bool appendToColumn(Table *table, 
                     const string columnName, 
		     const int iStart,    	      
                     const vector<double> &data)
{
     

 
  
   
    // Create the column
    Column *column = table->column( columnName );
 
    ColumnData *columnData = column->data();

    setColumnData( (int)data.size(), columnData, data, iStart);


    delete columnData;


          
    return true;
};


/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
	       int nData,
                     vector<int8> &data, 
                     int position)
{
     

  
    // Get the size of the table
   
  
   
 
   



    // Create the column
    


    longVec size(1);
    size[0] = 1;



   
    Column *column = 0;

    if(table->hasColumn( columnName ) )
    {
        column = table->column(columnName);
    }
    else
    {

        column = table->addColumn( columnName, Dal::Int8, comment, units, size, position);
    } 





 
    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data);


    delete columnData;


          
    return true;
};


/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int> &data, 
                     int position)
{
     

  
    // Get the size of the table
   
   

   
    // Create the column
   




    longVec size(1);
    size[0] = 1;

 
    Column *column = 0;


    if(table->hasColumn( columnName ) )
    {
        column = table->column(columnName);
    }
    else
    {

        column = table->addColumn( columnName, Dal::Int32, comment, units, size, position);
    } 

 
    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data);


    delete columnData;


          
    return true;
};










/**********************************************************************************
*
**********************************************************************************/
bool appendToColumn(Table *table, 
                     const string columnName, 
		     const int iStart,	     
                     const vector<int8> &data)
{
     

 
  
   
    // Create the column
    Column *column = table->column( columnName );

    ColumnData *columnData = column->data();


    column -> nullValue( 127 );




    setColumnData( (int)data.size(), columnData, data, iStart );


    delete columnData;


          
    return true;
};


/********************************************************
*
*
******************************************************/
using namespace SasDalUtils;  

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int16> &data, 
                     int position)
{
     

  
    // Get the size of the table
   



  


  
    longVec size(1);
    size[0] = 1;

     Column *column = 0;

    
    if(table->hasColumn( columnName ) )
    {
        column = table->column(columnName);
    }
    else
    {

        column = table->addColumn( columnName, Dal::Int16, comment, units, size, position);
    } 


 
    ColumnData *columnData = column->data();

    setColumnData( nData, columnData, data);


    delete columnData;


          
    return true;
};
/********************************************************************
*
*
********************************************************************/
bool appendToColumn(Table *table, 
                     const string columnName, 
		     const int iStart,    
                     const vector<int16> &data)
{
     

  

  
   
    // Create the column
    Column *column = table->column( columnName );

    column -> nullValue( -9999 );

 
    ColumnData *columnData = column->data();

    setColumnData( (int)data.size(), columnData, data, iStart );


    delete columnData;


          
    return true;
};




/********************************************************
*
*
******************************************************/
using namespace SasDalUtils;  


/***************************************************
*
*
**************************************************/
bool appendToColumn(Table *table, 
                     const string columnName,                  
		     const int iStart,   
                     const vector<int32> &data)
{
     

 
  
   
    // Create the column
    Column *column = table->column( columnName );
 
    column -> nullValue( -9999 );


    ColumnData *columnData = column->data();

    setColumnData( (int)data.size(), columnData, data, iStart );


    delete columnData;


          
    return true;
};




/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName, 
               const string tableName, 
               const string columnName, 
               const string units,
               const string comment,
               int columnWidth,   
	       int nData,
               vector<string>& data, 
               int pos)
{




 
     

    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }

    // Read the table
    Table *table = dataSet->table(tableName, Dal::Modify);



    int position = pos;  
    




    // Get the size of the table
    vector<unsigned long> size(1);

    size[0] = columnWidth;

  

    // Create the column
    Column *column = 0;
    if ( table->hasColumn( columnName )) 
    {
        column = table->column( columnName );


    }

    else
    {
  
        column = table->addColumn( columnName, Dal::DString, comment, units, size, position);
    
    }


    if (column )
    { 
        ColumnData *columnData = column->data();

        if (!columnData) exit(0);
	
	//	cout << "Line 1874 ndata=" << nData << endl;
	for (int i = 0; i < nData; i++) 
        {

	  if(data[i].length() > 1) {  
	    //	    cout << i <<  ") sourcelist.cc line 1378 adding comment " << data[i] << " to source-list " << fileName 
	    //      << " comment length=" << data[i].length() << endl;
	    setStringColumnRow( columnData, i, data[i] );
	   
 
	  } 
	}
        delete columnData;

     }
    else
        exit(0);     
         
    dataSetServer->close(dataSet); // Finished with the file
 
    
    return true;
};


//  vector<unsigned long> strlen;
//  strlen.push_back(10);
//  col = tab->addColumn("string",Column::DString,"string data","",strlen);


//  d = col->data();
//  char *s = d->stringData();
//  int len = d->size()[0];
//  assert( len == 10 );
//  for (unsigned long i = 0; i < nRows; ++i) {
//    char	ss[ len + 1 ];
//    sprintf( ss, "string%ld", i );
    
//    strncpy(s + d->position(i), ss, len);
//  }
//  delete d;













/**********************************************************************
*  Load all the source data
*
**********************************************************************/
bool loadSourceParameters(const string &fileName, const char* tableName,
			  int& size, vector < OmSourceParameter >& parameters, string & filter, string &expId) 
                     

{
  
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }


    filter = (dataSet->attribute( "FILTER" )->asString()).c_str();
    float exposure =  (dataSet->attribute( "EXPOSURE" ))->asReal();
    expId = (dataSet->attribute( "EXP_ID" )->asString()).c_str();



     

 


    






    // Open the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
 

    parameters.empty();
    parameters.resize( size ); 

    
 
    for (int i = 0; i < size; i++) 
    {
        parameters[i].exposure = exposure; 
        
    }
   


    // Load the columns one by one

    



   
    if (table->hasColumn( "SRC_ID" )) 
    { 
        const ColumnData *columnData = table->column( "SRC_ID" )->data();
	    

	
       
        int32 *xData = columnData->int32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].number = xData[i]; 
       	 
       

	}
     }
     














    // ************ XPOS *************************

    if (table->hasColumn( "XPOS" )) 
    { 
        const ColumnData *columnData = table->column( "XPOS" )->data();
	    


       
        real32 *xData = columnData->real32Data();
       

    
	for (int i = 0; i < size; i++) 
        {
	  parameters[i].xcoord = xData[i];           
         
        
	}
     


     }
     
  
    // ************ YPOS *************************
    if (table->hasColumn( "YPOS" )) 
    { 
        const ColumnData *columnData = table->column( "YPOS" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].ycoord = xData[i]; 
       
	}
     }
     
    // ************ POSERR *************************



    if (table->hasColumn( "POSERR" )) 
    { 
        const ColumnData *columnData = table->column( "POSERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].positionerror = xData[i]; 
       
	}
     }
     
    // ************ RA *************************



    if (table->hasColumn( "RA" )) 
    { 
        const ColumnData *columnData = table->column( "RA" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rightascension = xData[i]; 
       
	}
     }
     

   
    // ************ DEC *************************

    if (table->hasColumn( "DEC" )) 
    { 
        const ColumnData *columnData = table->column( "DEC" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].declination = xData[i]; 
       
	}
     }
     


  





    if (table->hasColumn( "RATE" )) 
    { 
        const ColumnData *columnData = table->column( "RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rawcountrate = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "RATE_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "RATE_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rawcountrateerror = xData[i]; 
       
	}
     }





    if (table->hasColumn( "BACKGROUND_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "BACKGROUND_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rawbackgroundcountrate = xData[i]; 
       
	}
     }
     
     

    if (table->hasColumn( "SIGNIFICANCE" )) 
    { 
        const ColumnData *columnData = table->column( "SIGNIFICANCE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].significance = xData[i]; 
       
	}
     }
     



    if (table->hasColumn( "CORR_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrcountrate = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "CORR_RATE_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_RATE_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrcountrateerror = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "CORR_BACKGROUND_RATE" )) 
    { 
        const ColumnData *columnData = table->column( "CORR_BACKGROUND_RATE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].corrbackgroundcountrate = xData[i]; 
       
	}
     }
     



    if (table->hasColumn( "PSF1_CORR" )) 
    { 
        const ColumnData *columnData = table->column( "PSF1_CORR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].psf1correction = xData[i]; 
       
	}
     }
     


    if (table->hasColumn( "PSF2_CORR" )) 
    { 
        const ColumnData *columnData = table->column( "PSF2_CORR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].psf2correction = xData[i]; 
       
	}
     }
     


    if (table->hasColumn( "MAG" )) 
    { 
        const ColumnData *columnData = table->column( "MAG" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].magnitude = xData[i]; 
       
	}
     }
     
    if (table->hasColumn( "MAGERR" )) 
    { 
        const ColumnData *columnData = table->column( "MAGERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].magnitudeerror = xData[i]; 
       
	}
     }
     

    
    if (table->hasColumn( "NOPIXELS" )) 
    { 
        const ColumnData *columnData = table->column( "NOPIXELS" )->data();
	    

	
       
        int32 *xData = columnData->int32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].nosourcepixels = xData[i]; 
       	 
       

	}
     }
     


   
 
     
    if (table->hasColumn( "APERTURE" )) 
    { 
        const ColumnData *columnData = table->column( "APERTURE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].photometryradius = xData[i]; 
       
	}
     }





    if (table->hasColumn( "FWHM_MAJ" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MAJ" )->data();
	    

     

       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semia = xData[i]; 
       
	}
    }

    if (table->hasColumn( "FWHM_MAJ_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MAJ_ERR" )->data();
	    

   

       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semiaerr = xData[i]; 
       
	}
    }





    if (table->hasColumn( "FWHM_MIN" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MIN" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semib = xData[i]; 
       
	}
     }
     


    if (table->hasColumn( "FWHM_MIN_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "FWHM_MIN_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semiberr = xData[i]; 
       
	}
     }
     




    if (table->hasColumn( "PA" )) 
    { 
        const ColumnData *columnData = table->column( "PA" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].theta = xData[i]; 
       
	}
     }
     

    if (table->hasColumn( "PA_ERR" )) 
    { 
        const ColumnData *columnData = table->column( "PA_ERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].thetaerr = xData[i]; 
	         

	}
     }
     

  


    if (table->hasColumn( "EFLAG" )) 
    { 
        const ColumnData *columnData = table->column( "EFLAG" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].extensionflag = xData[i]; 
	  
        
	}
     }
     
 

    if (table->hasColumn( "QFLAG" )) 
    { 
        const ColumnData *data = table->column( "QFLAG" )->data();
	    
     
      
	int type = data->dataType();
	//        cout << "datatype=" << type << endl;
        switch (type ) 
        {
	    case 2:
	    {     
	        int8 *xData = data->int8Data();
	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	  
       
	        }
                break;
	    }
        

	    case 4: 
	    {     
	        int16 *xData = data->int16Data();

        


	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	            
       
	        }
                break;
	    }
	    default:
	    {
	        cout << "bug";
	        exit(0);
            } 

        }




     }
     



    if (table->hasColumn( "CFLAG" )) 
    { 
        const ColumnData *columnData = table->column( "CFLAG" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].confusionflag = xData[i]; 
       
	}
     }
            
    dataSetServer->close(dataSet); // Finished with the file



      
    return true;
} 

/**********************************************************************
*  Load all the source data from an OBS file
*
**********************************************************************/
bool loadSourceParameters(const string &fileName, const char* tableName,
			  int& size, vector < OmSourceParameter >& parameters) 
                     

{
  
    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }






    // Open the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
   
 

    parameters.empty();
    parameters.resize( size ); 

    


    // Load the columns one by one

    



    if (table->hasColumn( "POSERR" )) 
    { 
        const ColumnData *columnData = table->column( "POSERR" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].positionerror = xData[i]; 
       
	}
     }
     
    // ************ RA *************************



    if (table->hasColumn( "RA" )) 
    { 
        const ColumnData *columnData = table->column( "RA" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].rightascension = xData[i]; 
       
	}
     }
     

   
    // ************ DEC *************************

    if (table->hasColumn( "DEC" )) 
    { 
        const ColumnData *columnData = table->column( "DEC" )->data();
	    

	
       
        real64 *xData = columnData->real64Data();
       


	for (int i = 0; i < size; i++) 
        {
	   parameters[i].declination = xData[i]; 
       
	}
     }
     


  



    if (table->hasColumn( "MEAN_MAJ_AXIS" )) 
    { 
        const ColumnData *columnData = table->column( "MEAN_MAJ_AXIS" )->data();
	    

     

       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semia = xData[i]; 
       
	}
    }



    if (table->hasColumn( "MEAN_MIN_AXIX" )) 
    { 
        const ColumnData *columnData = table->column( "MEAN_MIN_AXIS" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].semib = xData[i]; 
       
	}
     }
     





    if (table->hasColumn( "MEAN_POS_ANGLE" )) 
    { 
        const ColumnData *columnData = table->column( "MEAN_POS_ANGLE" )->data();
	    

	
       
        real32 *xData = columnData->real32Data();
       


	for (int i = 0; i < size; i++) 
        {
	  parameters[i].theta = xData[i]; 
       
	}
     }
     

  


    if (table->hasColumn( "EFLAG_MAX" )) 
    { 
        const ColumnData *columnData = table->column( "EFLAG_MAX" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].extensionflag = xData[i]; 
	  
        
	}
     }
     
 

    if (table->hasColumn( "QFLAG_COMB" )) 
    { 
        const ColumnData *data = table->column( "QFLAG_COMB" )->data();
	    

	int type = data->dataType();
	//        cout << "datatype=" << type << endl;
        switch (type ) 
        {
	    case 2:
	    {     
	        int8 *xData = data->int8Data();
	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	  
       
	        }
                break;
	    }

	    case 4: 
	    {     
       


	        int16 *xData = data->int16Data();
	        for (int i = 0; i < size; i++) 
                {
	            parameters[i].qualityflag = xData[i]; 
	  
		         
	        }
                break;
	    }
	    default:
	    {
	        cout << "bug";
	        exit(0);
            } 

        }







     }
     



    if (table->hasColumn( "CFLAG_COMB" )) 
    { 
        const ColumnData *columnData = table->column( "CFLAG_COMB" )->data();
	    

	
       
        int8 *xData = columnData->int8Data();
       


	for (int i = 0; i < size; i++) 
        {
	    parameters[i].confusionflag = xData[i]; 
       
	}
     }
            
    dataSetServer->close(dataSet); // Finished with the file



      
    return true;
} 
/**********************************************************************
*
*********************************************************************/
bool upDateSrcListFile(const string &fileName, const char* tableName,
		       vector< OmSourceParameter > & parameters)
{ 
                     


    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);
   
    
    if (!dataSet->hasBlock(tableName)) 
    {
        dataSetServer->close(dataSet); // Finished with the file
        exit(0); 
        return false;

    }

    // Open the table
    Table *table = dataSet->table(tableName);


    int size = table->rows();


    ColumnData *columnData = table->column( "QFLAG")->data();

	
    int16* data1 = columnData->int16Data();


    for (int i = 0; i < size; i++) 
    {
        data1[i] = parameters[i].qualityflag; 
       
    }
    columnData->release();    



    columnData = table->column( "EFLAG")->data();

	
    int8 *data = columnData->int8Data();


    for (int i = 0; i < size; i++) 
    {
        data[i] = parameters[i].extensionflag; 
       
    }
    columnData->release();    



    columnData = table->column( "CFLAG")->data();

	
    data = columnData->int8Data();


    for (int i = 0; i < size; i++) 
    {
        data[i] = parameters[i].confusionflag; 
       
    }
    columnData->release();    



    dataSetServer->close(dataSet); // Finished with the file

  

    return true;
}




/**********************************************************************
*
*********************************************************************/
bool createSrcListFile(const string newSrcFile, const string oldSrcFile, vector< OmSourceParameter > & parameters)
{ 
                     
    cout << "In create"<<endl;
    dataSetServer->copy( oldSrcFile, newSrcFile );
    cout << "In cr2eate"<<endl;

    // Open the file
    DataSet *dataSet = dataSetServer->open(newSrcFile, Dal::Modify);
  
    if ( !dataSet ) return false;
    cout << "line 2121\n";
    dataSet->deleteBlock( "SRCLIST" );

    // Add the table
    cout<<"Adding the SRCLIST table"<<endl;
    Table *table = dataSet->addTable( "SRCLIST", parameters.size() );




    vector<real32> data;
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].xcoord );


    if ( !addColumn( table, "XPOS", "pixels", "x coordinate",   
			   (int)parameters.size(), data) )
      return false;



    data.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].ycoord );

    if ( !addColumn( table, "YPOS", "pixels", "y coordinate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();


    vector<real64> data1;
    for (unsigned i = 0; i < parameters.size(); i++)
        data1.push_back( parameters[i].rightascension );


    if ( !addColumn( table, "RA", "degrees", "Right-ascension",   
			   (int)parameters.size(), data1) )
      return false;

    data1.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data1.push_back( parameters[i].declination );


    if ( !addColumn( table, "DEC", "degrees", "Declination",   
			   (int)parameters.size(), data1) )
      return false;

    data1.clear();


    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].positionerror );


    if ( !addColumn( table, "POSERR", "arc secs", "Position error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    /**********************************************************************
     * Count rates
     ********************************************************************/

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrate );


    if ( !addColumn( table, "RATE", "counts/sec", "Raw count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrateerror );


    if ( !addColumn( table, "RATE_ERR", "counts/sec", "Raw count-rate error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawbackgroundcountrate );


    if ( !addColumn( table, "BACKGROUND_RATE", "counts/sec", "Raw background count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].significance );


    if ( !addColumn( table, "SIGNIFICANCE", "", "Signa-to-noise",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrate );


    if ( !addColumn( table, "CORR_RATE", "counts/sec", "Corrected count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrateerror );


    if ( !addColumn( table, "CORR_RATE_ERR", "counts/sec", "Corrected count-rate error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrbackgroundcountrate );


    if ( !addColumn( table, "CORR_BACKGROUND_RATE", "counts/sec", "Corrected background count-rate",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    /*****************************************************************************
     * Corrections applied
     ****************************************************************************/ 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf1correction );


    if ( !addColumn( table, "PSF1_CORR", "", "PSF1 correction",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf2correction );


    if ( !addColumn( table, "PSF2_CORR", "", "UV PSF2 correction",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    /*************************************************************************************
     * Instrumental magnitudes
     ************************************************************************************/
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitude);


    if ( !addColumn( table, "MAG", "mag", "Instrumental magnitude",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitudeerror);


    if ( !addColumn( table, "MAGERR", "mag", "Instrumental magnitude error",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

    vector<int> data3;
    for (unsigned i = 0; i < parameters.size(); i++)
        data3.push_back( parameters[i].nosourcepixels);


    if ( !addColumn( table, "NOPIXELS", "", "No of source pixels (extended sources)",   
			   (int)parameters.size(), data3) )
      return false;


    data3.clear();    

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].photometryradius);


    if ( !addColumn( table, "APERTURE", "unbinned pixels", "Extraction aperture radius (point-sources)",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semia);


    if ( !addColumn( table, "FWHM_MAJ", "arcsec", "FWHM major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiaerr);


    if ( !addColumn( table, "FWHM_MAJ_ERR", "arcsecs", "FWHM major-axis error of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semib);


    if ( !addColumn( table, "FWHM_MIN", "arcsecs", "FWHM minor-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiberr);


    if ( !addColumn( table, "FWHM_MIN_ERR", "arcsecs", "FWHM minor-axis error of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 
    
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].theta);


    if ( !addColumn( table, "PA", "counts/sec", "Position angle of major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 

 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].thetaerr);


    if ( !addColumn( table, "PA_ERR", "counts/sec", "Error in position-angel of major-axis of source",   
			   (int)parameters.size(), data) )
      return false;


    data.clear();
 




    vector<int8> data4;
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].extensionflag);


    if ( !addColumn( table, "EFLAG", "", "Source extension flag (0-point, 1-extended)",   
			   (int)parameters.size(), data4) )
      return false;


    data4.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].qualityflag);


    if ( !addColumn( table, "QFLAG", "", "Quality flag",   
			   (int)parameters.size(), data4) )
      return false;


    data4.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].confusionflag);


    if ( !addColumn( table, "CFLAG", "", "Confusion flag",   
			   (int)parameters.size(), data4) )
      return false;


    data4.clear();
 

    //    table->insertRows( (int)parameters.size(), 100);

    dataSetServer->close( dataSet );

   
    return true;
}

/**********************************************************************
*  Appends new rows to the end of the source-list file
*
*********************************************************************/
bool appendToSrcListFile( const string newSrcFile, vector< OmSourceParameter > & parameters )
{ 
             


    // Open the file
    DataSet *dataSet = dataSetServer->open( newSrcFile, Dal::Modify);
  
   
    if ( !dataSet ) return false;
  

    // Update the table
    Table *table = dataSet->table( "SRCLIST", Dal::Modify );
    int iStart = table->rows();


    // Insert new rows at the end of the file
    table->insertRows( iStart, parameters.size() );
 

    vector<int32> data3;
    for (unsigned i = 0; i < parameters.size(); i++)
        data3.push_back( iStart + i + 1 );


    if ( !appendToColumn( table, "SRCNUM",  iStart, data3) )
      return false;

    data3.clear();

  


    vector<real32> data;
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].xcoord );


    if ( !appendToColumn( table, "XPOS",  iStart, data) )
      return false;



    data.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].ycoord );

    if ( !appendToColumn( table, "YPOS", iStart, data) )
      return false;


    data.clear();


        vector<real64> data1;
    //    for (unsigned i = 0; i < parameters.size(); i++)
    //        data1.push_back( parameters[i].rightascension );


    //    if ( !appendToColumn( table, "RA", iStart, (int)parameters.size(), data1) )
    //      return false;

    //    data1.clear();
    //    for (unsigned i = 0; i < parameters.size(); i++)
    //        data1.push_back( parameters[i].declination );


    //    if ( !appendToColumn( table, "DEC", iStart, (int)parameters.size(), data1) )
    //     return false;

	//    data1.clear();


    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].positionerror );


    if ( !appendToColumn( table, "POSERR", iStart, data) )
      return false;


    data.clear();
 

    /**********************************************************************
     * Count rates
     ********************************************************************/

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrate );


    if ( !appendToColumn( table, "RATE", iStart, data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawcountrateerror );


    if ( !appendToColumn( table, "RATE_ERR", iStart, data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rawbackgroundcountrate );


    if ( !appendToColumn( table, "BACKGROUND_RATE", iStart, data) )
      return false;


    data.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].significance );


    if ( !appendToColumn( table, "SIGNIFICANCE", iStart, data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrate );


    if ( !appendToColumn( table, "CORR_RATE", iStart,  data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrcountrateerror );


    if ( !appendToColumn( table, "CORR_RATE_ERR", iStart,  data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].corrbackgroundcountrate );


    if ( !appendToColumn( table, "CORR_BACKGROUND_RATE", iStart,  data) )
      return false;


    data.clear();
 
    /*****************************************************************************
     * Corrections applied
     ****************************************************************************/ 





    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf1correction );


    if ( !appendToColumn( table, "PSF1_CORR", iStart,  data) )
      return false;


    data.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].psf2correction );


    if ( !appendToColumn( table, "PSF2_CORR", iStart, data) )
      return false;


    data.clear();





    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].rightascension );


    if ( !appendToColumn( table, "SPB_COILOSS", iStart, data) )
        return false;

    data.clear();
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].declination );


    if ( !appendToColumn( table, "BK_COILOSS", iStart,  data) )
        return false;

    data.clear();








 

    /*************************************************************************************
     * Instrumental magnitudes
     ************************************************************************************/
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitude);


    if ( !appendToColumn( table, "MAG", iStart,  data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].magnitudeerror);


    if ( !appendToColumn( table, "MAGERR", iStart,  data) )
      return false;


    data.clear();
 

   
    for (unsigned i = 0; i < parameters.size(); i++)
        data3.push_back( parameters[i].nosourcepixels);


    if ( !appendToColumn( table, "NOPIXELS", iStart,  data3) )
      return false;


    data3.clear();    

    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].photometryradius);


    if ( !appendToColumn( table, "APERTURE", iStart,  data) )
      return false;


    data.clear();
     
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semia);


    if ( !appendToColumn( table, "FWHM_MAJ", iStart,  data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiaerr);


    if ( !appendToColumn( table, "FWHM_MAJ_ERR", iStart,  data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semib);


    if ( !appendToColumn( table, "FWHM_MIN", iStart,  data) )
      return false;


    data.clear();
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].semiberr);


    if ( !appendToColumn( table, "FWHM_MIN_ERR", iStart,  data) )
      return false;


    data.clear();
 
    
 
    
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].theta);


    if ( !appendToColumn( table, "PA", iStart,  data) )
      return false;


    data.clear();
 

 
    for (unsigned i = 0; i < parameters.size(); i++)
        data.push_back( parameters[i].thetaerr);


    if ( !appendToColumn( table, "PA_ERR", iStart, data) )
      return false;


    data.clear();
 




    vector<int8> data4;
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].extensionflag);


    if ( !appendToColumn( table, "EFLAG", iStart, data4) )
      return false;


    data4.clear();
 

    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].qualityflag);


    if ( !appendToColumn( table, "QFLAG", iStart,  data4) )
      return false;


    data4.clear();
 
    for (unsigned i = 0; i < parameters.size(); i++)
        data4.push_back( parameters[i].confusionflag);


    if ( !appendToColumn( table, "CFLAG", iStart,  data4) )
      return false;


    data4.clear();
 

    dataSetServer->close( dataSet );

   
    return true;
}






/********************************************************
*
*
******************************************************/
vector<float> LoadVectorFloatColumnData(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
                      int &size, int il1, int iu1)
{
     


  
   
    // Open the file
    size = 0;
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);
    if (!dataSet)
    {
      exit(0);

    }    


    if (!dataSet->hasBlock(tableName)) 
    {
        cout << "Table doesn't exist\n";
        dataSetServer->close(dataSet); // Finished with the file
        exit(0);  
      

    }


    // Read the table
    const Table *table = dataSet->table(tableName);
  
    size = table->rows();
    int il, iu;
    if (iu1 < il1)
    {
      il = 0;
      iu = size - 1; 
    }
    else
    {
        il = il1,
	iu = iu1;
        size = iu1 - il1 + 1;

    } 


    vector<float> data(size, -1.0);

  
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        float *xData = columnData->real32Data();
       

        
	for (int i = il, k = 0; i <= iu; i++, k++) 
        {
	  data[k] = xData[i]; 
	 
	}

 

     }
         
    dataSetServer->close(dataSet); // Finished with the file
 
    return data;
};


/*************************************************************************
*
*
**************************************************************************/
bool addVectorFloatColumn(string fileName, 
                          string tableName, 
                          string columnName,
                          string comment,
                          string units,
                          int nCols,
                          vector< vector<float> > &data,
                          int pos)
{



    DataSet* set = dataSetServer->open(fileName ,DataSet::Modify);
    Table* tab = set->table( tableName, Dal::Modify);
    Column *col;
    ColumnData *d;

    unsigned long vectorLen = nCols;
    vector<unsigned long> size;
    size.push_back(vectorLen);

    int position = pos;
    if(pos < 0) position = tab->columns();
    col = tab->addColumn(columnName,Column::Real32, comment, units, size, position);



    assert(col->size() == size);
    assert(col->elements() == vectorLen);

    d = col->data();

    assert(d->elements() == vectorLen);
    assert(d->size() == size);
 

    real32* r32 = d->real32Data();
    for (unsigned long row = d->from(); row < d->from()+d->count(); ++row) 
    {
        for (unsigned long element = 0; element < data[row].size(); ++element) 
        {
            unsigned long pos = d->position(row,element);
            assert( pos == row * vectorLen + element );

            if( abs( data[row][element] + 9999.9) < 1e-3)
                d -> setToNull( pos );
 
            else

                r32[d->position(row, element)] = data[row][element];
        }

        for (unsigned long element = data[row].size(); element < d->elements(); ++element) 
        {
            unsigned long pos = d->position(row,element);
            assert( pos == row * vectorLen + element );
        
            d -> setToNull( pos );
 

        }




    }
    delete d;

 
    dataSetServer->close(set); // Finished with the file

    return true;
}

/*************************************************************************
*
*
**************************************************************************/
bool addVectorDoubleColumn(string fileName, 
                          string tableName, 
                          string columnName,
                          string comment,
                          string units,
                          int nCols,
                          vector< vector<double> > &data,
                          int pos)
{



    DataSet* set = dataSetServer->open(fileName ,DataSet::Modify);
    Table* tab = set->table( tableName, Dal::Modify);
    Column *col;
    ColumnData *d;

    unsigned long vectorLen = nCols;
    vector<unsigned long> size;
    size.push_back(vectorLen);



    int position = pos;
    if(pos < 0) position = tab->columns();



    col = tab->addColumn(columnName,Column::Real64, comment, units, size, position);
  assert(col->size() == size);
  assert(col->elements() == vectorLen);

    d = col->data();

    assert(d->elements() == vectorLen);
    assert(d->size() == size);
 

    real64* r32 = d->real64Data();
    for (unsigned long row = d->from(); row < d->from()+d->count(); ++row) 
    {
        for (unsigned long element = 0; element < data[row].size(); ++element) 
        {
            unsigned long pos = d->position(row,element);
            assert( pos == row * vectorLen + element );

            if( abs( data[row][element] + 9999.9) < 1e-3)
                d -> setToNull( pos );
 
            else

                r32[d->position(row, element)] = data[row][element];
        }

        for (unsigned long element = data[row].size(); element < d->elements(); ++element) 
        {
            unsigned long pos = d->position(row,element);
            assert( pos == row * vectorLen + element );
        
            d -> setToNull( pos );
 

        }




    }
    delete d;

 


    dataSetServer->close(set); // Finished with the file

    return true;
}









/*************************************************************************
*
*
**************************************************************************/
bool addVectorInt8Column(string fileName, 
                          string tableName, 
                          string columnName,
                          string comment,
                          string units,
                          int nCols,
                          vector< vector<int8> > &data,
                          int pos)
{



    DataSet* set = dataSetServer->open(fileName ,DataSet::Modify);
    Table* tab = set->table( tableName, Dal::Modify);
    Column *col;
    ColumnData *d;

    unsigned long vectorLen = nCols;
    vector<unsigned long> size;
    size.push_back(vectorLen);



    int position = pos;
    if(pos < 0) position = tab->columns();

    col = tab->addColumn( columnName, Column::Int8, comment, units, size, position);
  

    d = col->data();

    col -> nullValue( 127);
   
    int8* r32 = d->int8Data();

    for (unsigned long row = d->from(); row < d->from()+d->count(); ++row) 
    {



    

        for (unsigned long element = 0; element < data[row].size(); ++element) 
        {
       
            unsigned long pos = d->position(row,element);
            assert( pos == row * vectorLen + element );

            if( data[row][element] == (int8)127 )
                d -> setToNull( pos );
 
            else
                r32[d->position(row, element)] = data[row][element];
        }

        for (unsigned long element = data[row].size(); element < d->elements(); ++element) 
        {
            unsigned long pos = d->position(row,element);
            assert( pos == row * vectorLen + element );
        
            d -> setToNull( pos );
 

        }




    }
    delete d;

 


    dataSetServer->close(set); // Finished with the file

    return true;
}











/*************************************************************************
*
*
**************************************************************************/
bool addVectorInt16Column(string fileName, 
                          string tableName, 
                          string columnName,
                          string comment,
                          string units,
                          int nCols,
                          vector< vector<int16> > &data,
                          int pos)
{



    DataSet* set = dataSetServer->open(fileName ,DataSet::Modify);
    Table* tab = set->table( tableName, Dal::Modify);
    Column *col;
    ColumnData *d;

    unsigned long vectorLen = nCols;
    vector<unsigned long> size;
    size.push_back(vectorLen);



    int position = pos;
    if(pos < 0) position = tab->columns();

    col = tab->addColumn( columnName, Column::Int16, comment, units, size, position);
  

    d = col->data();

    col -> nullValue( -9999);
   
    int16* r32 = d->int16Data();

    for (unsigned long row = d->from(); row < d->from()+d->count(); ++row) 
    {



    

        for (unsigned long element = 0; element < data[row].size(); ++element) 
        {
       
            unsigned long pos = d->position(row,element);
            assert( pos == row * vectorLen + element );

            if( data[row][element] == -9999 )
                d -> setToNull( pos );
 
            else
                r32[d->position(row, element)] = data[row][element];
        }

        for (unsigned long element = data[row].size(); element < d->elements(); ++element) 
        {
            unsigned long pos = d->position(row,element);
            assert( pos == row * vectorLen + element );
        
            d -> setToNull( pos );
 

        }




    }
    delete d;

 


    dataSetServer->close(set); // Finished with the file

    return true;
}







/*************************************************************************
*
*
**************************************************************************/
bool addVectorInt32Column(string fileName, 
                          string tableName, 
                          string columnName,
                          string comment,
                          string units,
                          int nCols,
                          vector< vector<int32> > &data,
                          int pos)
{



    DataSet* set = dataSetServer->open(fileName ,DataSet::Modify);
    Table* tab = set->table( tableName, Dal::Modify);
    Column *col;
    ColumnData *d;

    unsigned long vectorLen = nCols;
    vector<unsigned long> size;
    size.push_back(vectorLen);



    int position = pos;
    if(pos < 0) position = tab->columns();

    col = tab->addColumn( columnName, Column::Int32, comment, units, size, position);
  

    d = col->data();

    col -> nullValue( -9999);
   
    int32* r32 = d->int32Data();

    for (unsigned long row = d->from(); row < d->from()+d->count(); ++row) 
    {



    

        for (unsigned long element = 0; element < data[row].size(); ++element) 
        {
       
            unsigned long pos = d->position(row,element);
            assert( pos == row * vectorLen + element );

            if( data[row][element] == -9999 )
                d -> setToNull( pos );
 
            else
                r32[d->position(row, element)] = data[row][element];
        }

        for (unsigned long element = data[row].size(); element < d->elements(); ++element) 
        {
            unsigned long pos = d->position(row,element);
            assert( pos == row * vectorLen + element );
        
            d -> setToNull( pos );
 

        }




    }
    delete d;

 


    dataSetServer->close(set); // Finished with the file

    return true;
}







