// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef SOURCELIST_H
#define SOURCELIST_H
#include <vector>
#include <qstring.h>
#include <f90cxx.h>
#include <typeinfo>
#include <Dal.h>
#include <errstr.h>
#include <iostream>

#include "OmSourceParameters.h"


struct OmSourceParameter;


int GetNoRows(const string& fileName, const string& tableName);

float *LoadColumnData(const string &fileName, const 
       string &tableName, const string& columnName, int &size);


double *LoadDoubleColumnData(const string &fileName, const 
       string &tableName, const string& columnName, int &size);








/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real32> &data, 
	             int pos=-1);

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real64> &data, 
	             int pos=-1);

     
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int8> &data, 
	             int pos=-1);

     
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int16> &data, 
	             int pos=-1);

     

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<int32> &data, 
	             int pos=-1);

     










  
/********************************************************
*
*
******************************************************/
bool LoadColumnData(const Table *table,  const string& columnName, vector<float> &);

bool LoadColumnData(const Table *table,  const string& columnName, vector<double> &);
bool LoadColumnData(const Table *table,  const string& columnName, vector<int> &);

bool LoadColumnData(const Table *table,  const string& columnName, vector<int8> &);
bool LoadColumnData(const Table *table,  const string& columnName, vector<int16> &);






/********************************************************
*
*
******************************************************/

vector<string> LoadVectorStringColumnData(const string& fileName, 
                       const string& tableName, 
                       const string& columnName, 
				       int &size);
   

 


vector<double> LoadVectorDoubleColumnData(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
		      int &size);
bool LoadVectorFloatColumnData(const Table *table, 
                      const string& columnName, 
		      int &size,
                      vector<float> &data);



bool LoadVectorDoubleColumnData(const Table *table, 
                      const string& columnName, 
		      int &size,
                      vector<double> &data);


vector<float> LoadVectorFloatColumnData(const string& fileName, 
                      const string& tableName, 
                      const string& columnName, 
		      int &size, int il = 0, int iu = -1);




double *LoadColumnData1(const string &fileName, const 
       string &tableName, const string& columnName, int &size);


int  *LoadIntColumnData(const string &fileName, const 
      string& tableName, const string& columnName, int &size);



vector<int> LoadVectorIntColumnData(const string &fileName, const 
      string& tableName, const string& columnName, int &size);



vector<int8> LoadVectorInt8ColumnData(const string &fileName, const 
      string& tableName, const string& columnName, int &size);


vector<int16> LoadVectorInt16ColumnData(const string &fileName, const 
      string& tableName, const string& columnName, int &size);

vector<int32> LoadVectorInt32ColumnData(const string &fileName, const 
      string& tableName, const string& columnName, int &size);






vector<double> LoadVectorDoubleColumnData(const string &fileName, const 
      string& tableName, const string& columnName, int &size);







Integer1  *LoadInt1ColumnData(const string &fileName, const 
			      string& tableName, const string& columnName, int &size);








float *LoadColumnData(const Table* table, const string& columnName);


double *LoadDoubleColumnData(const Table* table, const string& columnName);


double *LoadColumnData1(const Table* table, const string& columnName);






int  *LoadIntColumnData(const Table* table, const string& columnName);











Integer1  *LoadInt1ColumnData(const Table*, const string& columnName);

Integer2  *LoadInt2ColumnData(const Table*, const string& columnName);


float GetAttribute(const string& fileName, const string& tableName, const string& name);

string GetAttribute(const string& fileName, const string& name);

bool GetBooleanAttribute(const string& fileName, const string& tableName, const string& name);

void addAttribute(const string& fileName, const string& name, const string&comment, const string & value);
void addAttribute(const string& fileName, const string& name, const string&comment, const bool & value);
void addAttribute(const string& fileName, const string& name, const string&comment, const float & value);
void addAttribute(const string& fileName, const string& name, const string&comment, const double & value);
void addAttribute(const string& fileName, const string& name, const string&comment, const int & value);


vector<float> GetAttributes(const string& fileName, const string& tableName, const string nameList[], int n);

vector<QString> loadDirectoryFiles(const string directory, 
				   const int type, const string filter, const string fits = "FTZ" );

vector<QString> loadDirectoryFiles(const string directory, 
				   const int type);



vector<QString> loadDirectoryFiles(char *filter, char c);
vector<QString> loadDirectoryFiles(char *filter1, char *filter2, char c);
vector<QString> loadDirectoryFiles(char *filter);
 




bool deleteRowFromTable(const string& fileName, 
                        const string& tableName, 
                        const int rowNumber);

void setColumnData( const int size, ColumnData* columnData, const real32* data, const int start = 0);
void setColumnData( const int size, ColumnData* columnData, const real64* data, const int start = 0);
void setColumnData( const int size, ColumnData* columnData, const int8* data, const int start = 0);
void setColumnData( const int size, ColumnData* columnData, const int16* data, const int start = 0);
void setColumnData( const int size, ColumnData* columnData, const int32* data, const int start = 0);
void setColumnData( const int size, ColumnData* columnData, const int width, const char* data, const int start = 0);

// For vectors

void setColumnData( const int size, ColumnData* columnData, const vector<real32>&  data, const int start = 0 );
void setColumnData( const int size, ColumnData* columnData, const vector<real64>&  data, const int start = 0 );
void setColumnData( const int size, ColumnData* columnData, const vector<int8>&    data, const int start = 0 );
void setColumnData( const int size, ColumnData* columnData, const vector<int16>&   data, const int start = 0 );
void setColumnData( const int size, ColumnData* columnData, const vector<int32>&   data, const int start = 0 );


/**********************************************************************************
*
*
*
*
**********************************************************************************/
bool appendToColumn(Table *table,
                     const string columnName,
                     const int iStart,
		     const vector<int8> &data);

bool appendToColumn(Table *table,
                     const string columnName,
                     const int iStart,
		     const vector<int16> &data);

bool appendToColumn(Table *table,
                     const string columnName,
                     const int iStart,
		     const vector<int32> &data);
bool appendToColumn(Table *table,
                    const string columnName,
                    const  int iStart,
		    const vector<float> &data);

bool appendToColumn(Table *table,
                    const string columnName,
                    const int iStart,
		    const vector<double> &data);




bool appendToColumn(Table *table,
               const string columnName,
	       const int iStart,
               const vector<string> &data);










/******************************************************
*
******************************************************/
bool appendToColumn( Table *tab, 
                     const string columnName, 
                     const int iStart,   
                     const vector< vector<float> > &data);
 




/******************************************************
*
******************************************************/
bool appendToColumn( Table *tab, 
                     const string  columnName, 
                     const int iStart,   
                     const vector< vector<double> > &data);
 



/******************************************************
*
******************************************************/
bool appendToColumn( Table *tab, 
                     const string  columnName, 
                     const int iStart,   
                     const vector< vector<int8> > &data);
 



/******************************************************
*
******************************************************/
bool appendToColumn( Table *tab, 
                     const string columnName, 
                     const int iStart,   
                     const vector< vector<int16> > &data);
 




/******************************************************
*
******************************************************/
bool appendToColumn( Table *tab, 
                     const string columnName, 
                     const int iStart,   
                     const vector< vector<int32> > &data);
 











/********************************************************
*
*
******************************************************/
template <class T>
bool replaceTableColumn(const string& fileName,
                        const string& tableName,
                        const string& columnName, const T *data)
{

    if (data == 0)
    {
        cout << "Data not allocated\n";
        return false;

    }

    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Modify);


    if (!dataSet)
    {
        cout << "failure to open data set\n";
        return false;
    }




    // Read the table

    if (!dataSet->hasBlock(tableName))
    {
      cout << "Table doesn't exist\n";
      dataSetServer->close(dataSet); // Finished with the file
      return false;

    }


    Table *table = dataSet->table(tableName);


    if (table->hasColumn(columnName))
    {

        const ColumnData *columnData = table->column(columnName)->data();
      

        int size = table->rows();

	//        setColumnData(size, columnData, data);


	string name = typeid(data).name();
	cout << "Line 133 name=" << name << endl;
	if (name == "fred")
	  {


	  }



	int k = Data::Int8;
        switch( k )
        {
            case Data::Bool :

                errstr << "Boolean Null Values are not supported." << Sas::error( "notSupported" );

                break;
            case Data::Int8 :
	    {   
                int8 * i8 = columnData->int8Data();
                for(int i = 0; i < size; i++)
		  i8[i] = data[i];


                break;

            }
            case Data::Int16 :
	    {
                int16 * i16 = columnData->int16Data();
                for(int i = 0; i < size; i++)
		    i16[i] = data[i];

                break;
            }
            case Data::Int32 :
	    {
                int32 * i32 = columnData->int32Data();
                for(int i = 0; i < size; i++)
		  i32[i] = data[i];

                break;
            }
            case Data::Real32 :
	    {
                real32 * r32 = columnData->real32Data();
                for(int i = 0; i < size; i++)
		  r32[i] = data[i];

                break;
            }
            case Data::Real64 :
	    {
                real64 * r64 = columnData->real64Data();
                for(int i = 0; i < size; i++)
		  r64[i] = data[i];

                break;
            }
            case Data::DString :
                errstr << "String Null Values are not supported." << Sas::error( "notSupported" );
                break;
            default :
                errstr << "Invalid data type." << Sas::error( "internalError" );
                break;
		} 
       
    }

    dataSetServer->close(dataSet);


    return true;
};





/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName,
	       const string tableName,   
               const string columnName, 
               const string units,
               const string comment,   
	       int nData,
	       vector<real32> &data, 
               int position = -1);

/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName,
	       const string tableName,   
               const string columnName, 
               const string units,
               const string comment,   
	       int nData,
	       vector<real64> &data, 
               int position = -1);







/********************************************************
*
*
******************************************************/
bool addColumn(const string fileName,
	       const string tableName,   
               const string columnName, 
               const string units,
               const string comment,   
	       int nData,
	       vector<int> &data, 
               int position = -1);





bool addColumn(const string fileName,
               const string tableName,
               const string columnName,
               const string units,
               const string comment,
               int columnWidth, 
               int nData,
               vector<string> &data, 
               int position = -1);









bool createSrcListFile(const string fileName, const string imagefile, vector< OmSourceParameter > & parameters);

bool appendToSrcListFile(const string fileName, vector< OmSourceParameter > & parameters);



bool upDateSrcListFile(const string &fileName, const char* tableName,
                       vector< OmSourceParameter > & parameters);



bool loadSourceParameters(const string &fileName, const char* tableName,
                          int& size, vector < OmSourceParameter > &parameters, string &filter, string &expId);


bool loadSourceParameters(const string &fileName, const char* tableName,
                          int& size, vector < OmSourceParameter > &parameters);







/*************************************************************************
*
*
**************************************************************************/
bool addVectorFloatColumn(string fileName, 
                          string tableName, 
                          string columnName,
                          string comment,
                          string units,
                          int nCols,
                          vector< vector<float> > &data, 
                          int position = -1);



/*************************************************************************
*
*
**************************************************************************/
bool addVectorDoubleColumn(string fileName, 
                          string tableName, 
                          string columnName,
                          string comment,
                          string units,
                          int nCols,
                          vector< vector<double> > &data, 
                          int position = -1);



/*************************************************************************
*
*
**************************************************************************/
bool addVectorInt8Column(string fileName, 
                          string tableName, 
                          string columnName,
                          string comment,
                          string units,
                          int nCols,
                          vector< vector<int8> > &data, 
                          int position = -1);







/*************************************************************************
*
*
**************************************************************************/
bool addVectorInt16Column(string fileName, 
                          string tableName, 
                          string columnName,
                          string comment,
                          string units,
                          int nCols,
                          vector< vector<int16> > &data, 
                          int position = -1);




/*************************************************************************
*
*
**************************************************************************/
bool addVectorInt32Column(string fileName, 
                          string tableName, 
                          string columnName,
                          string comment,
                          string units,
                          int nCols,
                          vector< vector<int32> > &data, 
                          int position = -1);






#endif
