// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

#ifndef STATISTICS_H
#define STATISTICS_H

#include<vector>
#include<iostream>
#include <cmath>
#include <algorithm>
#include<numeric>
#include<iomanip>
#include <cstring>
#include "polint.h"
#include "leastsquares.h"
#include "utility.h"

 
#include <f90cxx.h>





/***************************************************************
*
*
****************************************************************/
template<class kind>
kind Median(vector<kind> &data1, bool sorted = false, bool nulls = true)
{

    
    vector<kind> data;
    if (nulls)
    {
        for(unsigned i = 0; i < data1.size(); i++)
            if( !isNull(data1[i]) ) data.push_back( data1[i] );
    }
    else
        data = data1;


    if(!sorted)
    { 
        sort(data.begin(), data.end());
    }



    double median;
    int nData = data.size();

    if (nData == 0) return (kind)0.0;
    if (nData == 1) return (kind)data[0]; 

    int n2 = nData / 2;

    if ( n2 * 2 == nData ) // Even number of points
        median = ((double)data[n2 - 1] + (double)data[n2]) / 2.0;
    else
      median = (kind)data[n2];

    return (kind)median;
}    



/***************************************************************
*
*
****************************************************************/
template<class kind>
kind Mean(kind *data, long nData)
{

    double mean = 0.0;
    if (nData == 0) return 0.0;

    for( long i = 0; i < nData; i++)
        mean += (double)data[i];

    return (kind)(mean / (double)nData);

}






/***************************************************************
*
*
****************************************************************/
template<class kind>
kind Mean(vector<kind> &data1, bool nulls = true)
{







    if (data1.size() == 0) return (kind)0.0;





    

    vector<kind> data;
    if (nulls)
    {
        for(unsigned i = 0; i < data1.size(); i++)
            if( !isNull(data1[i]) ) data.push_back( data1[i] );
    }
    else
        data = data1;

   




    sort(data.begin(), data.end());




    return (kind)(accumulate( data.begin(), data.end(), 0.0 ) / double(data.size() ));


}





/***************************************************************
*
*
****************************************************************/
template<class kind>
kind Median(kind *data, long nData)
{
    kind median;


    if (nData == 0) return 0.0;
    if (nData == 1) return data[0]; 



  
    int n2 = nData / 2;

    if ( n2 * 2 == nData ) // Even number of points
        median = (data[n2 - 1] + data[n2]) / 2.0;
    else
      median = data[n2];

    return median;
}    







/***************************************************************
*
*
****************************************************************/
template<class kind1, class kind2>
kind2 Percentile(vector<kind1> &data1, kind2 perCent, bool sorted = false, bool nulls = true)
{



   

    vector<kind1> data;
    if (nulls)
    {
        for(unsigned i = 0; i < data1.size(); i++)
            if( !isNull(data1[i]) ) data.push_back( data1[i] );
    }
    else
        data = data1;

   



    if(!sorted)
    {
        sort(data.begin(), data.end());
    }





    long size = data.size();
    
    long index = (long)(perCent * (float)size + 0.00001) - 1;
    index = max((long)0, index);
    index = min(index, size - 1); 
     
    //    cout << "n=" << size << " percent=" << perCent << " index=" << index << " percentile=" << data[index] << endl;
    //    cout << "data[0]=" << data[0] << " data[" << (size - 1) << "]" << data[size - 1] << endl;
    return (kind2)data[index];

}





/***************************************************************
*
*
****************************************************************/
template<class kind1, class kind2>
kind2 Percentile(kind1 *data, long size, kind2 perCent)
{

 
    
    long index = (long)(perCent * (float)size + 0.00001) - 1;
    index = max((long)0, index);
    index = min(index, size - 1); 

    return (kind2)data[index];

}








F90MODPROCPROTO(double, omfunctions, gammaq)(double*, double*);





#endif




































