// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "utility.h"
#include <string>
#include <cmath>
#include<vector>
#include<numeric>
#include <iostream>
#include <cstdlib>

#include "f90cxx.h"






F90MODPROCPROTO(double, coords, modifiedjuliandate)(int *, int*, int*, int*, int*, int*);



/*******************************************************************
*
*******************************************************************/
double distance(double r1, double d1, double r2, double d2)
{



    return acos( sin(d1) * sin(d2) + cos(d1) * cos(d2) 
	            * cos(r1 - r2) ) * radiansToArcSecs; 


};
/*************************************************************************
*
***************************************************************************/
double Error(double dx, double dy)
{
  if (dx > dy)
    return dx * sqrt(1.0 + (dy / dx) * (dy / dx));
  else
    return dy * sqrt(1.0 + (dx / dy) * (dx / dy));
} 

/*****************************************************************************
*
*****************************************************************************/ 
double getTime(string date)
{
   
    char c;
    int year, month, day, hour, min;
    double sec;

 
    sscanf( date.c_str(), "%4d %c %2d %c %2d %c %2d %c %2d %c %lf", 
                      &year, &c, &month, &c, &day, &c, &hour, &c, &min, &c, &sec); 

  
    int second = (int)sec;



    //    cout <<  "date=" << date << endl;
    //    cout << "year=" << year << " month=" <<  month <<  " day=" <<  day << " hour="
    //	 <<   hour <<  " min=" <<  min <<  " sec=" <<  sec << endl;   

    double result = (F90MODPROC( coords, modifiedjuliandate)( &year, &month, &day, &hour, &min, &second));
    //    cout << "MJD=" << result << endl;

    return result;   

    
}





/*******************************************************
*
*
*******************************************************/


vector<vector<float> >  loadImage(const DataSet *set, const char * name, int &nx, int &ny)
{


    /*****************************************************************
    * Get the size of the image
    *****************************************************************/
    const Array* array = set->array(name);
 

    const ArrayData * arraydata = array->data();
    const float * theData = arraydata->real32Data();
 

    /*****************************************************************
    * Get the size of the image
    *****************************************************************/
 
    longVec size = array->size();
   



    nx = (int)size[1];
    ny = (int)size[0];


    vector< vector<float> > image(nx); 


    for (int i = 0; i < nx; i++)
    {
        image[i].resize(ny); 
        for (int j = 0; j < ny; j++)
        {
	  image[i][j] = theData[i + (j * nx)];  
	 

        }

  
    }


    delete arraydata;
   

    return image;
}

/*****************************************************
*
*****************************************************/

bool isNull( float value)
{



   
    char text[20];

    sprintf(text, "%e", (double)value);

    if (strncmp( text, "nan", 3) == 0 || 
        strncmp( text, "NAN", 3) == 0 || 
        strncmp( text, "NaN", 3) == 0) 
        return true;
    else
        return false;    


};
/*******************************************************
*
******************************************************/
bool isNull( double value)
{



   
    char text[20];

    sprintf(text, "%e", (double)value);

    if (strncmp( text, "nan", 3) == 0 || 
        strncmp( text, "NAN", 3) == 0 || 
        strncmp( text, "NaN", 3) == 0) 
        return true;
    else
        return false;    


};

/*****************************************************************
*
******************************************************************/
bool isNull( int32 value)
{



   
    char text[20];

    sprintf(text, "%e", (double)value);

    if (strncmp( text, "nan", 3) == 0 || 
        strncmp( text, "NAN", 3) == 0 || 
        strncmp( text, "NaN", 3) == 0) 
        return true;
    else
        return false;    


};





bool isNull( int8 value)
{
  if ((int)value == 127)
      return true;
  else
      return false;
}

bool isNull( int16 value)
{
  if ((int)value == 32767)
      return true;
  else
      return false;
}









