// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef UTILITY_H
#define UTILITY_H


#include <cmath>
#include <cerrno>
#include<vector>
#include <algorithm>
#include <numeric>
#include <qmessagebox.h>
#include <iostream>
#include <Dal.h>

const double TWO_PI = 2.0 * M_PI;
const double radiansToDegrees = (double)180.0 / M_PI;
const double radiansToArcSecs = (double)(180. * 3600.) / M_PI;



const double degreesToRadians = M_PI / (double)180.0;



double distance(double r1, double d1, double r2, double d2);

double Error(double mag1, double mag2);




double getTime(string date);

//gcc4.8.2 portability
bool isNull( float value);
bool isNull( double value);
bool isNull( int32 value);
bool isNull( int8 value);
bool isNull( int16 value);




vector<vector<float> >  loadImage(const DataSet *set, const char * name, int &nx, int &ny);


/**********************************************
 * int Locate                                 *
 **********************************************/
template <class kind1, class kind2>
kind2 locate(kind1 *xx, kind2 n, kind1  x, kind2 jl)
{
    if (n == 1) return 0;
    if (xx[0] > x) return 0;
    if (xx[n - 1] < x) return n - 1;

    kind2 ju, jm;
    
    ju = n - 1;
    
    while (ju - jl > 1) {
      jm = (ju + jl) >> 1;
      if (xx[jm] < x)
          jl = jm;
      else
          ju = jm;
    }
    return jl;

};

/************************************************
*
************************************************/



/********************************************************************
*
*********************************************************************/
template<class kind>
kind minVal(kind *data, int size)
{
    kind minData = data[0];
    for(int i = 0; i < size; i++)
      minData = min(minData, data[i]);

    return minData;



}




/********************************************************************
*
*********************************************************************/
template<class kind>
kind minVal(vector<kind> &data1, bool sorted = false, bool nulls = true)
{

 

    vector<kind> data;
    if (nulls)
    {
        for(unsigned i = 0; i < data1.size(); i++)
            if( !isNull(data1[i]) ) data.push_back( data1[i] );
    }
    else
        data = data1;

   

    if(data.size() == 0) return (kind)0;


    if(!sorted)
        std::sort(data.begin(), data.end() );

   



    return data[0];


 
}
/********************************************************************
*
*********************************************************************/
template<class kind>
kind maxVal(kind *data, int size)
{
    kind maxData = data[0];
    for(int i = 0; i < size; i++)
      maxData = max(maxData, data[i]);

    return maxData;



}

/********************************************************************
*
*********************************************************************/
template<class kind>
kind maxVal(vector<kind> &data1, bool sorted = false, bool nulls = true)
{


  //  return (kind)100;  

    vector<kind> data;


    if (nulls)
    {
        for(unsigned i = 0; i < data1.size(); i++)
            if( !isNull(data1[i]) ) data.push_back( data1[i] );
    }
    else
        data = data1;

   


    if(data.size() == 0) return (kind)0;




    if(!sorted)
        std::sort(data.begin(), data.end() );
 



    return data[ data.size() - 1 ];
    



}


/**************************************************************
*
**************************************************************/
template<class kind>
bool getValue(string text, kind &value, kind xMin = 0, kind xMax = 0)
{

    char *endPtr;
    double value1 = strtod(text.c_str(), &endPtr);


    value = (kind) value1;
    


    if( strlen(endPtr) > 0)
    {
     
   

        for(unsigned i = 0; i < strlen(endPtr); i++)
	{
            if( endPtr[i] != ' ') {
                char text1[100];
                sprintf(text1, "Invalid number - %s - Please try again!", text.c_str());
      

                QMessageBox::critical( 0, "OMCATALOGUE", text1);

                return false;
            }
        }

    } 


    if(errno != 0) {
    
        char text1[100];
        sprintf(text1, "Invalid number - %s (%s) - Please try again!", text.c_str(), strerror(errno));
      

        QMessageBox::critical( 0, "OMCATALOGUE", text1);
 

        return false;

    }

   






     if(xMin != xMax)
    {
        if(value < xMin || value > xMax) {

            char text1[100];
            sprintf(text1, "Number %e outside allowed range- %e - %e - Please try again!", value1, (double)xMin, (double)xMax );
      

            QMessageBox::critical( 0, "OMCATALOGUE", text1);
 

            return false;
        }





    }








    
    value = (kind) value1;
    return true;

   




};








/**************************************************************
*
**************************************************************/
template<int>
bool getValue(string text, int &value, int xMin = 0, int xMax = 0)
{

    char *endPtr;
    




 
    value = strtol( text.c_str(), &endPtr, 10);
    


    if( strlen(endPtr) > 0)
    {
     
   

        for(unsigned i = 0; i < strlen(endPtr); i++)
	{
            if( endPtr[i] != ' ') {
                char text1[100];
                sprintf(text1, "Invalid number - %s - Please try again!", text.c_str());
      

                QMessageBox::critical( 0, "OMCATALOGUE", text1);

                return false;
            }
        }

    } 


    if(errno != 0) {
    
        char text1[100];
        sprintf(text1, "Invalid number - %s (%s) - Please try again!", text.c_str(), strerror(errno));
      

        QMessageBox::critical( 0, "OMCATALOGUE", text1);
 

        return false;

    }

    if(xMin != xMax)
    {
        if(value < xMin || value > xMax) {

            char text1[100];
            sprintf(text1, "Number %d outside allowed range- %d - %d - Please try again!", value, xMin, xMax );
      

            QMessageBox::critical( 0, "OMCATALOGUE", text1);
 

            return false;
        }





    }

 
  
    return true;

   
}




/******************************************************
*
******************************************************/
template<class kind>
int noNoneNulls( vector<kind>& data)
{
    int n = 0;
    for(unsigned i = 0; i < data.size(); i++)
      if ( !isNull(data[i]) ) n++;
     
    return n;

}







#endif
