// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "Dal.h"
#include "error.h"
#include <iostream>


#include "SasDalUtils.h"
#include "dal_utilities.h"
#include "DataSubspace.h"

#include "f90cxx.h"

#include<vector>
#include<cmath>
#include<string>
#include<cstring>
/******************************************************
*
*******************************************************/
bool LoadVectorFloatColumnData(const Table *table, 
                      const string& columnName, 
		      int &size, vector<float> &data);

bool LoadVectorDoubleColumnData(const Table *table, 
                      const string& columnName, 
		      int &size, vector<double> &data);
void loadData(string file, vector<double> &ra, vector<double> &dec);



F90MODPROCPROTO( double, coords, chisqprobability)(double *, int *);



   



float computeChiSquared(vector<float> &x, vector<float> &dx, float &theMean, float &prob);

void testChiSquared();


/*****************************************************
*
*****************************************************/

bool isNull( float value)
{



   
    char text[20];

    sprintf(text, "%e", (double)value);

    if (strncmp( text, "nan", 3) == 0 || 
        strncmp( text, "NAN", 3) == 0 || 
        strncmp( text, "NaN", 3) == 0) 
        return true;
    else
        return false;    


};
/*******************************************************
*
******************************************************/
bool isNull( double value)
{



   
    char text[20];

    sprintf(text, "%e", (double)value);

    if (strncmp( text, "nan", 3) == 0 || 
        strncmp( text, "NAN", 3) == 0 || 
        strncmp( text, "NaN", 3) == 0) 
        return true;
    else
        return false;    


};

/*****************************************************************
*
******************************************************************/
bool isNull( int32 value)
{



   
    char text[20];

    sprintf(text, "%e", (double)value);

    if (strncmp( text, "nan", 3) == 0 || 
        strncmp( text, "NAN", 3) == 0 || 
        strncmp( text, "NaN", 3) == 0) 
        return true;
    else
        return false;    


};





bool isNull( int8 value)
{
  if ((int)value == 127)
      return true;
  else
      return false;
}

bool isNull( int16 value)
{
  if ((int)value == 32767)
      return true;
  else
      return false;
}





/***********************************************************************
*
*
*
***********************************************************************/

template<class T>
void compareData(vector<T> & data1, vector<T> &data2, T tolerance)
{

    for(unsigned i = 0; i < data1.size(); i++)
    {
   cout << i << ") data1=" << data1[i] << " 2=" << data2[i] << endl;
        

        if( isNull(data1[i] ) && isNull(data2[i] ) ) continue;
        if( isNull(data1[i] ) || isNull(data2[i] ) ) 
	{
	  cout << "checkoutput.cc line 153\n"; 
	    cout << i << ") data1=" << data1[i] << " 2=" << data2[i] << endl;
            Sas::fatal("Wrong data values in SRCLIST table-", " ");


        }
 

        if( abs( double(data1[i] - data2[i])) > (double)tolerance)
        {
	  cout << i << ") " << data1[i] << " " << data2[i] << " diff=" << data1[i] - data2[i] << endl;
	  cout << "checkoutput.cc line 164\n"; 

          Sas::fatal("Wrong data values in SRCLIST table-", " ");

        }


    }



}









void checkVariabilityColumns( string, string);


/******************************************************************
*
*
*******************************************************************/

int main()
{
  testChiSquared();


    vector<double> ra1, dec1;
    loadData( "checkfile.fits", ra1, dec1);


    vector<double> ra2, dec2; 
    loadData( "masterfile.fits", ra2, dec2);

    
    checkVariabilityColumns( "checkfile.fits", "masterfile.fits");



    if(ra1.size() != ra2.size() )
    {


     

        cout << "number of rows in test3=" << ra1.size() << endl;
        cout << "number of rows in test4=" << ra2.size() << endl;


        Sas::fatal("Wrong number of rows in SRCLIST table-", "test3.fits");


    }

    //   compareData( ra3, ra4, 0.1 / 3600 );
    // compareData( dec3, dec4, 0.1 / 3600 );




    return 0;
}
/***********************************************************
*
*
*
*
**********************************************************/
void loadData(string file, vector<double> &ra, vector<double> &dec)
{
    DataSet *dataSet = dataSetServer->open( file, Dal::Read);

    if (!dataSet)
    {


    }    
 
    const Table *table = dataSet->table( "SRCLIST", Dal::Read);
    if( !table) {};

    /**********************************************************************
    *
    *
    **********************************************************************/
  
    int size = 0;

 
    if ( !LoadVectorDoubleColumnData( table, "RA_CORR", size, ra))
    {

    

    }

 
    if ( !LoadVectorDoubleColumnData( table, "DEC_CORR", size, dec))
    {

    

    }


   dataSetServer->close( dataSet );

}
/********************************************************
*
*
******************************************************/
bool LoadVectorFloatColumnData(const Table *table, 
                      const string& columnName, 
                      int &size, vector<float> &data)
{
     



  
  
    size = table->rows();
   
  
  
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        float *xData = columnData->real32Data();
       


	for (int i=0; i< size; i++) 
        {
	  data.push_back( xData[i] ); 
	 
	}
     }
         
   
    return true;
};


/********************************************************
*
*
******************************************************/
bool LoadVectorDoubleColumnData(const Table *table, 
                      const string& columnName, 
                      int &size, vector<double> &data)
{
     



  
  
    size = table->rows();
   
  
  
    if (table->hasColumn(columnName))
    { 
        const ColumnData *columnData = table->column(columnName)->data();
        if (!columnData) exit(0);
	

      
        double *xData = columnData->real64Data();
       


	for (int i=0; i< size; i++) 
        {
	  data.push_back( xData[i] ); 
	 
	}
     }
         
   
    return true;
};




/*****************************************************************
*
*
******************************************************************/


void checkVariabilityColumns( string file1, string file2)
{

    DataSet *set1 = dataSetServer->open( file1, Dal::Read);

 
 
    const Table *table1 = set1->table( "SRCLIST", Dal::Read);
  
  



  
    DataSet *set2 = dataSetServer->open( file2, Dal::Read);

 
 
    const Table *table2 = set2->table( "SRCLIST", Dal::Read);
  

    for(unsigned i = 0; i < table1->columns(); i++)
    {


        string name = table1->column(i)->name();
        size_t k = name.find("CHI");
 
        if( k == string::npos ) continue;
        
	if (!table2->hasColumn( name ) ) 
	{
           Sas::fatal("NoColumintable-", name);


        }
        else
	{
	    vector<float> data1;
            int size = 0;
            if ( !LoadVectorFloatColumnData( table1, name, size, data1))
            {

    

            }
            vector<float> data2;
            size = 0;
            if ( !LoadVectorFloatColumnData( table2, name, size, data2))
            {

    

            }
            float tolerance = 1e-6;

            cout << "checkoutput line 425 Comparing data from column " << name << endl;
            compareData(data1, data2, tolerance);




        }

	
    }


   dataSetServer->close( set1 );

   dataSetServer->close( set2 );

 

}
/*******************************************************************************
*
*
* Compute the chi-squared
*
*
*********************************************************************************/
float computeChiSquared(vector<float> &x, vector<float> &dx, float &theMean, float &prob)
{

    /**********************************************************
    * Compute the weighted mean
    ***********************************************************/

    double mean = (double)0.0;
    vector<double> weight(x.size() );
    double sumWeight = (double)0.0;

    for(unsigned i = 0; i < x.size(); i++)
    {
        double w = (double)1.0 / ( double(dx[i]) * double(dx[i]));
        weight[i] = w;
        mean += double(x[i]) * w;     
        sumWeight += w;
      
    }

    mean /= sumWeight;
    
    double chiSquared = (double)0.0;

    for(unsigned i = 0; i < x.size(); i++)
    {
        double diff = x[i] - mean;
        chiSquared += diff * diff * weight[i];

    }
    theMean = (float)mean;

 

    int ndf = x.size() - 1;
    prob = F90MODPROC( coords, chisqprobability)( &chiSquared, &ndf);


         
    return float( chiSquared / double(x.size() - 1) );
 

}
/*******************************************************
*
*
******************************************************/
void testChiSquared()
{


    float mean = 0.0, prob = 0.0;
    vector<float> x, dx;

    for(int i = 0; i < 100; i++)
    {
        x.push_back(1.0);
        dx.push_back(1.0);
    }
    float chiTrue = 0.0;
    float chiComputed = computeChiSquared(x, dx, mean, prob);

    if(abs(double(chiTrue - chiComputed)) > 1e-6)
    {
        cout << "chi true=" << chiTrue << " computed=" << chiComputed << endl;
        Sas::fatal("Wrong computed chi-squared value-", " ");


    }
    x.clear();
    dx.clear();


    for(int i = 0; i < 5; i++)
    {
        x.push_back(float(i + 1));
        dx.push_back(1.0);
    }
    chiTrue = 2.5;
    chiComputed = computeChiSquared(x, dx, mean, prob);

    if(abs(double(chiTrue - chiComputed)) > 1e-6)
    {
        cout << "chi true=" << chiTrue << " computed=" << chiComputed << endl;
        Sas::fatal("Wrong computed chi-squared value-", " ");


    }


}
